Package: cancensus
Type: Package
Title: Access, Retrieve, and Work with Canadian Census Data and
        Geography
Version: 0.6.1
Authors@R: c(
    person("Jens", "von Bergmann", email = "jens@mountainmath.ca", role = c("aut"), comment = "API creator and maintainer"),
    person("Dmitry", "Shkolnik", email = "shkolnikd@gmail.com", role = c("aut", "cre"), comment = "Package maintainer, responsible for correspondence"),
    person("Aaron", "Jacobs", email = "atheriel@gmail.com", role = "aut"))
Description: Integrated, convenient, and uniform access to Canadian
    Census data and geography retrieved using the 'CensusMapper' API. This package produces analysis-ready 
    tidy data frames and spatial data in multiple formats, as well as convenience functions
    for working with Census variables, variable hierarchies, and region selection. API
    keys are freely available with free registration at <https://censusmapper.ca/api>.
    Census data and boundary geometries are reproduced and distributed on an "as
    is" basis with the permission of Statistics Canada (Statistics Canada 1996; 2001; 2006;
    2011; 2016; 2021).
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: yes
ByteCompile: yes
NeedsCompilation: no
Imports: digest (>= 0.1), dplyr (>= 1.0), httr (>= 1.0.0), jsonlite (>=
        1.0), sf, geojsonsf, rlang, readr
Suggests: knitr, ggplot2, leaflet, mapdeck, rmarkdown, scales, sp,
        tidyr, lwgeom, xml2, testthat (>= 3.0.0), microbenchmark
VignetteBuilder: knitr
URL: https://github.com/mountainMath/cancensus,
        https://mountainmath.github.io/cancensus/,
        https://censusmapper.ca/api
BugReports: https://github.com/mountainMath/cancensus/issues
Depends: R (>= 4.1)
Config/roxygen2/version: 8.0.0
Packaged: 2026-06-17 23:11:34 UTC; jens
Author: Jens von Bergmann [aut] (API creator and maintainer),
  Dmitry Shkolnik [aut, cre] (Package maintainer, responsible for
    correspondence),
  Aaron Jacobs [aut]
Maintainer: Dmitry Shkolnik <shkolnikd@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-19 06:50:08 UTC
Built: R 4.5.2; ; 2026-06-19 10:34:29 UTC; unix

# Generated by roxygen2: do not edit by hand

export(add_unique_names_to_region_list)
export(as_census_region_list)
export(census_vectors)
export(child_census_vectors)
export(dataset_attribution)
export(explore_census_regions)
export(explore_census_vectors)
export(find_census_vectors)
export(get_census)
export(get_census_geometry)
export(get_intersecting_geometries)
export(get_statcan_geo_suite)
export(get_statcan_geographic_attributes)
export(get_statcan_geographies)
export(get_statcan_geography_relationships)
export(get_statcan_wds_data)
export(get_statcan_wds_metadata)
export(label_vectors)
export(list_cancensus_cache)
export(list_census_datasets)
export(list_census_regions)
export(list_census_vectors)
export(list_recalled_cached_data)
export(parent_census_vectors)
export(remove_from_cancensus_cache)
export(remove_recalled_cached_data)
export(search_census_regions)
export(search_census_vectors)
export(set_cancensus_api_key)
export(set_cancensus_cache_path)
export(show_cancensus_api_key)
export(show_cancensus_cache_path)
export(visualize_vector_hierarchy)
importFrom(dplyr,"%>%")
importFrom(rlang,.data)
importFrom(stats,setNames)

# cancensus

[![R-CMD-check](https://github.com/mountainMath/cancensus/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/mountainMath/cancensus/actions/workflows/R-CMD-check.yaml)
[![CRAN_Status_Badge](http://www.r-pkg.org/badges/version/cancensus)](https://cran.r-project.org/package=cancensus)
[![CRAN_Downloads_Badge](https://cranlogs.r-pkg.org/badges/cancensus)](https://cranlogs.r-pkg.org/badges/cancensus)
[![DOI](https://img.shields.io/badge/DOI-10.32614/CRAN.package.cancensus-d2b24a.svg)](https://doi.org/10.32614/CRAN.package.cancensus)

<a href="https://mountainmath.github.io/cancensus/index.html"><img id="readme-logo" src="https://raw.githubusercontent.com/mountainMath/cancensus/master/images/cancensus-sticker.png" alt="cancensus logo" align="right" width = "25%" height = "25%"/></a>


Access, retrieve, and work with Canadian Census data and geography. 

* Download data and Census geography in tidy and analysis-ready format
* Convenience tools for searching for and working with Census regions and variable hierarchies
* Provides Census geography in multiple R spatial formats
* Provides data and geography at multiple Census geographic levels including province, Census Metropolitan Area, Census Division, Census Subdivision, Census Tract, Dissemination Areas, Enumeration Areas (for 1996), and Dissemination Blocks (for 2001-2021)
* Provides data for the 2021, 2016, 2011, 2006, 2001, and 1996 Census releases
* Access to taxfiler data at the Census Tract level for tax years 2000 through 2018
 
### Reference

[**Cancensus home page and reference guide**](https://mountainmath.github.io/cancensus/index.html)

### Installing the package

```
install.packages("cancensus")
library(cancensus)
```

Alternatively, the latest development version can be installed from Github.
```
remotes::install_github("mountainmath/cancensus")
library(cancensus)
```

### API key

**cancensus** requires a valid CensusMapper API key to use. You can obtain a free API key by [signing up](https://censusmapper.ca/users/sign_up) for a CensusMapper account. To check your API key, just go to "Edit Profile" (in the top-right of the CensusMapper menu bar). Once you have your key, you can store it in your system environment so it is automatically used in API calls. To do so just enter `set_cancensus_api_key('<your_api_key>', install = TRUE)`.

CensusMapper API keys are free and public API quotas are generous; however, due to incremental costs of serving large quantities of data, there are some limits to API usage in place. For most use cases, these API limits should not be an issue. Production uses with large extracts of detailed geographies may run into API quota limits.

The new `get_intersecting_geometries` function has a separate API quota. This functionality puts higher demands on server resources than other API calls and therefore comes from a different bucket. By default user keys are capped to 500 region identifiers a day or 5000 per month. This _should_ suffice for most casual use cases, but we will continue to monitor the impact on server load and may increase the default limits in the future.

For larger quotas, please get in touch with Jens [directly](mailto:jens@mountainmath.ca).

### Local Cache

For performance reasons, and to avoid unnecessarily drawing down API quotas, **cancensus** caches data queries under the hood. By default, **cancensus** caches in R's temporary directory, but this cache is not persistent across sessions. In order to speed up performance, reduce quota usage, and reduce the need for unnecessary network calls, we recommend assigning a persistent local cache using `set_cancensus_cache_path('<local cache path>', install = TRUE)`, this enables more efficient loading and reuse of downloaded data. Users will be prompted with a suggestion to change their default cache location when making API calls if one has not been set yet. 

Starting with version 0.5.2 **cancensus** will automatically check if for data that has been recalled by Statistics Canada and is stored in the local cache via the new data recall API implemented in [CensusMapper](https://censusmapper.ca). Statistics Canada occasionally detects and corrects errors in their census data releases, and **cancensus** will download a list of recalled data at the first invocation of `get_census()` in each session and emit a warning if it detected locally cached data that has been recalled. Removal of the cached recalled data has to be done explicitly by the user via the `remove_recalled_chached_data()` function. If data was cached with **cancensus** versions prior to version 0.5.0 there is insufficient metadata to determine all instances of recalled cached data, but the package will check every time cached data is loaded and can identify recalled data at this point at the latest and issues a warning if recalled data is loaded.

### Currently available datasets

**cancensus** can access Statistics Canada Census data for Census years 1996, 2001, 2006, 2011, 2016, and 2021. You can run `list_census_datasets` to check what datasets are currently available for access through the CensusMapper API.

Thanks to contributions by the Canada Mortgage and Housing Corporation (CMHC), **cancensus** now includes additional Census-linked datasets as open-data releases. These include annual taxfiler data at the census tract level for tax years 2000 through 2018, which includes data on incomes and demographics, as well as specialized crosstabs for Structural type of dwelling by Document type, which details occupancy status for residences. These crosstabs are available for the 2001, 2006, 2011, 2016, and 2021 Census years at all levels starting with census tract.

### Picking regions and variables

Census data contains thousands of different geographic regions as well as thousands of unique variables. There are several useful functions within **cancensus** to simplify accessing Census metadata, locating regions, and identifying variables.

```
# To view available Census datasets
list_census_datasets()

# To view available named regions at different levels of Census hierarchy for the 2016 Census (for example)
list_census_regions("CA16")

# To view available Census variables for the 2016 Census
list_census_vectors("CA16")
```

There is also an interactive tool that is available at the [CensusMapper API](https://censusmapper.ca/api) to visually select regions and variables and generate code for the API call. Calling `explore_census_vectors(dataset = "CA16")` or `explore_census_regions(dataset = "CA16")` will open a new browser window to this interactive tool, preconfigured for whichever Census dataset is set as an argument. 

### Getting the data

**cancensus** can return census data with or without associated Census geographical information that can be used for mapping and GIS. By default, **cancensus** returns tidy tabular data only, but has options to return spatial data objects in either [**sf**](https://cran.r-project.org/package=sf) or [**sp**](https://cran.r-project.org/package=sp) formats. 
```
# Return data only
census_data <- get_census(dataset='CA16', regions=list(CMA="59933"), 
                          vectors=c("v_CA16_408","v_CA16_409","v_CA16_410"), level='CSD')

# Return an sf-class data frame
census_data <- get_census(dataset='CA16', regions=list(CMA="59933"), 
                          vectors=c("v_CA16_408","v_CA16_409","v_CA16_410"), level='CSD', geo_format = "sf")
```
**cancensus** attempts to minimize bandwidth usage and download time by caching downloads. When attempting to download data that has previously been downloaded,  **cancensus** will instead access the locally cached equivalent. 

### Examples of work using cancensus

* [Income: A first look](https://doodles.mountainmath.ca/blog/2017/09/14/income-a-first-look/)
* [Language Diversity in Canada](https://dshkol.com/posts/language-diversity-in-canada/)
* [Diversity and Segregation in Canadian cities](https://dshkol.com/posts/diversity-and-segregation-canadian-cities/)
* [Census tract level T1FF data](https://doodles.mountainmath.ca/blog/2020/04/23/census-tract-level-t1ff-tax-data/)

We'd love to feature examples of work or projects that use cancensus.

### Vignettes 

* [Getting started with cancensus](https://mountainmath.github.io/cancensus/articles/cancensus.html)
* [Making maps with cancensus](https://mountainmath.github.io/cancensus/articles/Making_maps_with_cancensus.html)
* [Data discovery: resources for finding available and relevant data](https://mountainmath.github.io/cancensus/articles/data_discovery.html)
* [Additional datasets: Structural type of dwelling by document type](https://mountainmath.github.io/cancensus/articles/Dwellings_by_document_type_cross_tabulation.html)
* [Additional datasets: Annual T1FF taxfiler data](https://mountainmath.github.io/cancensus/articles/Taxfiler_Data.html)


### Python port
A Python port of **cancensus** is available via the [pycancensus package](https://github.com/dshkol/pycancensus/).

### Related packages
The cancensus package is designed for working with Canadian Census data. In addition to Census data, Statistics Canada provides access to a vast [socio-economic data repository](https://www150.statcan.gc.ca/n1/en/type/data) with thousands of data tables available for public access. 

The [cansim package](https://mountainmath.github.io/cansim/index.html) is designed to retrieve and work with public Statistics Canada data tables. The cansim prepares retrieved data tables as analysis-ready tidy dataframes and provides a number of convenience tools and functions to make it easier to work with Statistics Canada data. It is available on CRAN and on [Github](https://github.com/mountainMath/cancensus). Data retrieved via the cansim package can be linked to census data via the `GeoUID` field.

The [tongfen package](https://mountainmath.github.io/tongfen/index.html) automates the task of obtaining census variables from different census years on a common geography. It is available on [Github](https://github.com/mountainMath/tongfen).

The [**cmhc**](https://mountainmath.github.io/cmhc/index.html) package, which accesses CMHC data on the housing in Canada and can be linked to census geographies via the `GeoUID`.

Taken together these packages offer a seamless view into important Canadian data.

### International census data
There are several other jurisdiction where census data is available via R packages, including

* US: [tidycensus](https://walker-data.com/tidycensus/) and [tigris](https://github.com/walkerke/tigris)
* Brasil: [geobr](https://github.com/ipeaGIT/geobr)
* Africa: [afrimapr](https://afrimapr.github.io/afrimapr.website/)
* Brazil: [geobr](https://ipeagit.github.io/geobr/)
* Chile: [chilemapas](https://pacha.dev/chilemapas/)
* Czech Republic: [RCzechia](https://github.com/jlacko/RCzechia)
* Finland: [geofi](https://ropengov.github.io/geofi/)
* Ghana: [rGhanaCensus](https://CRAN.R-project.org/package=rGhanaCensus)
* Spain: [mapSpain](https://github.com/rOpenSpain/mapSpain/)
* UK: [geographr](https://github.com/humaniverse/geographr)
* Uruguay: [geouy](https://github.com/RichDeto/geouy)
* Global (political administrative boundaries): [rgeoboundaries](https://github.com/wmgeolab/rgeoboundaries)


### Contributing

* We encourage contributions to improve this project. The best way is through issues and pull requests.
* If you want to get in touch, we are pretty good at responding via email or via x/bluesky at [@dshkol](https://x.com/dshkol)/[@dshkol.bsky.social](https://bsky.app/profile/dshkol.bsky.social) or [@vb_jens](https://x.com/vb_jens)/[@jensvb.bsky.social](https://bsky.app/profile/jensvb.bsky.social).

### Cite **cancensus**

If you wish to cite cancensus:

  von Bergmann, J., Aaron Jacobs, Dmitry Shkolnik (2025). cancensus: R package to
  access, retrieve, and work with Canadian Census data and geography. v0.6.0.


A BibTeX entry for LaTeX users is
```
  @Manual{cancensus,
    author = {Jens {von Bergmann} and Dmitry Shkolnik and Aaron Jacobs},
    title = {cancensus: R package to access, retrieve, and work with Canadian Census data and geography},
    year = {2025},
    doi = {10.32614/CRAN.package.cancensus},
    note = {R package version 0.6.0},
    url = {https://mountainmath.github.io/cancensus/}
  }
```


### Statistics Canada Attribution

Subject to the Statistics Canada Open Data License Agreement, licensed products using Statistics Canada data should employ the following acknowledgement of source:

```
Acknowledgment of Source

(a) You shall include and maintain the following notice on all licensed rights of the Information:

  - Source: Statistics Canada, name of product, reference date. Reproduced and distributed on an "as is" basis with the permission of Statistics Canada.
 
(b) Where any Information is contained within a Value-added Product, you shall include on such Value-added Product the following notice:

  - Adapted from Statistics Canada, name of product, reference date. This does not constitute an endorsement by Statistics Canada of this product.
```

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)

## ----setup--------------------------------------------------------------------
# Packages used for example
library(cancensus)
library(dplyr)
library(tidyr)
library(ggplot2)
library(sf)

## -----------------------------------------------------------------------------
list_census_datasets() %>% 
  filter(grepl("taxfiler",description))

## -----------------------------------------------------------------------------
list_census_vectors('TX2017')

## -----------------------------------------------------------------------------
years <- c(2006,2011,2014,2018)
# Attribution for the dataset to be used in graphs
attribution <- dataset_attribution(paste0("TX",years))

plot_data <- years %>%
  lapply(function(year) {
    dataset <- paste0("TX",year)
    vectors <- c("Families"=paste0("v_",dataset,"_607"),
                 "CFLIM-AT"=paste0("v_",dataset,"_786"))
    
    get_census(dataset,regions=list(CMA="59933"),vectors = vectors,
                    geo_format = 'sf', level="CT", quiet = TRUE) %>%
      select(c("GeoUID",names(vectors))) %>%
      mutate(Year=year)
  }) %>%
  bind_rows() %>%
  mutate(share=`CFLIM-AT`/Families)

## -----------------------------------------------------------------------------
ggplot(plot_data,aes(fill=share)) +
  geom_sf(size=0.1,color="white") +
  facet_wrap("Year") +
  scale_fill_viridis_c(labels=scales::percent,option = "inferno",
                       trans="log",breaks = c(0.05,0.1,0.2,0.4)) +
  coord_sf(datum=NA,xlim=c(-123.4, -122.5), ylim=c(49.01, 49.4)) +
  labs(title="Share of census families in low income",fill="Share",
       caption=attribution)

## -----------------------------------------------------------------------------
change_data <- plot_data %>% 
  filter(Year==2006) %>% 
  select(GeoUID,`2006`=share) %>%
  left_join(plot_data %>%
              st_set_geometry(NULL) %>%
              filter(Year==2011) %>% 
              select(GeoUID,`2011`=share),
            by="GeoUID") %>%
  mutate(change=`2011`-`2006`)
  
ggplot(change_data,aes(fill=change)) +
  geom_sf(size=0.1) +
  scale_fill_gradient2(labels=scales::percent) +
  #scale_fill_viridis_c(labels=scales::percent,option = "inferno") +
  coord_sf(datum=NA,xlim=c(-123.4, -122.5), ylim=c(49.01, 49.4)) +
  labs(title="Change in share of census families in low income 2006-2011",fill="Percentage\npoint change",caption=dataset_attribution(paste0("TX",c(2006,2011))))

## ----setup, echo=FALSE, message=FALSE, warning=FALSE--------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)
library(cancensus)
library(dplyr)
# set_cancensus_api_key(<your_api_key>', install = TRUE)
# set_cancensus_cache_path(<local cache path>, install = TRUE)

## ----load_package_cran, echo=TRUE, message=FALSE, warning=FALSE, eval = FALSE----
#  install.packages("cancensus")
#  
#  library(cancensus)

## ----load_package_git, echo=TRUE, message=FALSE, warning=FALSE, eval = FALSE----
#  # install.packages("devtools")
#  remotes::install_github("mountainmath/cancensus")
#  
#  library(cancensus)
#  
#  options(cancensus.api_key = "your_api_key")
#  options(cancensus.cache_path = "custom cache path")

## ----install_api_key_and_cache_path, echo=TRUE, message=FALSE, warning=FALSE, eval = FALSE----
#  # Only need to install api key can cache path once
#  set_cancensus_api_key('<your_api_key>', install = TRUE)
#  set_cancensus_cache_path('<local cache path>', install = TRUE)

## ----get_census example, echo=TRUE, warning=FALSE, message=FALSE--------------
# Returns a data frame with data only
census_data <- get_census(dataset='CA21', regions=list(CMA="59933"),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, geo_format = NA, quiet = TRUE)

# Returns data and geography as an sf-class data frame
census_data <- get_census(dataset='CA21', regions=list(CMA="59933"),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, geo_format = 'sf', quiet = TRUE)

# Returns a SpatialPolygonsDataFrame object with data and geography
census_data <- get_census(dataset='CA21', regions=list(CMA="59933"),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, geo_format = 'sp', quiet = TRUE)

## ----list datasets, message=FALSE, warning=FALSE------------------------------
list_census_datasets()

## ----list regions, message=FALSE, warning=FALSE-------------------------------
list_census_regions("CA21")

## ----message=FALSE, warning=FALSE---------------------------------------------
# Retrieves Vancouver and Toronto
list_census_regions('CA21') %>% 
  filter(level == "CMA", name %in% c("Vancouver","Toronto"))

census_data <- get_census(dataset='CA21', regions=list(CMA=c("59933","35535")),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, quiet = TRUE)

## ----list_vectors, message=FALSE, warning=FALSE-------------------------------
list_census_vectors("CA21")

## ----search_vectors1, message=FALSE, warning=FALSE----------------------------
# Find the variable indicating the number of people of Austrian ethnic origin
find_census_vectors("Australia", dataset = "CA16", type = "total", query_type = "exact")

find_census_vectors("Australia origin", dataset = "CA16", type = "total", query_type = "semantic")

find_census_vectors("Australian ethnic", dataset = "CA16", type = "total", query_type = "keyword", interactive = FALSE)

## ----parent_vectors, message=FALSE, warning=FALSE-----------------------------
list_census_vectors("CA16") %>% 
  filter(vector == "v_CA16_4092") %>% 
  parent_census_vectors()

## ----child_vectors1, message=FALSE, warning=FALSE-----------------------------
# Find the variable indicating the Northern European aggregate
find_census_vectors("Northern European", dataset = "CA16", type = "Total")

## ----child_vectors2, message=FALSE, warning=FALSE-----------------------------
# Show all child variable leaves
list_census_vectors("CA16") %>% 
  filter(vector == "v_CA16_4122") %>% child_census_vectors(leaves = TRUE)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE, 
  warning = FALSE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)

## ----setup, message = FALSE, warning = FALSE----------------------------------
# Packages used in this example
library(cancensus)
library(dplyr)
library(tidyr)
library(ggplot2)

## -----------------------------------------------------------------------------
# Attribution for the dataset to be used in graphs
attribution <- dataset_attribution("CA16xSD")

# Select all variables base variables, this gives us total counts by structural type of dwelling
vars <- list_census_vectors("CA16xSD") %>% 
  filter(is.na(parent_vector))
variables <- setNames(vars$vector,vars$label)

variables

## -----------------------------------------------------------------------------
# Separate out the individual dwelling types
dwelling_types <- setdiff(names(variables),"Total dwellings")

# Grab the census data and compute shares for each dwelling type
census_data <- get_census("CA16xSD",regions=list(CSD="3520005"), vectors = variables, quiet = TRUE) %>%
  pivot_longer(cols = all_of(dwelling_types)) %>%
  mutate(share=value/`Total dwellings`)

## ----fig.alt="Toronto dwelling units by structural type"----------------------
ggplot(census_data,aes(x=reorder(name,share),y=share)) +
  geom_bar(stat="identity",fill="steelblue") +
  coord_flip() +
  scale_y_continuous(labels=scales::percent) +
  labs(title="City of Toronto dwelling units by structural type",
       x=NULL,y=NULL,caption=attribution)

## ----fig.alt="Toronto dwellings unoccupied on census day"---------------------
# Use explore_census_vectors() to browse and select variables of interest
vars <- c(Total="v_CA16xSD_1", Unoccupied="v_CA16xSD_28")

# Retrieve data with attached geography
census_data <- get_census("CA16xSD",regions=list(CSD="3520005"), level="CT", quiet = TRUE, geo_format = "sf",
                          vectors = vars,use_cache = FALSE) %>%
  mutate(share=Unoccupied/Total)

# Visualize
ggplot(census_data,aes(fill=share)) +
  geom_sf(size=0.1) +
  scale_fill_viridis_c(labels=scales::percent) +
  coord_sf(datum=NA) +
  labs(title="City of Toronto dwellings unoccupied on census day",
       fill="Share",
       x=NULL,y=NULL,caption=attribution)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE, 
  warning = FALSE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)

## ----setup--------------------------------------------------------------------
library(cancensus)
library(dplyr)
library(ggplot2)
library(sf)

## -----------------------------------------------------------------------------
cov_station_buffers <- COV_SKYTRAIN_STATIONS %>%
  st_set_crs(4326)  # needed for Ubuntu or systems with old GDAL but can otherwise be ignored

## -----------------------------------------------------------------------------
station_city_ids <- get_intersecting_geometries("CA16", level = "CSD", geometry = cov_station_buffers,
                                                quiet=TRUE)
station_ct_ids <- get_intersecting_geometries("CA16", level = "CT", geometry = cov_station_buffers,
                                              quiet=TRUE)

## -----------------------------------------------------------------------------
variables <- c(mode_base="v_CA16_5792",transit="v_CA16_5801",walk="v_CA16_5804")
station_city <-  get_census("CA16", regions = station_city_ids, vectors = variables, 
                            geo_format = 'sf', quiet=TRUE) %>% 
  filter(name == "Vancouver (CY)")
station_cts <-   get_census("CA16", regions = station_ct_ids, vectors = variables, 
                            geo_format = 'sf', quiet=TRUE)

## -----------------------------------------------------------------------------
ggplot(station_city) +
  geom_sf(fill=NA) +
  geom_sf(data=station_cts,aes(fill=((walk+transit)/mode_base))) +
  geom_sf(data=cov_station_buffers,fill=NA,alpha=0.5,color="steelblue") +
  scale_fill_viridis_c(option="magma",labels=scales::percent) +
  coord_sf(datum=NA) +
  labs(title="CTs within City of Vancouver skytrain station catchments",
       fill="Walk or\ntransit\nto work",
       caption="StatCan Census 2016")

## -----------------------------------------------------------------------------
station_das <-   get_intersecting_geometries("CA16", level = "DA", geometry = cov_station_buffers,
                                             quiet=TRUE) %>% 
  get_census("CA16", regions = ., vectors=variables, geo_format = 'sf', quiet=TRUE)

ggplot(station_city) +
  geom_sf(fill=NA) +
  geom_sf(data=station_das,aes(fill=((walk+transit)/mode_base))) +
  geom_sf(data=cov_station_buffers,fill=NA,alpha=0.5,color="steelblue") +
  scale_fill_viridis_c(option="magma",labels=scales::percent) +
  coord_sf(datum=NA) +
  labs(title="DAs within City of Vancouver skytrain station catchments",
       fill="Walk or\ntransit\nto work",
       caption="StatCan Census 2016")


## -----------------------------------------------------------------------------
station_das2 <- get_census("CA16", regions = station_ct_ids, vectors=variables, 
                           geo_format = 'sf', level="DA", quiet=TRUE) %>%
  sf::st_filter(cov_station_buffers)

ggplot(station_city) +
  geom_sf(fill=NA) +
  geom_sf(data=station_das2,aes(fill=((walk+transit)/mode_base))) +
  geom_sf(data=cov_station_buffers,fill=NA,alpha=0.5,color="steelblue") +
  scale_fill_viridis_c(option="magma",labels=scales::percent) +
  coord_sf(datum=NA) +
  labs(title="DAs within City of Vancouver skytrain station catchments",
       fill="Walk or\ntransit\nto work",
       caption="StatCan Census 2016")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE, 
  warning = FALSE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)

## ----setup--------------------------------------------------------------------
library(cancensus)
library(dplyr)
library(tidyr)
library(ggplot2)

## -----------------------------------------------------------------------------
fed_version = "1.3"

## -----------------------------------------------------------------------------
metadata <- get_statcan_wds_metadata("2021","FED",version=fed_version)

characteristics <- metadata |> 
  filter(`Codelist en`=="Characteristic") |>
  mutate(across(matches("ID"),as.integer))

ethnic_base <- characteristics |> 
  filter(grepl("Total - Ethnic",en))
ukranian <- characteristics |> 
  filter(grepl("Ukrainian",en), `Parent ID`==ethnic_base$ID) 

selected_characteristics <- bind_rows(ethnic_base,ukranian)

selected_characteristics |> select(ID,en)

## -----------------------------------------------------------------------------
dguids <- metadata |>
  filter(`Codelist ID`=="CL_GEO_FED") |>
  pull(ID)

## -----------------------------------------------------------------------------
data <- get_statcan_wds_data(dguids,members=selected_characteristics$ID,
                             gender="Total",version=fed_version)

## -----------------------------------------------------------------------------
plot_data <- data |> 
  select(DGUID=GEO_DESC,Name=GEO_NAME,name=CHARACTERISTIC_NAME,value=OBS_VALUE) |>
  pivot_wider() |>
  mutate(Share=Ukrainian/`Total - Ethnic or cultural origin for the population in private households - 25% sample data`)

## ----fig.height=4.5, fig.width=6----------------------------------------------
plot_data |> slice_max(Share,n=20) |>
  ggplot(aes(y=reorder(Name,Share),x=Share)) +
  geom_bar(stat="identity",fill="steelblue") +
  scale_x_continuous(labels=scales::percent) +
  labs(title="Ukrainian ethnic origin",
       y="Federal electoral district (2013 Represenation Order)",
       x="Share of population in private households",
       caption="StatCan Census 2021")

## -----------------------------------------------------------------------------
fed_geos <- get_statcan_geographies("2021","FED")

## ----fig.height=4.5, fig.width=6----------------------------------------------
fed_geos |>
  left_join(plot_data,by="DGUID") |>
  ggplot(aes(fill=Share)) +
  geom_sf() +
  scale_fill_viridis_c(labels=scales::percent) +
  coord_sf(datum=NA) +
  labs(title="Ukrainian ethnic origin by (2013) Federal Electoral District",
       fill="Share of\npopulation",
       caption="StatCan Census 2021")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE, 
  warning = FALSE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)

## ----setup--------------------------------------------------------------------
library(cancensus)
library(dplyr)

## -----------------------------------------------------------------------------
attributes <- get_statcan_geographic_attributes("2021")

attributes %>% colnames()

## -----------------------------------------------------------------------------
attributes %>%
  filter(CMATYPE_RMRGENRE %in% c("B","K")) |> # filter areas not in CTs
  group_by(CTCODE_SRCODE,CMATYPE_RMRGENRE) |>
  summarise(`Number of municipalities`=length(unique(CSDUID_SDRIDU)),.groups="drop") |>
  count(`Number of municipalities`,CMATYPE_RMRGENRE) |>
  arrange(CMATYPE_RMRGENRE,`Number of municipalities`)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)

mb_token <- Sys.getenv("MB_TOKEN")

## ----message = FALSE, warning = FALSE-----------------------------------------
library(cancensus)
library(sf)
# retrieve sf dataframe
toronto <- get_census(dataset='CA21', regions=list(CMA="35535"),
                         vectors=c("median_hh_income"="v_CA21_906"), level='CSD', quiet = TRUE, 
                         geo_format = 'sf', labels = 'short')

## -----------------------------------------------------------------------------
plot(toronto["median_hh_income"], main = "Toronto Household Income by CSD in 2020")

## -----------------------------------------------------------------------------
plot(st_geometry(toronto), col = NA, main = "Toronto CSDs with Median HH Income > $125,000 in 2020", lty = 3)
plot(st_geometry(toronto[toronto$median_hh_income > 125000,]), col = "red", add = TRUE)

## -----------------------------------------------------------------------------
library(ggplot2)
ggplot(toronto) + geom_sf(aes(fill = median_hh_income))

## -----------------------------------------------------------------------------
ggplot(toronto) + geom_sf(aes(fill = median_hh_income), colour = "grey") +
  scale_fill_viridis_c("Median HH Income", labels = scales::dollar) + theme_minimal() +
  theme(panel.grid = element_blank(),
        axis.text = element_blank(),
        axis.ticks = element_blank()) + 
  coord_sf(datum=NA) +
  labs(title = "Median Household Income in 2020", subtitle = "Toronto Census Subdivisions, 2021 Census")

## ----message=FALSE, warning=FALSE, include=FALSE------------------------------
# This chunk disables leaflet and mapdeck output in remaining chunks for PDF vignettes
if(Sys.getenv("COMPILE_VIG")!="TRUE") {
  knitr::opts_chunk$set(eval = FALSE)}

## ----message = FALSE----------------------------------------------------------
library(leaflet)
leaflet(toronto) %>% 
  addProviderTiles(providers$CartoDB.Positron) %>%
  addPolygons()

## -----------------------------------------------------------------------------
bins <- c(0, 30000,40000, 50000,60000, 70000,80000, 90000,100000, 110000, Inf)
pal <- colorBin("RdYlBu", domain = toronto$v_CA16_2397, bins = bins)
leaflet(toronto) %>% 
  addProviderTiles(providers$CartoDB.Positron) %>%
  addPolygons(fillColor = ~pal(median_hh_income),
              color = "white",
              weight = 1,
              opacity = 1,
              fillOpacity = 0.65)

#' Query the CensusMapper API for available regions in a given dataset.
#'
#' @param dataset The dataset to query for available regions, e.g.
#'   \code{"CA16"}.
#' @param use_cache If set to TRUE (the default), data will be read from a local cache
#'   that is maintained for the duration of the R session, if
#'   available. If set to FALSE, query the API for the data, and
#'   refresh the local cache with the result.
#' @param quiet When TRUE, suppress messages and warnings.
#'
#' @return
#'
#' Returns a data frame with the following columns:
#'
#' \describe{
#'   \item{\code{region}}{The region identifier.}
#'
#'   \item{\code{name}}{The name of that region.}
#'
#'   \item{\code{level}}{The census aggregation level of that region.}
#'
#'   \item{\code{pop}}{The population of that region.}
#'
#'   \item{\code{municipal_status}}{Additional identifiers to distinguish the
#'     municipal status of census subdivisions.}
#'
#'   \item{\code{CMA_UID}}{The identifier for the Census Metropolitan Area the region is in (if any).}
#'
#'   \item{\code{CD_UID}}{The identifier for the Census District the region is in (if any).}
#'
#'   \item{\code{PR_UID}}{The identifier for the Province the region is in (if unique).}
#' }
#'
#' @export
#'
#' @examples
#' \dontrun{
#' list_census_regions('CA16')
#' }
list_census_regions <- function(dataset, use_cache = TRUE, quiet = FALSE) {
  dataset <- translate_dataset(dataset)
  cache_file <- file.path(tempdir(),paste0(dataset, "_regions.rda"))

  if (!use_cache || !file.exists(cache_file)) {
    if (!quiet) message("Querying CensusMapper API for regions data...")
    response <- httr::GET(paste0("https://censusmapper.ca/data_sets/", dataset,
                                 "/place_names.csv"))
    handle_cm_status_code(response, NULL)
    content <- httr::content(response, type = "text", encoding = "UTF-8")
    result <- if (!requireNamespace("readr", quietly = TRUE)) {
      dplyr::as_tibble(utils::read.csv(textConnection(content),
                                       colClasses = 'character',
                                       stringsAsFactors = FALSE),
                       .name_repair = "minimal")
    } else {
      readr::read_csv(content,col_types = readr::cols(.default='c'))
    }
    result <- dplyr::select(result, region = .data$geo_uid, .data$name,
                            level = .data$type, pop = .data$population,
                            municipal_status = .data$flag, .data$CMA_UID,
                            .data$CD_UID, .data$PR_UID)  %>%
      dplyr::mutate(pop=as.integer(.data$pop))
    attr(result, "last_updated") <- Sys.time()
    save(result, file = cache_file)
    result$level[result$level=="CMA"&result$municipal_status == "K"] <- "CA"
    result
  } else {
    if (!quiet) message("Reading regions list from local cache.")
    load(file = cache_file)
    last_updated <- attr(result, "last_updated")
    if (!quiet && is.null(last_updated) ||
        difftime(Sys.time(), last_updated, units = "days") > 1) {
      warning(paste("Cached regions list may be out of date. Set `use_cache =",
                    "FALSE` to update it."))
    }
    result$level[result$level=="CMA"&result$municipal_status == "K"] <- "CA"
    result
  }
}

#' Query the CensusMapper API for regions with names matching a searchterm.
#'
#' @description Runs a query against the CensusMapper API to retrieve region data with
#' names matching specific queries. Users can optionally specify the target geography level
#' (e.g. \code{level = 'CMA'}, \code{level = 'CSD'}, etc.). Alternatively, calling
#' \code{explore_census_vectors()} will launch the interactive region selection tool on
#' the CensusMapper site in a new web page or tab.
#'
#' @param searchterm The term to search for e.g. \code{"Victoria"}.
#' Search terms are case insensitive. If unable to find a given search term,
#' this function will suggest the correct spelling to use when possible.
#' @param dataset The dataset to query for available regions, e.g.
#'   \code{"CA16"}.
#' @param level One of \code{NA}, \code{'C'}, \code{'PR'}, \code{'CMA'}, \code{'CD'}, or \code{'CSD'}.
#' If specified, only return variables of specified `level`.
#' @param ... Further arguments passed on to \code{\link{list_census_regions}}.
#' @return A census region list of the same format as `list_census_regions()` containing the matches.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' # This will return a warning that no match was found, but will suggest similar named regions.
#' search_census_regions('Victorea', 'CA16')
#'
#' # This will limit region results to only include CMA level regions
#' search_census_regions('Victoria', 'CA16', level = "CMA")
#' }
search_census_regions <- function(searchterm, dataset, level=NA, ...) {
  reglist <- list_census_regions(dataset, ...)
  result <- reglist[grep(searchterm, reglist$name, ignore.case = TRUE),]

  # filter by type if needed
  if (!is.na(level) && length(rownames(result)) > 0) {
    result <- result[result$level==level,]
  }

  # Check if searchterm returned anything
  if (length(rownames(result)) > 0 ) {
    attr(result, "dataset") <- dataset
    return(result)
  }
  # If nothing matches, throw a warning and suggested alternatives.
  # If no suggested alternatives because the typo is too egregious, throw an error.
  else {
    # Check for similarly named terms. Uses base function agrep which is based on the Levenshtein edit distance for string similarity.
    # Default is set to 0.1 - can expand this to be more tolerant still.
    hintlist <- dplyr::as_tibble(unique(agrep(searchterm, reglist$name, ignore.case = TRUE, value = TRUE)),
                                 .name_repair = "minimal")
    names(hintlist) <- "Similar named regions"
    #
    if (length(hintlist) > 0) {
      warning("No results found. Please use accurate spelling. See above for list of similarly named regions.")
      print(hintlist)
    } else {
      stop("No results found.")
    }
  }
}

#' Convert a (suitably filtered) data frame from
#' \code{\link{list_census_regions}} to a list suitable for passing to
#' \code{\link{get_census}}.
#'
#' @param tbl A data frame, suitably filtered, as returned by
#'   \code{\link{list_census_regions}}.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' library(dplyr, warn.conflicts = FALSE)
#'
#' # Query the CensusMapper API for the total occupied dwellings
#' # of 20 random Census Subdivisions, in Census 2016.
#' regions <- list_census_regions("CA16") %>%
#'   filter(level == "CSD") %>%
#'   sample_n(20) %>%
#'   as_census_region_list()
#'
#' occupied <- get_census("CA16", regions = regions,
#'                             vectors = c("v_CA16_408"),
#'                             level = "Regions")
#' }
as_census_region_list <- function(tbl) {
  # This isn't bulletproof validation, but it should deter some misuse.
  if (!all(c("level", "region") %in% names(tbl))) {
    stop(paste("`as_region_list()` can only handle data frames",
               "returned by `list_regions()`."))
  }
  nested <- dplyr::group_by(tbl, .data$level) %>%
    # Use the dark magic of list columns...
    dplyr::summarise(regions = list(.data$region))

  regions <- nested$regions
  names(regions) <- nested$level
  regions
}

#' Convenience function for creating unique names from region list
#'
#' @description Names of municipalities are not always unique, especially at the CSD level. This function
#' takes as input a subset of a regions list as generated from `list_census_regions()` and de-duplicates names as
#' needed by adding the municipal status in parenthesis. If this does not de-duplicate the name then the
#' geographic identifier will be further added in parenthesis behind that.
#'
#' @param region_list a subset of a regions list as gotten from `list_census_regions()`
#' @return The same list of regions with an extra column `Name` with de-duplicated names.
#' @export
#'
#' @examples
#' \dontrun{
#' # This will return a warning that no match was found, but will suggest similar named regions.
#' library(dplyr)
#' list_census_regions("CA21") %>%
#'   filter(level=="CSD", CMA_UID=="59933") %>%
#'   add_unique_names_to_region_list()
#' }
add_unique_names_to_region_list <- function(region_list) {
  gs <- dplyr::groups(region_list)
  r<-region_list %>%
    dplyr::group_by(.data$name) %>%
    dplyr::mutate(count=dplyr::n()) %>%
    dplyr::mutate(Name=dplyr::case_when(.data$count==1 ~ name,
                                        TRUE ~ paste0(.data$name," (",.data$municipal_status,")"))) %>%
    dplyr::group_by(.data$Name) %>%
    dplyr::mutate(count=dplyr::n()) %>%
    dplyr::mutate(Name=dplyr::case_when(.data$count==1 ~ Name,
                                        TRUE ~ paste0(.data$Name," (",.data$region,")"))) %>%
    dplyr::select(-.data$count) %>%
    dplyr::ungroup()

  if (length(gs)>1) {
    r <- r %>%
      dplyr::group_by(dplyr::across(dplyr::all_of(gs)))
  }
  r
}


#' Lookup a municipal geography type from code - BETA
#'
#' @description Retrieved Census divisions and subdivisions include a code indicating the
#' municipality type or municipal status. There are 12 CD and 53 CSD distinct status
#' codes based on official designations used by provinces, territories, and federal
#' authorities. These are often used to distinguish Census divisions and subdivisions
#' with similar or identical names. Using \code{muni_status(code)} provides
#' some additional context in understanding the specific type referenced by the code.
#' For additional information consult the latest data dictionaries for relevant geographic levels.
#' Information for CSDs is at
#' \url{https://www12.statcan.gc.ca/census-recensement/2016/ref/dict/geo012-eng.cfm} and
#' for CDs at \url{https://www12.statcan.gc.ca/census-recensement/2016/ref/dict/geo008-eng.cfm}.
#'
#' @param code code/municipality status.
#'
#'
#'
#' @examples
#' muni_status("C")
#' muni_status("RD")
#' muni_status("CT")
#' muni_status("CV")
#'
# muni_status = function(code) {
#   geography <- c("CD", "CD", "CD", "CD", "CD", "CD", "CD", "CD", "CD", "CD", "CD",
#                  "CD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD",
#                  "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD",
#                  "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD",
#                  "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD",
#                  "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD", "CSD",
#                  "CSD", "CSD", "CSD", "CSD")
#   status_code <- c("CDR", "CT", "CTY", "DIS", "DM", "MRC", "RD", "REG", "RM", "TÉ", "TER",
#             "UC", "C", "CC", "CG", "CN", "COM", "CT", "CU", "CV", "CY", "DM", "HAM",
#             "ID", "IGD", "IM", "IRI", "LGD", "LOT", "M", "MD", "MÉ", "MU", "NH", "NL",
#             "NO", "NV", "P", "PE", "RCR", "RDA", "RGM", "RM", "RV", "S-É", "SA", "SC",
#             "SÉ", "SET", "SG", "SM", "SNO", "SV", "T", "TC", "TI", "TK", "TL", "TP",
#             "TV", "V", "VC", "VK", "VL", "VN")
#   status <- c("Census division / Division de recensement", "County / Comté", "County",
#               "District", "District municipality", "Municipalité régionale de comté",
#               "Regional district", "Region", "Regional municipality", "Territoire équivalent",
#               "Territory / Territoire", "United counties", "City / Cité", "Chartered community",
#               "Community government", "Crown colony / Colonie de la couronne", "Community",
#               "Canton (municipalité de)", "Cantons unis (municipalité de)", "City / Ville",
#               "City", "District municipality", "Hamlet", "Improvement district",
#               "Indian government district", "Island municipality",
#               "Indian reserve / Réserve indienne", "Local government district",
#               "Township and royalty", "Municipality / Municipalité", "Municipal district",
#               "Municipalité", "Municipality", "Northern hamlet", "Nisga'a land",
#               "Unorganized / Non organisé", "Northern village",
#               "Parish / Paroisse (municipalité de)", "Paroisse (municipalité de)",
#               "Rural community / Communauté rurale", "Regional district electoral area",
#               "Regional municipality", "Rural municipality", "Resort village",
#               "Indian settlement / Établissement indien", "Special area",
#               "Subdivision of county municipality / Subdivision municipalité de comté",
#               "Settlement / Établissement", "Settlement",
#               "Self-government / Autonomie gouvernementale", "Specialized municipality",
#               "Subdivision of unorganized / Subdivision non organisée", "Summer village",
#               "Town", "Terres réservées aux Cris", "Terre inuite",
#               "Terres réservées aux Naskapis", "Teslin land", "Township", "Town / Ville",
#               "Ville", "Village cri", "Village naskapi", "Village", "Village nordique")
#   muni_types <- data.frame(geography = geography, status_code = status_code, status = status)
#   result <- muni_types[muni_types$status_code == code,]
#   return(result)
#   if(nrow(result)>0) result else {
#     stop(paste0('There is no Census division or subdivision or municipality status with code "',
#                 code,'"'),
#          call. = FALSE)
#   }
# }

# Functions for managing user settings, API keys, cache locations

#' Set Censusmapper API key
#'
#' @description Cancensus requires a free Censusmapper API key to retrieve data. This function helps set the key for either the duration of the session (default) or permanently for use across sessions.
#'
#' @param key a Censusmapper API key. For more information on keys see the \href{https://mountainmath.github.io/cancensus/index.html#api-key}{API key section}
#' @param overwrite Option to overwrite any existing Censusmapper keys already stored locally.
#' @param install Option to install permanently for use across sessions.
#'
#' @export
#'
#' @examples
#'\dontrun{
#' set_cancensus_api_key("YOUR_CM_API_KEY")
#'
#' # This will set the key permanently until overwritten again
#' set_cancensus_api_key("YOUR_CM_API_KEY", install = TRUE)
#' }
set_cancensus_api_key <- function(key, overwrite = FALSE, install = FALSE){
  if (install) {
    home <- Sys.getenv("HOME")
    renv <- file.path(home, ".Renviron")
    if(!file.exists(renv)){
      file.create(renv)
    } else{
      # Backup original .Renviron before doing anything else here.
      file.copy(renv, file.path(home, ".Renviron_backup"))
      if(isTRUE(overwrite)){
        message("Adding key to your .Renviron file. Your original .Renviron will be backed up and stored in your R HOME directory if needed.")
        oldenv=readLines(renv)
        newenv <- oldenv[-grep("CM_API_KEY", oldenv)]
        writeLines(newenv, renv, sep = "\n")
      }
      else{
        tv <- readLines(renv)
        if(any(grepl("CM_API_KEY",tv))){
          stop("An existing Censusmapper API key is already saved. You can overwrite it with the argument overwrite=TRUE.", call.=FALSE)
        }
      }
    }

    keyconcat <- paste0("CM_API_KEY='", key, "'")
    # Append API key to .Renviron file
    write(keyconcat, renv, sep = "\n", append = TRUE)
    Sys.setenv(CM_API_KEY = key)
    message('Your API key has been stored in your .Renviron and can be accessed by Sys.getenv("CM_API_KEY").')
  } else {
    message("API key set for duration of session. To install your API key for use across sessions, run this function with `install = TRUE`.")
    Sys.setenv(CM_API_KEY = key)
  }

}

#' Set persistent cancensus cache location
#'
#' @description Cancensus provides session caching for retrieved data to increase speeds and reduce API usage. This function will create a persistent cache across sessions.
#'
#' @param cache_path a local directory to use for saving cached data
#' @param overwrite Option to overwrite any existing cache path already stored locally.
#' @param install Option to install permanently for use across sessions.
#'
#' @export
#'
#' @examples
#'\dontrun{
#' set_cancensus_cache_path("~/cancensus_cache")
#'
#' # This will set the cache path permanently until overwritten again
#' set_cancensus_cache_path("~/cancensus_cache", install = TRUE)
#' }
set_cancensus_cache_path <- function(cache_path, overwrite = FALSE, install = FALSE){
  if (install) {
    home <- Sys.getenv("HOME")
    renv <- file.path(home, ".Renviron")
    if(!file.exists(renv)){
      file.create(renv)
    } else{
      # Backup original .Renviron before doing anything else here.
      file.copy(renv, file.path(home, ".Renviron_backup"))
      if(isTRUE(overwrite)){
        message("Your original .Renviron will be backed up and stored in your R HOME directory if needed.")
        oldenv=readLines(renv)
        newenv <- oldenv[-grep("CM_CACHE_PATH", oldenv)]
        writeLines(newenv, renv, sep = "\n")
      } else{
        tv <- readLines(renv)
        if(any(grepl("CM_CACHE_PATH",tv))){
          stop("A saved cache already exists. You can overwrite it with the argument overwrite=TRUE.", call.=FALSE)
        }
      }
    }

    keyconcat <- paste0("CM_CACHE_PATH='", cache_path, "'")
    # Append cache path .Renviron file
    write(keyconcat, renv, sep = "\n", append = TRUE)
    message('Your cache path has been stored in your .Renviron and can be accessed by Sys.getenv("CM_CACHE_PATH").')
    Sys.setenv('CM_CACHE_PATH' = cache_path)
  } else {
    message("Cache set for duration of session. To permanently add your cache path for use across sessions, run this function with install = TRUE.")
    Sys.setenv('CM_CACHE_PATH' = cache_path)
  }
  cache_path
}

#' View saved Censusmapper API key
#'
#' @description View saved API key'
#'
#' @export
#'
#' @examples
#' show_cancensus_api_key()
show_cancensus_api_key <- function() {
  key <- Sys.getenv('CM_API_KEY')
  if (key==""){
    message("No api key path set")
    key <- NULL
  }
  key
}

#' View saved cache directory path
#'
#' @description View saved cache path'
#'
#' @export
#'
#' @examples
#' show_cancensus_cache_path()
show_cancensus_cache_path <- function() {
  path <- Sys.getenv('CM_CACHE_PATH')
  if (path==""){
    message("No cache path set")
    path <- NULL
  }
  path
}

cm_no_cache_path_message <- paste(
  "Census data is currently stored temporarily.\n\n",
  "In order to speed up performance, reduce API quota usage, and reduce",
  "unnecessary network calls, please set up a persistent cache directory via",
  "`set_cancensus_cache_path('<local cache path>', install = TRUE)`.\n",
  "This will add your cache directory as environment varianble to your .Renviron to be",
  "used across sessions and projects.\n\n"
)

#' Visualize Census vector hierarchies as ASCII tree
#'
#' @description Displays an ASCII tree representation of the hierarchical structure
#' of Census vectors. This helps users understand the relationship between parent
#' and child vectors when selecting variables for their analysis.
#'
#' @param vector A Census vector code (e.g., "v_CA16_2510") or a filtered tibble
#'   as returned from \code{list_census_vectors}.
#' @param dataset The dataset to query for vector information, e.g. \code{"CA16"}.
#'   Only required if \code{vector} is a character string.
#' @param max_depth Maximum depth of the tree to display. Default is \code{NA}
#'   which shows the entire hierarchy.
#' @param show_type Logical. If \code{TRUE}, shows the type (Total/Male/Female)
#'   next to each vector. Default is \code{FALSE}.
#' @param quiet When \code{TRUE}, suppress messages. Default is \code{FALSE}.
#'
#' @return Invisibly returns a tibble of the vectors displayed in the tree.
#'   The tree is printed to the console as a side effect.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' # Visualize the age hierarchy for 2016 Census
#' visualize_vector_hierarchy("v_CA16_2510", dataset = "CA16")
#'
#' # Show only first two levels with type information
#' visualize_vector_hierarchy("v_CA16_2510", dataset = "CA16",
#'                            max_depth = 2, show_type = TRUE)
#'
#' # Using a vector tibble from list_census_vectors
#' library(dplyr)
#' list_census_vectors("CA16") %>%
#'   filter(vector == "v_CA16_2510") %>%
#'   visualize_vector_hierarchy()
#' }
visualize_vector_hierarchy <- function(vector, dataset = NULL, max_depth = NA,
                                       show_type = FALSE, quiet = FALSE) {

  # Handle input - either character vector code or tibble

  if (is.character(vector)) {
    if (is.null(dataset)) {
      # Try to extract dataset from vector code
      dataset <- tryCatch({
        dataset_from_vector_list(vector)
      }, error = function(e) {
        stop("Cannot determine dataset from vector code. Please provide the 'dataset' parameter.",
             call. = FALSE)
      })
    }
    dataset <- translate_dataset(dataset)

    # Get the root vector info
    all_vectors <- list_census_vectors(dataset, quiet = TRUE)
    root_info <- all_vectors %>% dplyr::filter(.data$vector == !!vector)

    if (nrow(root_info) == 0) {
      stop(paste0("Vector '", vector, "' not found in dataset '", dataset, "'."),
           call. = FALSE)
    }
  } else if (inherits(vector, "data.frame")) {
    # It's a tibble from list_census_vectors
    if (nrow(vector) == 0) {
      stop("Empty vector tibble provided.", call. = FALSE)
    }
    dataset <- attr(vector, "dataset")
    if (is.null(dataset)) {
      dataset <- dataset_from_vector_list(vector)
    }
    dataset <- translate_dataset(dataset)
    root_info <- vector
    all_vectors <- list_census_vectors(dataset, quiet = TRUE)
  } else {
    stop("'vector' must be a character string or a tibble from list_census_vectors().",
         call. = FALSE)
  }

  # Use first row if multiple vectors provided
  if (nrow(root_info) > 1) {
    if (!quiet) message("Multiple vectors provided. Using first vector: ", root_info$vector[1])
    root_info <- root_info[1, ]
  }

  root_vector <- root_info$vector

  # Get all children
  children <- child_census_vectors(root_vector, leaves_only = FALSE, max_level = max_depth)

  # Combine root with children
  tree_vectors <- dplyr::bind_rows(root_info, children)

  # Build the tree structure
  if (!quiet) {
    message("Vector hierarchy for ", root_vector, " (", dataset, "):\n")
  }

  # Print root
  root_label <- root_info$label
  if (show_type) {
    root_label <- paste0(root_label, " [", root_info$type, "]")
  }
  cat(paste0(root_vector, ": ", root_label, "\n"))

  # Recursively print children
  print_children(root_vector, all_vectors, tree_vectors, prefix = "",
                 show_type = show_type, current_depth = 1, max_depth = max_depth)

  invisible(tree_vectors)
}

#' Internal function to recursively print tree children
#' @noRd
print_children <- function(parent_vector, all_vectors, tree_vectors, prefix,
                           show_type, current_depth, max_depth) {

  # Check depth limit

if (!is.na(max_depth) && current_depth > max_depth) {
    return(invisible(NULL))
  }

  # Find direct children of this parent that are in our tree
  direct_children <- tree_vectors %>%
    dplyr::filter(.data$parent_vector == !!parent_vector)

  if (nrow(direct_children) == 0) return(invisible(NULL))

  n_children <- nrow(direct_children)

  for (i in seq_len(n_children)) {
    child <- direct_children[i, ]
    is_last <- (i == n_children)

    # Determine connectors
    connector <- if (is_last) "\u2514\u2500\u2500 " else "\u251C\u2500\u2500 "
    child_prefix <- if (is_last) "    " else "\u2502   "

    # Build label
    label <- child$label
    if (show_type) {
      label <- paste0(label, " [", child$type, "]")
    }

    # Check if this is a leaf (no children in tree)
    has_children_in_tree <- any(tree_vectors$parent_vector == child$vector, na.rm = TRUE)
    leaf_indicator <- if (!has_children_in_tree) " (leaf)" else ""

    # Print this child
    cat(paste0(prefix, connector, child$vector, ": ", label, leaf_indicator, "\n"))

    # Recursively print grandchildren
    print_children(child$vector, all_vectors, tree_vectors,
                   prefix = paste0(prefix, child_prefix),
                   show_type = show_type,
                   current_depth = current_depth + 1,
                   max_depth = max_depth)
  }

  invisible(NULL)
}

#' Read the geosuite data
#'
#' @description
#' Reads the geosuite data for the given level and census year. Data gets cached after first download if the
#' cancensus cache path has been set. For older
#' years `get_statcan_geographic_attributes()` can fill in most of the information
#'
#' @param level geographic level to return the data for, valid choices are
#' "DB", "DA", "ADA", "CT", "CSD", "CMA", "CD", "PR", "FED", "DPL", "ER", "PN", "POPCTR"
#' @param census_year census year to get the data for, right now only 2021 is supported
#' @param refresh (logical) refresh the cache if true
#' @return tibble with the geosuite data
#'
#'
#' @examples
#' # list add the cached census data
#' \dontrun{
#' get_statcan_geo_suite("DA","2021")
#' }
#' @export
get_statcan_geo_suite <- function(level,census_year="2021",refresh=FALSE){
  valid_years <- c("2021") #seq(2001,2021,5) %>% as.character()
  valid_levels <- c("DB", "DA", "CT", "ADA", "CSD", "CMA", "CD", "PR","FED","DPL","ER","PN","POPCTR")
  if (!(as.character(census_year) %in% valid_years)) {
    stop(paste0("Only census years ",paste0(valid_years,collapse = ", "),
                " are supported for GeoSuite"))
  }
  if (level=="CMA" || level=="CA") {level="CMA_CA"}
  if (!(level %in% c(valid_levels,"CMA_CA"))) {
    stop(paste0("Only levels ",paste0(valid_levels,collapse = ", ")," are supported for GeoSuite"))
  }
  path <- cache_path(paste0("geo_suite_",census_year))
  file_path <- file.path(path,"2021_92-150-X_eng",paste0(level,".csv"))
  if (refresh || !dir.exists(path) || !file.exists(file_path)) {
    tmp <- tempfile()
    urls <- c("2016"="https://www12.statcan.gc.ca/census-recensement/2016/geo/ref/geosuite/files-fichiers/GeoSuite_2016_92-150_XBB_eng.zip",
              "2021"="https://www12.statcan.gc.ca/census-recensement/2021/geo/aip-pia/geosuite/files-fichiers/2021_92-150-X_eng.zip",
              "2011"="https://www12.statcan.gc.ca/census-recensement/2011/geo/ref/files-fichiers/2011_92-150_XBB_eng.zip",
              "2006"="https://www12.statcan.gc.ca/census-recensement/2011/geo/ref/files-fichiers/2006_92-150_XBB_eng.zip",
              "2001"="https://www12.statcan.gc.ca/census-recensement/2011/geo/ref/files-fichiers/92F0150WCB2001000.zip")
    url <- urls[[census_year]]
    old_timeout <- getOption("timeout")
    options(timeout=10000)
    utils::download.file(url,destfile=tmp, mode = "wb")
    options(timeout=old_timeout)
    if (!dir.exists(path)) dir.create(path)
    if (Sys.info()[['sysname']]=="Darwin") {
      system(paste0("ditto -V -x -k --sequesterRsrc --rsrc '",tmp,"' '",path,"'"))
    } else {
      utils::unzip(tmp,exdir = path)
    }
  }
  gs <- NULL
  if (census_year=="2021") {
    gd <- readr::read_csv(file_path,
                          locale = readr::locale(encoding ="Windows-1252"),
                          col_types = readr::cols(.default="c")) %>%
      dplyr::mutate(dplyr::across(dplyr::matches("pop_|tdwell_|urdwell_|area"),as.numeric))
  } else { #if (census_year=="2021") {
    stop("GeoSuite data is only available for the 2021 census year at this time.")
    # if (level=="DB") level="CB"
    # gd<-Hmisc::mdb.get(file.path(path,"data/geosuite2001.mdb"),
    #                   tables=level) %>%
    #   as_tibble() %>%
    #   dplyr::mutate(dplyr::across(dplyr::matches("\\.POP$|\\.PODWELL$"),as.numeric),
    #                 dplyr::across(dplyr::matches("\\.ID$|.+CODE$|.+NAME$|IRRFLAG$"),as.character))
  }

  gd
}




#' Read the Dissemination Geographies Relationship File
#'
#' @description
#' Reads the Dissemination Geographies Relationship File for the given census year. The table contains
#' the information on how all the geographic levels are related for each area. Data gets cached after first download if the
#' cancensus cache path has been set. A reference guide is available
#' at https://www150.statcan.gc.ca/n1/en/catalogue/982600032021001
#'
#' @param census_year census year to get the data for, right now only 2021 is supported, for older
#' years `get_statcan_geographic_attributes()` can fill in most of the information
#' @param refresh (logical) refresh the cache if true
#' @return tibble with the relationship data
#'
#' @keywords internal
#'
#' @examples
#' # list add the cached census data
#' \dontrun{
#' get_statcan_geography_relationships("2021")
#' }
#' @export
get_statcan_geography_relationships <- function(census_year="2021", refresh=FALSE){
  valid_years <- c("2021")
  if (!(as.character(census_year) %in% valid_years)) {
    stop(paste0("Only census years ",paste0(valid_years,collapse = ", "),
                " are supported for the geographic relationship file."))
  }
  file_path <- cache_path(paste0("geography_relationship_",census_year,".zip"))
  if (refresh || !file.exists(file_path)) {
    url <- paste0("https://www12.statcan.gc.ca/census-recensement/",census_year,"/geo/sip-pis/dguid-idugd/files-fichiers/",census_year,"_98260004.zip")
    utils::download.file(url,  file_path,mode="wb")
  }
  readr::read_csv(file_path,
                  locale = readr::locale(encoding ="Windows-1252"),
                  col_types = readr::cols(.default="c"))
}

#' Read the Geographic Attributes File
#'
#' @description
#' Reads the Geographies Attributes File for the given census year. The table contains
#' the information on how all the geographic levels are related for each area, and population, dwelling and household counts.
#' Data gets cached after first download if the
#' cancensus cache path has been set. A reference guide is available
#' at https://www150.statcan.gc.ca/n1/en/catalogue/92-151-G2021001
#'
#' @param census_year census year to get the data for, right now only 2006, 2011, 2016, 2021 are supported
#' @param refresh (logical) refresh the cache if true
#' @return tibble with the relationship data
#'
#' @examples
#' # list add the cached census data
#' \dontrun{
#' get_statcan_geographic_attributes("2021")
#' }
#' @export
get_statcan_geographic_attributes <- function(census_year="2021",refresh=FALSE){
  census_year <- as.character(census_year)[1]
  valid_years <- seq(2006,2021,5) %>% as.character
  if (!(as.character(census_year) %in% valid_years)) {
    stop(paste0("Only census years ",paste0(valid_years,collapse = ", "),
                " are supported for the geographic relationship file."))
  }
  urls <- c("2021"="https://www12.statcan.gc.ca/census-recensement/2021/geo/aip-pia/attribute-attribs/files-fichiers/2021_92-151_X.zip",
            "2016"="https://www12.statcan.gc.ca/census-recensement/2016/geo/ref/gaf/files-fichiers/2016_92-151_XBB_txt.zip",
            "2011"="https://www12.statcan.gc.ca/census-recensement/2011/geo/ref/files-fichiers/2011_92-151_XBB_txt.zip",
            "2006"="https://www12.statcan.gc.ca/census-recensement/2011/geo/ref/files-fichiers/2006_92-151_XBB_txt.zip")

  base_path <- cache_path("attribute_files")
  if (!dir.exists(base_path)) dir.create(base_path)
  base_path_year <- file.path(base_path,census_year)
  if (refresh || !dir.exists(base_path_year)) {
    if (dir.exists(base_path_year)) unlink(base_path_year,recursive = TRUE)
    dir.create(base_path_year)
    tmp<-tempfile(fileext = ".zip")
    status <- utils::download.file(urls[[census_year]],tmp,mode="wb")
    utils::unzip(tmp,exdir = base_path_year)
  }
  if (census_year=="2021") file <- dir(base_path_year,pattern="\\.csv",full.names = TRUE)
  else file <- dir(base_path_year,pattern="\\.txt",full.names = TRUE)
  if (census_year %in% c("2016","2021")) {
    result <- readr::read_csv(file,col_types = readr::cols(.default="c"),
                              locale = readr::locale(encoding ="Windows-1252"))
  } else {
  headers <- c("DBuid",paste0(c("DBpop","DBtdwell","DBurdwell","DBarea","DB_ir"),census_year),"DAuid",
               "DAlamx","DAlamy","DAlat","DAlong",
               "PRuid","PRname", "PRename", "PRfname", "PReabbr", "PRfabbr", "FEDuid",
               "FEDname", "ERuid", "ERname", "CDuid", "CDname", "CDtype", "CSDuid", "CSDname",
               "CSDtype","SACtype","SACcode","CCSuid","CCSname","DPLuid","DPLname","DPLtype",
               "CMAPuid","CMAuid","CMAname","CMAtype","CTuid","CTcode","CTname","POPCTRRAPuid",
               "POPCTRRAuid","POPCTRRAname","POPCTRRAtype","POPCTRRAclass")

  positions <- c(1,11, 19, 27, 35, 48, 49, 57, 74,91,100,111,113,168,198,228,238,248,253, 338,
                 342, 427,431, 471, 474,481,536,539,540,543,550,605,611,696,699,704,704,807,
                 808,818,822,829,835, 839, 939,940)
  widths <- c(10,8,8,8,13,1,8,17,17,9,11,2,55,30,30,10,10,5,85,4,84,4,40,3,7,44,3,1,3,7,55,
              6,85,3,5,3,100,1,10,4,7,6,4,100,1,1)

  result <- readr::read_fwf(file,col_types = readr::cols(.default="c"),
                  locale = readr::locale(encoding ="Windows-1252"),
                  col_positions = readr::fwf_widths(widths)) %>%
    setNames(headers)
  }
  result %>%
    dplyr::mutate(dplyr::across(dplyr::matches("DBpop\\d{4}|DBtdwell\\d{4}|DBurdwell\\d{4}|DBarea"),as.numeric))
}

#' Query the StatCan WDS for metadata
#'
#' @description
#' Get official metadata information from Statistics Canada for a given geographic level. Only available for the 2021 census.
#' Data is cached for the duration of the R session.
#'
#' @param census_year census year to get the data for, right now only 2021 is supported
#' @param level geographic level to return the data for, valid choices are
#' "PR","CD","CMACA","CSD","CT","ADA","DA","ER","FED","DPL","POPCNTR", "FSA", "HR"
#' @param version optionally specify a version of the data to download. For example, for FED level data, version 1.3 will
#' access the 2013 represenation order, whereas version 2.0 will access the 2023 representation order. By default the latest
#' available version is accessed.
#' @param refresh default is `FALSE` will refresh the temporary cache if `TRUE`
#' @return tibble with the metadata
#'
#' @examples
#' # get metadata for federal electoral districts
#' \dontrun{
#' get_statcan_wds_metadata(census_year="2021",level="FED")
#' }
#' @export
get_statcan_wds_metadata <- function(census_year,level,version=NULL,refresh=FALSE){
  valid_census_years <- c("2021")
  valid_levels <- c("PR","CD","CMACA","CSD","CT","ADA","DA","ER","FED","DPL","POPCNTR","FSA","HR")
  if (!(census_year %in% valid_census_years)) {
    stop(paste0("Census year must be one of ",paste0(valid_census_years,collapse = ", "),"."))
  }
  if (!(level %in% valid_levels)) {
    stop(paste0("Level must be one of ",paste0(valid_levels,collapse = ", "),"."))
  }
  meta_url <- paste0("https://api.statcan.gc.ca/census-recensement/profile/sdmx/rest/dataflow/STC_CP/DF_",level,
                     ifelse(is.null(version),"",paste0("/",version)),
                     "?references=all")
  metadata_tempfile <- file.path(tempdir(),paste0("census_wds_metadata_",digest::digest(meta_url),".sdmx"))
  if (refresh || !file.exists(metadata_tempfile)) {
    utils::download.file(meta_url,metadata_tempfile)
  }
  d <- xml2::read_xml(metadata_tempfile)
  code_lists <- xml2::xml_find_all(d,"//structure:Codelist")

  meta_data <- lapply(code_lists, function(cl){
    codelist_id <- cl %>% xml2::xml_attr("id")
    agencyID <- cl %>% xml2::xml_attr("agencyID")
    codelist_en <- cl %>% xml2::xml_find_all("common:Name[@xml:lang='en']") %>% xml2::xml_text()
    codelist_fr <- cl %>% xml2::xml_find_all("common:Name[@xml:lang='fr']") %>% xml2::xml_text()
    description_en <- cl %>% xml2::xml_find_all("common:Name[@xml:lang='en']") %>% xml2::xml_text()
    description_fr <- cl %>% xml2::xml_find_all("common:Name[@xml:lang='fr']") %>% xml2::xml_text()
    codes <- cl %>% xml2::xml_find_all("structure:Code")
    dplyr::tibble(`Agency ID`=agencyID,
           `Codelist ID`=codelist_id,
           `Codelist en`=codelist_en,
           `Codelist fr`=codelist_fr,
           ID=codes %>% xml2::xml_attr("id"),
           en=codes %>% xml2::xml_find_all("common:Name[@xml:lang='en']") %>% xml2::xml_text(),
           fr=codes %>% xml2::xml_find_all("common:Name[@xml:lang='fr']") %>% xml2::xml_text(),
           `Parent ID`=codes %>% xml2::xml_find_all("structure:Parent/Ref",flatten=FALSE) %>%
             lapply(function(d)ifelse(is.null(d),NA,xml2::xml_attr(d,"id")))  %>% unlist()
             )
  }) %>%
    dplyr::bind_rows()
 meta_data
}

#' Query the StatCan WDS for data
#'
#' @description
#' Get official census data from Statistics Canada for a given set of DGUIDs. Only available for the 2021 census. The
#' downloaded data gets enriched by geographic and characteristic names based on metadata obtained via `get_statcan_wds_metadata()`.
#' Data is cached for the duration of the R session.
#'
#' @param DGUIDs census year to get the data for, right now only 2021 is supported. Valid DGUIDs for a given geographic
#' level can be queried via `get_statcan_wds_metadata()`.
#' @param members list of Member IOs to download data for. By default all characteristics are downloaded. Valid
#' Member IDs and their descriptions can be queried via the `get_statcan_wds_metadata()` call.
#' @param version optionally specify a version of the data to download. For example, for FED level data, version 1.3 will
#' access the 2013 represenation order, whereas version 2.0 will access the 2023 representation order. By default the latest
#' available version is accessed.
#' @param gender optionally query data for only one gender. By default this queries data for all genders, possible
#' values are "Total", "Male", "Female" to only query total data, or for males only or for females only.
#' @param language specify language for geography and characteristic names that get added, valid choices are "en" and "fr"
#' @param refresh default is `FALSE` will refresh the temporary cache if `TRUE`
#' @return tibble with the enriched census data
#'
#' @examples
#' # get data for federal electoral district 2013A000459021
#' \dontrun{
#' get_statcan_wds_data(DGUIDs="2013A000459021",level="FED")
#' }
#' @export
get_statcan_wds_data <- function(DGUIDs,
                           members = NULL,
                           version= NULL,
                           gender="All",
                           language="en",
                           refresh=FALSE) {
  DGUIDs <- sort(DGUIDs)
  members <- sort(members)
  level <- geo_level_from_DGUID(DGUIDs[1])
  url <- paste0("https://api.statcan.gc.ca/census-recensement/profile/sdmx/rest/data/STC_CP")
  gender <- tolower(gender)
  gender <- paste0(toupper(substr(gender,1,1)),substr(gender,2,100))
  valid_genders <- c("All","Total","Male","Female")
  if (!(gender %in% valid_genders)) {
    stop(paste0("Gender must be one of ",paste0(valid_genders,collapse = ", "),"."))
  }
  language <- tolower(language)
  valid_languages <- c("en","fr")
  if (!(language %in% valid_languages)) {
    stop(paste0("Language must be one of ",paste0(valid_languages,collapse = ", "),"."))
  }
  gender <- c("All"="","Total"="1","Male"="2","Female"="3")[[gender]]
  dguid_string <- paste0(DGUIDs,collapse="+")
  member_string <- paste0(members,collapse = "+")
  add=paste0("DF_",level,ifelse(is.null(version),"",paste0(",",version)),"/A5.",dguid_string,".",gender,".",member_string,".1")
  wds_data_tempfile <- file.path(tempdir(),paste0("wds_data_",digest::digest(add),".csv"))
  if (!file.exists(wds_data_tempfile)) {
    response <- httr::GET(paste0(url,",",add),
                          httr::accept("text/csv"),
                          httr::add_headers("Accept-Encoding"="deflate, gzip, br"),
                          httr::write_disk(wds_data_tempfile,overwrite = TRUE))
    if (!response$status_code=="200") {
      if (!is.null(response$error) && ("curl_error_peer_failed_verification" %in% class(response$error))) {
        stop(paste0(strwrap(gsub(".+\\): ","",as.character(response$error),80)),collapse = "\n"),"\n",
             "This means that the authenticity of the StatCan API server can't be verified.\n",
             "Statistics Canada has a history of failty SSL certificats on their API,\n",
             "if you are reasonably sure that your connection is not getting hijacked you\n",
             "can disable peer checking for the duration of the R session by typing\n\n",
             "httr::set_config(httr::config(ssl_verifypeer=0,ssl_verifystatus=0))","\n\n","into the console.")
      }
      stop(paste0("Invalid request.\n",httr::content(response)))
    }
  }
  census_year <- "2021"
  meta_data <- get_statcan_wds_metadata(census_year,level=level,version = version,refresh = refresh)

  levels <- meta_data %>%
    dplyr::filter(.data$`Codelist ID`=="CL_GEO_LEVEL")

  meta_geos <- meta_data %>%
    dplyr::filter(.data$`Codelist ID`==paste0("CL_GEO_",level))
  meta_characteristics <- meta_data %>%
    dplyr::filter(.data$`Codelist ID`=="CL_CHARACTERISTIC")

  name_field <- language #paste0(language,"_description")

  data <- readr::read_csv(wds_data_tempfile,col_types = readr::cols(.default="c")) %>%
    dplyr::mutate(dplyr::across(dplyr::matches("OBS_VALUE|TNR_CI_"),as.numeric)) %>%
    dplyr::left_join(meta_geos %>%
                       dplyr::select(GEO_DESC=.data$ID,GEO_NAME=!!as.name(name_field)),
                     by="GEO_DESC") %>%
    dplyr::left_join(meta_characteristics %>%
                       dplyr::select(CHARACTERISTIC=.data$ID,CHARACTERISTIC_NAME=!!as.name(name_field)),
              by="CHARACTERISTIC")

  data
}



geo_level_from_DGUID <- function(DGUID,simple=TRUE){
  schema <- substr(DGUID,"6","9")
  schema_to_level <- c("0000"="C","0001"="C",
                       "0002"="PR",
                       "0003"="CD",
                       "0004"="FED",
                       "0005"="CSD",
                       "0006"="DPL",
                       "0007"="HR",
                       "0008"="LHR",
                       "0011"="FSA",
                       "0500"="ER",
                       "0501"="CAR",
                       "0502"="CCSD",
                       "0503"="CMA",
                       "0504"="CA",
                       "0505"="CMAP",
                       "0507"="CT",
                       "0510"="PC",
                       "0511"="PCP",
                       "0512"="DA",
                       "0513"="DB",
                       "0516"="ADA")
  level <- schema_to_level[[schema]]
  if (simple){
    simple_translation <- c("LHR"="HR","CMA"="CMACA","CA"="CMACA","CMAP"="CMACA")
    if (level %in% names(simple_translation)) level <- simple_translation[[level]]
  }
  level
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)

## ----setup--------------------------------------------------------------------
# Packages used for example
library(cancensus)
library(dplyr)
library(tidyr)
library(ggplot2)
library(sf)

## -----------------------------------------------------------------------------
list_census_datasets() %>% 
  filter(grepl("taxfiler",description))

## -----------------------------------------------------------------------------
list_census_vectors('TX2017')

## -----------------------------------------------------------------------------
years <- c(2006,2011,2014,2018)
# Attribution for the dataset to be used in graphs
attribution <- dataset_attribution(paste0("TX",years))

plot_data <- years %>%
  lapply(function(year) {
    dataset <- paste0("TX",year)
    vectors <- c("Families"=paste0("v_",dataset,"_607"),
                 "CFLIM-AT"=paste0("v_",dataset,"_786"))
    
    get_census(dataset,regions=list(CMA="59933"),vectors = vectors,
                    geo_format = 'sf', level="CT", quiet = TRUE) %>%
      select(c("GeoUID",names(vectors))) %>%
      mutate(Year=year)
  }) %>%
  bind_rows() %>%
  mutate(share=`CFLIM-AT`/Families)

## ----fig.alt="Share of census families in low income"-------------------------
ggplot(plot_data,aes(fill=share)) +
  geom_sf(size=0.1,color="white") +
  facet_wrap("Year") +
  scale_fill_viridis_c(labels=scales::percent,option = "inferno",
                       trans="log",breaks = c(0.05,0.1,0.2,0.4)) +
  coord_sf(datum=NA,xlim=c(-123.4, -122.5), ylim=c(49.01, 49.4)) +
  labs(title="Share of census families in low income",fill="Share",
       caption=attribution)

## ----fig.alt="Change in share of census families in low income 2006-2011"-----
change_data <- plot_data %>% 
  filter(Year==2006) %>% 
  select(GeoUID,`2006`=share) %>%
  left_join(plot_data %>%
              st_set_geometry(NULL) %>%
              filter(Year==2011) %>% 
              select(GeoUID,`2011`=share),
            by="GeoUID") %>%
  mutate(change=`2011`-`2006`)
  
ggplot(change_data,aes(fill=change)) +
  geom_sf(size=0.1) +
  scale_fill_gradient2(labels=scales::percent) +
  #scale_fill_viridis_c(labels=scales::percent,option = "inferno") +
  coord_sf(datum=NA,xlim=c(-123.4, -122.5), ylim=c(49.01, 49.4)) +
  labs(title="Change in share of census families in low income 2006-2011",fill="Percentage\npoint change",caption=dataset_attribution(paste0("TX",c(2006,2011))))

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE, 
  message = FALSE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)
library(cancensus)

## -----------------------------------------------------------------------------
list_census_datasets()

## ----echo=FALSE, fig.alt="Total number of unique variable vectors by Census dataset"----
library(ggplot2)
library(dplyr)
datasets <- c("CA1996","CA01","CA06","CA11","CA16","CA21")

datasets %>%
  lapply(function(ds){
    dplyr::tibble(dataset=ds, vectors=nrow(list_census_vectors(ds)))
  }) %>%
  bind_rows() %>%
  mutate(dataset=factor(dataset,levels=datasets)) %>%
  ggplot(., aes(x = dataset, y = vectors)) +
  geom_col() +
  theme_minimal() +
  theme(panel.grid = element_blank()) +
  labs(x = "Census dataset", y = "",
       title = "Total number of unique variable vectors by Census dataset") +
  scale_y_continuous(labels = scales::comma)

## ----message=FALSE, warning=FALSE---------------------------------------------
list_census_vectors('CA21')

## ----warning=TRUE, message=TRUE-----------------------------------------------
find_census_vectors("Oji-cree", dataset = "CA16", type = "total", query_type = "exact")

## ----warning=TRUE, message=TRUE-----------------------------------------------
find_census_vectors("Ojib-cree", dataset = "CA16", type = "total", query_type = "exact")

## ----warning=TRUE, message=TRUE-----------------------------------------------
find_census_vectors('commute mode', dataset = 'CA16', type = 'female', query_type = 'keyword', interactive = FALSE)

## -----------------------------------------------------------------------------
find_census_vectors("after tax incomes", dataset = "CA16", type = "total", query_type = "semantic")

## ----warning=TRUE, message=TRUE-----------------------------------------------
find_census_vectors("ojib cree", dataset = "CA16", type = "total", query_type = "exact")

## ----warning=TRUE, message=TRUE-----------------------------------------------
find_census_vectors('ojib cree', dataset = 'CA16', type = 'total', query_type = 'semantic')

## ----echo=FALSE---------------------------------------------------------------
tibble(PR = c(35,35,35), CD = c(NA, 18, 18), CSD = c(NA, NA, 013), name = c("Ontario","Durham (Regional municipality","Oshawa (City)"))

## ----echo=FALSE---------------------------------------------------------------
list_census_regions('CA21') %>% 
  group_by(level) %>% 
  tally()

## -----------------------------------------------------------------------------
list_census_regions('CA21')

## ----echo=FALSE, paged.print=TRUE---------------------------------------------
if(Sys.getenv("COMPILE_VIG")=="TRUE") {
    rmarkdown::paged_table(CODES_TABLE)
}

## -----------------------------------------------------------------------------
search_census_regions("Vancouver","CA21")

# recalled data functionality

recall_data_path <- function(){
  file.path(tempdir(),"recall.csv")
}


#' Get metadata for recalled data
#'
#' @description
#' Grabs recall data from server if needed and returns list of recalled data
#'
#' @keywords internal
#'
#' @param refresh Will force refresh of recalled database
#' @param warn_only_once Will only warn on first run, `FALSE` by default
#' @return tibble with rows describing recalled data
#'
#' @examples
#' \dontrun{
#' get_recalled_database()
#' }
get_recalled_database <- function(refresh=FALSE, warn_only_once=FALSE){
  problem <- FALSE
  path <- recall_data_path()
  if (refresh || !file.exists(path)) {
    url <- paste0(cancensus_base_url(),"/api/v1/recall.csv")
    tryCatch(
      utils::download.file(url,path,mode="wb",quiet=TRUE),
      error = function(e) {
        warning("Unable to download recall database at this point.")
        problem <- TRUE
      },
      warning = function(e) {
        warning("Unable to download recall database at this point.")
        problem <<- TRUE
      }
    )
  }
  data <- NULL
  if (!problem) {
    data <- readr::read_csv(path,col_types = readr::cols(.default="c"))
  }
  if (length(names(data))==0) {
    if (!warn_only_once && !problem) {
      data <- get_recalled_database(refresh=TRUE,warn_only_once = TRUE)
    } else {
      data <- NULL
    }
  }
  data
}

#' List recalled data stored in local cache
#'
#' @description
#' Checks the local cached database for recalled data and lists all recalled cached entries
#'
#' @param cached_data List of locally cached data to check for recall, default is `list_cancensus_cache()` in which case
#' it will get checked against all locally cached data
#' @param warn_only_once Will only warn on first run during each session, `FALSE` by default
#' @return tibble with rows describing locally cached recalled data
#'
#' @export
#'
#' @examples
#' \dontrun{
#' list_recalled_cached_data()
#' }
list_recalled_cached_data <- function(cached_data=list_cancensus_cache(),warn_only_once=FALSE){
  data <- NULL
  recall_database <- get_recalled_database(warn_only_once=warn_only_once)
  if (!is.null(recall_database)) {
    #if (is.null(cached_data)) cached_data <- list_cancensus_cache()
    vector_recall <- recall_database %>%
      dplyr::filter(grepl("^d\\.",.data$api_version)) %>%
      dplyr::group_by(.data$api_version,.data$dataset,.data$level) %>%
      dplyr::group_map(~dplyr::filter(cached_data %>% dplyr::filter(grepl("^d\\.",.data$version)),
                                      is.na(.y$level) | .y$level==.data$level | .data$level=="Regions",
                                      .data$version<=.y$api_version,
                                      .data$dataset==.y$dataset,
                                      grepl(paste0(.x$vector,collapse = "|"),.data$vectors))) %>%
      dplyr::bind_rows() %>%
      dplyr::ungroup()

    geo_recall <- recall_database %>%
      dplyr::filter(grepl("^g\\.",.data$api_version)) %>%
      dplyr::group_by(.data$api_version,.data$dataset,.data$level) %>%
      dplyr::group_map(~dplyr::filter(cached_data %>% dplyr::filter(grepl("^g\\.",.data$version)),
                                      is.na(.y$level) | .y$level==.data$level | .data$level=="Regions",
                                      .data$version<=.y$api_version,
                                      .data$dataset==.y$dataset)) %>%
      dplyr::bind_rows() %>%
      dplyr::ungroup()

    data <- dplyr::bind_rows(vector_recall,geo_recall)
  }
  data
}



#' Remove recalled data from local cache
#'
#' @description
#' Checks the local cached database for recalled data and removes cached data that has been recalled
#'
#' @return Storage size of removed locally cached data that got freed up in number of bytes.
#' @export
#'
#' @examples
#' \dontrun{
#' remove_recalled_cached_data()
#' }
remove_recalled_cached_data <- function(){
  recalled_data <- list_recalled_cached_data()
  size <- 0
  if (is.null(recalled_data)) {
  } else if (nrow(recalled_data)>0) {
    size<-remove_from_cancensus_cache(recalled_data$path)
    message(paste0("Removing ",nrow(recalled_data)," datasets totalling ",size," bytes."))
  } else {
    message("No recalled data in cached data.")
  }
  size
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)

mb_token <- Sys.getenv("MB_TOKEN")

## ----message = FALSE, warning = FALSE-----------------------------------------
library(cancensus)
library(sf)
# retrieve sf dataframe
toronto <- get_census(dataset='CA21', regions=list(CMA="35535"),
                         vectors=c("median_hh_income"="v_CA21_906"), level='CSD', quiet = TRUE, 
                         geo_format = 'sf', labels = 'short')

## ----fig.alt="Toronto Household Income by CSD in 2020"------------------------
plot(toronto["median_hh_income"], main = "Toronto Household Income by CSD in 2020")

## ----fig.alt="Toronto CSDs with Median HH Income > $125,000 in 2020"----------
plot(st_geometry(toronto), col = NA, main = "Toronto CSDs with Median HH Income > $125,000 in 2020", lty = 3)
plot(st_geometry(toronto[toronto$median_hh_income > 125000,]), col = "red", add = TRUE)

## ----fig.alt="Toronto CSDs with Median HH Income > $125,000 in 2020"----------
library(ggplot2)
ggplot(toronto) + geom_sf(aes(fill = median_hh_income))

## ----fig.alt="Toronto Median Household Income in 2020"------------------------
ggplot(toronto) + geom_sf(aes(fill = median_hh_income), colour = "grey") +
  scale_fill_viridis_c("Median HH Income", labels = scales::dollar) + theme_minimal() +
  theme(panel.grid = element_blank(),
        axis.text = element_blank(),
        axis.ticks = element_blank()) + 
  coord_sf(datum=NA) +
  labs(title = "Median Household Income in 2020", subtitle = "Toronto Census Subdivisions, 2021 Census")

## ----message=FALSE, warning=FALSE, include=FALSE------------------------------
# This chunk disables leaflet and mapdeck output in remaining chunks for PDF vignettes
if(Sys.getenv("COMPILE_VIG")!="TRUE") {
  knitr::opts_chunk$set(eval = FALSE)}

## ----message = FALSE----------------------------------------------------------
library(leaflet)
leaflet(toronto) %>% 
  addProviderTiles(providers$CartoDB.Positron) %>%
  addPolygons()

## -----------------------------------------------------------------------------
bins <- c(0, 30000,40000, 50000,60000, 70000,80000, 90000,100000, 110000, Inf)
pal <- colorBin("RdYlBu", domain = toronto$v_CA16_2397, bins = bins)
leaflet(toronto) %>% 
  addProviderTiles(providers$CartoDB.Positron) %>%
  addPolygons(fillColor = ~pal(median_hh_income),
              color = "white",
              weight = 1,
              opacity = 1,
              fillOpacity = 0.65)

# Internal functions that do useful things frequently required in other functions

cancensus_base_url <- function(){
  url <- getOption("cancensus.base_url")
  if (is.null(url)) url <- "https://censusmapper.ca"
  url
}

valid_api_key <- function(api_key){
  !is.null(api_key) && is.character(api_key) && substr(api_key,1,13)=="CensusMapper_"
}

robust_api_key <- function(api_key){
  api_key <- if (!valid_api_key(api_key) && nchar(Sys.getenv("CM_API_KEY")) > 1) { Sys.getenv("CM_API_KEY") } else { api_key }
  api_key <- if (!valid_api_key(api_key) && !is.null(getOption("cancensus.api_key"))) { getOption("cancensus.api_key") } else { api_key }
  api_key
}

# Append arguments to the path of the local cache directory.
cache_path <- function(...) {
  cache_dir <- Sys.getenv("CM_CACHE_PATH")
  if (nchar(cache_dir)==0) {
    if (!is.null(getOption("cancensus.cache_path"))) {
      cache_dir <- getOption("cancensus.cache_path")
    } else {
      cache_dir <- tempdir()
      message(cm_no_cache_path_message)
    }
  }
  if (!is.character(cache_dir)) {
    stop("Corrupt 'CM_CACHE_PATH' environment variable or 'cancensus.cache_path' option. Must be a path.",
         .call = FALSE)
  }
  if (!file.exists(cache_dir)) {
    dir.create(cache_dir, showWarnings = FALSE)
  }
  cache_key <- paste0(...)
  if (!identical(cache_key, character(0)))
    cache_dir <- file.path(cache_dir, paste0(...))
  cache_dir
}


translate_dataset <- function(dataset) {
  dataset <- as.character(dataset)
  translations <- c("1996"="CA1996",
                    "2001"="CA01",
                    "2006"="CA06",
                    "2011"="CA11",
                    "2016"="CA16",
                    "2021"="CA21")
  #dataset <- toupper(dataset)
  if (dataset %in% names(translations)) dataset=as.character(translations[dataset])
  dataset
}

clean_vector_list <- function(vector_list,dataset=NULL){
  if (!inherits(vector_list,"data.frame")) {
    if (inherits(vector_list,"character")) {
      if (is.null(dataset))  dataset <- dataset_from_vector_list(vector_list)
      vector_list = list_census_vectors(dataset) %>%
        dplyr::filter(vector %in% vector_list)
    } else
      stop(paste0("Don't know how to parse vector list: ",vector_list))
  }
  vector_list
}

dataset_from_vector_list <- function(vector_list){
  dataset <- attr(vector_list,'dataset')
  if (is.null(dataset)) {
    vectors = ifelse(inherits(vector_list,"character"),vector_list,vector_list$vector)
    dataset <- vectors %>%
      as.character() %>%
      lapply(function(d)unlist(strsplit(d,"_"))[2]) %>%
      unlist() %>%
      unique()
    if (length(dataset)!=1) stop("Unable to determine dataset")
  }
  dataset
}

cancensus_na_strings <- c("x", "X", "F", "...", "..", "-","N","*","**")

#' Retry an API call with exponential backoff
#'
#' @param call_fn A function that performs the API call and returns an httr response
#' @param max_retries Maximum number of retry attempts (default: 3)
#' @param quiet If TRUE, suppress retry messages
#' @return The httr response object
#' @keywords internal
retry_api_call <- function(call_fn, max_retries = 3, quiet = FALSE) {
 attempt <- 1
 last_error <- NULL

 while (attempt <= max_retries) {
   tryCatch({
     response <- call_fn()

     # Check for transient HTTP errors (5xx, timeout, connection errors)
     status <- httr::status_code(response)
     if (status >= 500 && status < 600 && attempt < max_retries) {
       # Server error - retry
       wait_time <- 2 ^ (attempt - 1)  # Exponential backoff: 1, 2, 4 seconds
       if (!quiet) {
         message(sprintf("Server error (HTTP %d), retrying in %ds (attempt %d/%d)...",
                         status, wait_time, attempt + 1, max_retries))
       }
       Sys.sleep(wait_time)
       attempt <- attempt + 1
       next
     }

     # Success or non-retryable error
     return(response)

   }, error = function(e) {
     last_error <<- e
     # Network errors - retry
     if (attempt < max_retries) {
       wait_time <- 2 ^ (attempt - 1)
       if (!quiet) {
         message(sprintf("Network error: %s. Retrying in %ds (attempt %d/%d)...",
                         conditionMessage(e), wait_time, attempt + 1, max_retries))
       }
       Sys.sleep(wait_time)
       attempt <<- attempt + 1
     } else {
       stop(e)
     }
   })
 }

 # If we've exhausted retries, throw the last error
 if (!is.null(last_error)) {
   stop(last_error)
 }
}

#' Format bytes to human-readable size
#' @keywords internal
format_bytes <- function(bytes) {
 if (bytes < 1024) return(paste0(bytes, " B"))
 if (bytes < 1024^2) return(sprintf("%.1f KB", bytes / 1024))
 if (bytes < 1024^3) return(sprintf("%.1f MB", bytes / 1024^2))
 return(sprintf("%.1f GB", bytes / 1024^3))
}

as.num = function(x, na.strings = cancensus_na_strings) {
  stopifnot(is.character(x))
  na = x %in% na.strings
  x[na] = 0
  x = as.numeric(x)
  x[na] = NA_real_
  x
}

as.int = function(x, na.strings = cancensus_na_strings) {
  stopifnot(is.character(x))
  na = x %in% na.strings
  x[na] = 0
  x = as.integer(x)
  x[na] = NA_integer_
  x
}

check_recalled_data_and_warn <- function(meta_file,params){
  cached_data<-generate_metadata(meta_file,params)
  recalled_data <- list_recalled_cached_data(cached_data,warn_only_once=TRUE)
  if (!is.null(recalled_data) && nrow(recalled_data)>0) {
    warning("Currently loaded data has been recalled. Use\nlist_recalled_cached_data()\nto inspect recalled locally cached data and\nremove_recalled_cached_data()\nto remove recalled data.")
  }
  d<-NULL
}

check_for_recalled_data_and_warn <- function(){
  recalled_data <- list_recalled_cached_data(warn_only_once=TRUE)
  if (!is.null(recalled_data) && nrow(recalled_data)>0) {
    warning(paste0("Some locally cached data has been recalled. Use\nlist_recalled_cached_data()\nto inspect recalled locally cached data and\nremove_recalled_cached_data()\nto remove recalled data."))
  }
  d<-NULL
}

first_run_checks <- function(){
  # Check caches for recalled data
  path=file.path(tempdir(),"cancensus_first_run_checks.info")
  if (!file.exists(path)) {
    # first time cancensus is run!
    check_for_recalled_data_and_warn()
    readr::write_lines("recall_check\n",path)
  }
  d<-NULL
}

#' A dataset with code table summaries for census data
#' @name CODES_TABLE
#' @docType data
#' @author derived from StatCan definitions
#' @references \url{https://www12.statcan.gc.ca/census-recensement/2021/geo/ref/domain-domaine/index2021-eng.cfm?lang=e&id=CSDtype}, \url{https://www12.statcan.gc.ca/census-recensement/2021/geo/ref/domain-domaine/index2021-eng.cfm?lang=e&id=CDtype}
#' @keywords data
NULL

#' A dataset City of Vancouver skytrain station locations
#' @name COV_SKYTRAIN_STATIONS
#' @docType data
#' @author City of Vancouver Open Data
#' @references \url{https://opendata.vancouver.ca/explore/dataset/rapid-transit-stations/information/}
#' @keywords data
NULL

#' Get identifiers for census regions intersecting a geometry
#'
#' @description
#' This function returns a list of regions that intersect a given geometry input. This list of regions can be used
#' directly to query census when one is interested in census data for a particular geographic region that does
#' not coincide with defined census geometries. The returned value can be used as the \code{regions}
#' parameter in \link{get_census} to get corresponding census geographies and variables that cover the give geometry.
#' Input spatial objects can be any \code{sf} or \code{sfc} class objects such as \code{POINT}, \code{MULTIPOINT} or \code{POLYGON}.
#'
#' This function comes with CensusMapper API limits
#'
#' @param dataset A CensusMapper dataset identifier.
#' @param level The census aggregation level to retrieve. One of \code{"Regions"}, \code{"PR"}, \code{"CMA"}, \code{"CD"}, \code{"CSD"}, \code{"CT"}, \code{"DA"}, \code{"EA"} (for 1996 census), or \code{"DB"} (for 2001-2016 censuses)..
#' @param geometry A valid \code{sf} or \code{sfc} class object. As long as the geometry is valid, any projection is accepted. Objects will be reprojected as server-side intersections use lat/lon projection.
#' @param simplified If \code{TRUE} will return a region list compatible with \link{get_census}, otherwise will return a character vector of matching region ids.
#' @param use_cache If set to \code{TRUE} (the default) data will be read from the local cache if available.
#' @param quiet When TRUE, suppress messages and warnings.
#' @param api_key An API key for the CensusMapper API. Defaults to \code{options()} and then the \code{CM_API_KEY} environment variable.
#'
#' @source Census data and boundary geographies are reproduced and distributed on
#' an "as is" basis with the permission of Statistics Canada (Statistics Canada
#' 1996; 2001; 2006; 2011; 2016).
#'
#' @export
#'
#' @examples
#'
#' \dontrun{
#' # Example using a POINT-class object from a pair of lat/lon coordinates
#' point_geo <- sf::st_sfc(sf::st_point(c(-123.25149, 49.27026)), crs=4326)
#' regions <- get_intersecting_geometries(dataset = 'CA16', level = 'CT', geometry = point_geo)
#' census_data <- get_census(dataset='CA16', regions=regions,
#'                           vectors=c("v_CA16_408","v_CA16_409","v_CA16_410"),
#'                           level='CT')
#'
#' # Example using a POLYGON-class object from a bounding box with four coordinates
#' poly_geo <- sf::st_as_sfc(sf::st_bbox(c(ymin = 49.25, ymax = 49.30,
#'                           xmin = -123.25, xmax = -123.30)), crs = 4326)
#' regions <- get_intersecting_geometries(dataset = 'CA16', level = 'CT', geometry = poly_geo)
#' census_data <- get_census(dataset='CA16', regions=regions,
#'                          vectors=c("v_CA16_408","v_CA16_409","v_CA16_410"), level='CT')
#'
#'}
get_intersecting_geometries <- function(dataset, level, geometry, simplified = FALSE,
                                        use_cache = TRUE, quiet = FALSE,
                                        api_key=Sys.getenv("CM_API_KEY")) {
  api_key <- robust_api_key(api_key)
  have_api_key <- valid_api_key(api_key)
  result <- NULL
  dataset <- translate_dataset(dataset)

    if ("sf" %in% class(geometry)) {
    geometry=sf::st_geometry(geometry)
  }
  if (!("sfc" %in% class(geometry))) {
    stop("The `geometry` parameter needs to be of class sf or sfc")
  }
  if (length(geometry)>1) {
    geometry <- sf::st_union(geometry)
  }

  crs=sf::st_crs(geometry)$epsg

  if (is.na(crs) || crs!=4326) {
    geometry <- geometry %>% sf::st_transform(4326)
  }

  geo <- geojsonsf::sfc_geojson(geometry)

  param_string <- paste0("dataset=", dataset,
                         "&level=", level,
                         "&geometry=", geo)
  data_base_name <- paste0("CM_data_intersect_",digest::digest(param_string, algo = "md5"))
  data_file <- cache_path(data_base_name, ".rda")
  meta_file <- paste0(data_file, ".meta")

  if (!use_cache || !file.exists(data_file)) {
    if (!have_api_key) {
      stop(paste("No API key set. Use set_cancensus_api_key('<your API key>') to set one, or set_cancensus_api_key('<your API key>', install = TRUE) to save it permanently in your .Renviron."))
    }
    url <- paste0(cancensus_base_url(),"/api/v1/intersecting_geographies")
    body <- list(dataset=dataset,
                 level=level,
                 area=sf::st_area(geometry) %>% as.numeric(),
                 geometry=geo,
                 api_key=api_key)
    response <- if (!quiet) {
      message("Querying CensusMapper API...")
      httr::POST(url, body = body,
                 config = httr::accept_json(),
                 httr::progress())
    } else {
      httr::POST(url, body = body,
                 config = httr::accept_json())
    }
    handle_cm_status_code(response, NULL)
    data_version <- response$headers$`data-version`

    result <- httr::content(response, type = "text", encoding = "UTF-8") %>%
      jsonlite::fromJSON()
    attr(result, "last_updated") <- Sys.time()
    save(result, file = data_file)
    file_info <- file.info(data_file)
    metadata <- dplyr::tibble(dataset=dataset,level=level,created_at=Sys.time(),
                               version=data_version,size=file_info$size)
    saveRDS(metadata, file = meta_file)
  } else {
    if (!quiet) message("Reading intersection data from local cache.")
    # Load `result` object from cache.
    load(file = data_file)
  }

  touch_metadata(meta_file)

  if(simplified) {result <- c(as.character(result))}
  result
}

#options("cancensus.base_url"="http://localhost:3000")
#options("cancensus.api_key"="CensusMapper_d79ad30f13e1daec24946aeaa8872d5d")
if (FALSE) {
  options("cancensus.base_url"="http://localhost:3000")
  Sys.setenv("CM_API_KEY"= 'CensusMapper_d79ad30f13e1daec24946aeaa8872d5d')
}

#' Query the CensusMapper API for available vectors for a given dataset.
#'
#' @param dataset The dataset to query for available vectors, e.g.
#'   \code{"CA16"}.
#' @param use_cache If set to TRUE (the default), data will be read from a local cache
#'   that is maintained for the duration of the R session, if
#'   available. If set to FALSE, query the API for the data, and
#'   refresh the local cache with the result.
#' @param quiet When FALSE, shows messages and warnings. Set to TRUE by default.
#'
#' @return
#' Returns a data frame detailing the available Census vectors (i.e. variables) for a given Census
#' dataset. This data frame has columns \code{vector} containing the short code for the
#' variable, \code{type} describing whether it's a female, male, or total aggregate, \code{label}
#' indicating the name of the variable, \code{units} indicating whether the value represents a
#' numeric integer, percentage, dollar figure, or ratio, \code{parent_vector} to show hierarchical
#' relationship, \code{aggregation} indicating whether the value is additive or a transformation,
#' and a column \code{details} with a detailed description of the variable generated by traversing
#' all labels within its hierarchical structure.
#'
#' @export
#'
#' @examples
#'
#' \dontrun{
#' # List all vectors for a given Census dataset in CensusMapper
#' list_census_vectors('CA16')
#' }
list_census_vectors <- function(dataset, use_cache = TRUE, quiet = TRUE) {
  dataset <- translate_dataset(dataset)
  cache_file <- file.path(tempdir(),paste0(dataset, "_vectors.rda"))
  if (!use_cache || !file.exists(cache_file)) {
    url <- paste0(cancensus_base_url(),"/api/v1/vector_info/", dataset, ".csv")
    response <- if (!quiet) {
      message("Querying CensusMapper API for vectors data...")
      httr::GET(url, httr::progress())
    } else {
      httr::GET(url)
    }
    handle_cm_status_code(response, NULL)
    content <- httr::content(response, type = "text", encoding = "UTF-8")
    result <- if (!requireNamespace("readr", quietly = TRUE)) {
      dplyr::as_tibble(utils::read.csv(textConnection(content), stringsAsFactors = FALSE),
                       .name_repair = "minimal")
    } else {
      readr::read_csv(content,col_types=readr::cols(.default="c"))
    }
    result <- dplyr::mutate(
      result, type = factor(.data$type),
      units = factor(units, levels = as.character(1:6),
                     labels = c("Number", "Percentage ratio (0.0-1.0)",
                                "Currency", "Ratio", "Percentage (0-100)","Currency (1000s)")),
      aggregation = dplyr::case_when(
        add == "1" ~ "Additive",
        add == "0" ~ "Not additive",
        grepl("^2.", add) ~ gsub(".", ", ", gsub("^2\\.", "Average of ", add),
                                 fixed = TRUE),
        grepl("^3.", add) ~ gsub(".", ", ", gsub("^3\\.", "Median of ", add),
                                 fixed = TRUE),
        grepl("^4.", add) ~ gsub(".", ", ", gsub("^4\\.", "Average to ", add),
                                 fixed = TRUE),
        grepl("^9.", add) ~ gsub(".", ", ", gsub("^9\\.", "Standard error based on ", add),
                                 fixed = TRUE)
      )) %>%
      dplyr::select(.data$vector, .data$type, .data$label, .data$units,
                    parent_vector = .data$parent, .data$aggregation,
                    .data$details)
    attr(result, "last_updated") <- Sys.time()
    attr(result, "dataset") <- dataset
    save(result, file = cache_file)
    result
  } else {
    if (!quiet) message("Reading vector information from local cache.")
    load(file = cache_file)
    result
  }
}

#' List all parent variables from vector hierarchies given either a list of Census
#' variables returned by
#' \code{list_census_vectors}, \code{search_census_vectors}, \code{find_census_vectors}, or a direct string reference to the vector code.
#'
#' @param vector_list The list of vectors to be used, either a character vector or a filtered tibble
#' as returned from \code{list_census_vectors}.
#'
#' @export
#'
#' @examples
#' # Query parent vectors directly using vector identifier
#' parent_census_vectors("v_CA16_2519")
#' \dontrun{
#' # Example using multiple vectors coerced into a list
#' parent_census_vectors(c("v_CA16_2519","v_CA16_2520","v_CA16_2521"))
#'
#' # or, equivalently
#' selected_vectors <- c("v_CA16_2519","v_CA16_2520","v_CA16_2521")
#' parent_census_vectors(selected_vectors)
#'
#' # Example using dplyr and piped arguments
#' library(dplyr, warn.conflicts = FALSE)
#'
#' list_census_vectors("CA16") %>%
#'   filter(vector == "v_CA16_2519") %>%
#'   parent_census_vectors()
#' }
parent_census_vectors <- function(vector_list){
  dataset <- dataset_from_vector_list(vector_list)
  vector_list <- clean_vector_list(vector_list,dataset)
  base_list <- vector_list

  # Cache the full vector list once instead of repeated API/cache lookups
  all_vectors <- list_census_vectors(dataset, use_cache = TRUE, quiet = TRUE)

  n=0
  vector_list <-
    all_vectors %>%
    dplyr::filter(vector %in% base_list$parent_vector) %>%
    dplyr::distinct(vector, .keep_all = TRUE)

  # Accumulate results in a list to avoid repeated rbind operations
  results_list <- list(vector_list)

  while (n!=nrow(vector_list)) {
    n=nrow(vector_list)
    new_list <- all_vectors %>%
      dplyr::filter(vector %in% vector_list$parent_vector)

    if (nrow(new_list) > 0) {
      results_list <- c(results_list, list(new_list))
      # Bind all results at once and get distinct vectors
      vector_list <- dplyr::bind_rows(results_list) %>%
        dplyr::distinct(vector, .keep_all = TRUE)
    }
  }
  attr(vector_list, "dataset") <- dataset
  return(vector_list)
}

#' List all child variables from vector hierarchies given either a list of Census
#' variables returned by
#' \code{list_census_vectors}, \code{search_census_vectors}, \code{find_census_vectors}, or a direct string reference to the vector code.
#'
#' @param vector_list the list of vectors to be used, either a character vector or a filtered tibble
#'   as returned from \code{list_census_vectors}.
#' @param leaves_only boolean flag to indicate if only final leaf vectors should be returned,
#' i.e. terminal vectors that themselves do not have children.
#' @param max_level optional, maximum depth to look for child vectors. Default is \code{NA} and will return all
#' child census vectors.
#' @param keep_parent optional, also return parent vector in list of results. Default is set to \code{FALSE}.
#'
#' @export
#'
#' @examples
#' # Query parent vectors directly using vector identifier
#' child_census_vectors("v_CA16_2510")
#'
#' \dontrun{
#'
#' # Example using multiple vectors coerced into a list
#' child_census_vectors(c("v_CA16_2510","v_CA16_2511","v_CA16_2512"))
#'
#' # or, equivalently
#' selected_vectors <- c("v_CA16_2510","v_CA16_2511","v_CA16_2512")
#' child_census_vectors(selected_vectors)
#'
#' # Example using dplyr and piped arguments
#' library(dplyr, warn.conflicts = FALSE)
#'
#' list_census_vectors("CA16") %>%
#'   filter(vector == "v_CA16_2510") %>%
#'   child_census_vectors(TRUE)
#'
#' # this will return the equivalent of c("v_CA16_2510", child_census_vectors("v_CA16_2510"))
#' list_census_vectors("CA16") %>%
#'   filter(vector == "v_CA16_2510") %>%
#'   child_census_vectors(TRUE, keep_parent = TRUE)
#'}
#'
child_census_vectors <- function(vector_list, leaves_only=FALSE,max_level=NA,keep_parent = FALSE){
  vector_list <- clean_vector_list(vector_list)
  base_list <- vector_list
  dataset <- dataset_from_vector_list(vector_list)
  n <- 0
  child_level <- 1
  if (!is.null(dataset)) {
    # Cache the full vector list once instead of repeated API/cache lookups
    all_vectors <- list_census_vectors(dataset, use_cache = TRUE, quiet = TRUE)

    vector_list <-
      all_vectors %>%
      dplyr::filter(.data$parent_vector %in% base_list$vector) %>%
      dplyr::distinct(vector, .keep_all = TRUE)

    # Accumulate results in a list to avoid repeated rbind operations
    results_list <- list(vector_list)

    while (n!=nrow(vector_list) && (is.na(max_level) || child_level<max_level)) {
      child_level <- child_level+1
      n=nrow(vector_list)
      new_list <- all_vectors %>%
        dplyr::filter(.data$parent_vector %in% vector_list$vector)

      if (nrow(new_list) > 0) {
        results_list <- c(results_list, list(new_list))
        # Bind all results at once and get distinct vectors
        vector_list <- dplyr::bind_rows(results_list) %>%
          dplyr::distinct(vector, .keep_all = TRUE)
      }
    }
    # only keep leaves if leaves_only==TRUE
    if (leaves_only) {
      vector_list <- vector_list %>%
        dplyr::filter(!(vector %in% all_vectors$parent_vector))
    }
    if (keep_parent) {
      vector_list <- dplyr::bind_rows(base_list,vector_list)
    }
    attr(vector_list, "dataset") <- dataset
  }
  return(vector_list)
}

test_that("parent_census_vectors returns all parent vectors", {
  skip_if_not_installed("dplyr")

  # Create simple 3-level hierarchy: v_TEST_3 -> v_TEST_2 -> v_TEST_1
  mock_vectors <- data.frame(
    vector = c("v_TEST_1", "v_TEST_2", "v_TEST_3"),
    parent_vector = c(NA, "v_TEST_1", "v_TEST_2"),
    label = c("Root", "Child", "Grandchild"),
    details = c("", "", ""),
    aggregation = rep("Additive", 3),
    type = rep("Total", 3),
    stringsAsFactors = FALSE
  )
  attr(mock_vectors, "dataset") <- "TEST"

  # Mock the list_census_vectors function temporarily
  with_mocked_bindings(
    list_census_vectors = function(dataset, use_cache = TRUE, quiet = TRUE) {
      mock_vectors
    },
    {
      # Test with grandchild - should return both parent and grandparent
      grandchild <- mock_vectors[3, , drop = FALSE]
      result <- parent_census_vectors(grandchild)

      # Should include v_TEST_2 and v_TEST_1 (all parents up the chain)
      expect_true("v_TEST_2" %in% result$vector, info = "Should include direct parent v_TEST_2")
      expect_true("v_TEST_1" %in% result$vector, info = "Should include grandparent v_TEST_1")
      expect_false("v_TEST_3" %in% result$vector, info = "Should not include itself")
      expect_equal(nrow(result), 2, info = "Should return exactly 2 parents")

      # Test with child - should return only the root
      child <- mock_vectors[2, , drop = FALSE]
      result2 <- parent_census_vectors(child)

      expect_true("v_TEST_1" %in% result2$vector, info = "Should include parent v_TEST_1")
      expect_false("v_TEST_2" %in% result2$vector, info = "Should not include itself")
      expect_equal(nrow(result2), 1, info = "Should return exactly 1 parent")
    }
  )
})

test_that("child_census_vectors works with mock data", {
  skip_if_not_installed("dplyr")

  # Create mock vector data structure
  mock_vectors <- data.frame(
    vector = c("v_TEST_1", "v_TEST_2", "v_TEST_3", "v_TEST_4", "v_TEST_5"),
    parent_vector = c(NA, "v_TEST_1", "v_TEST_1", "v_TEST_2", "v_TEST_3"),
    label = c("Root", "Child1", "Child2", "Grandchild1", "GreatGrandchild1"),
    details = c("", "", "", "", ""),
    aggregation = rep("Additive", 5),
    type = rep("Total", 5),
    stringsAsFactors = FALSE
  )
  attr(mock_vectors, "dataset") <- "TEST"

  with_mocked_bindings(
    list_census_vectors = function(dataset, use_cache = TRUE, quiet = TRUE) {
      mock_vectors
    },
    {
      # Test with root node - should return all children
      root_vector <- mock_vectors[1, , drop = FALSE]
      result <- child_census_vectors(root_vector)

      # Should include all descendants
      expect_true("v_TEST_2" %in% result$vector)
      expect_true("v_TEST_3" %in% result$vector)
      expect_true("v_TEST_4" %in% result$vector)
      expect_true("v_TEST_5" %in% result$vector)
      expect_false("v_TEST_1" %in% result$vector) # Should not include itself by default

      # Test max_level parameter
      result_level1 <- child_census_vectors(root_vector, max_level = 1)
      expect_true("v_TEST_2" %in% result_level1$vector)
      expect_true("v_TEST_3" %in% result_level1$vector)
      expect_false("v_TEST_4" %in% result_level1$vector) # Should not go deeper

      # Test keep_parent parameter
      result_with_parent <- child_census_vectors(root_vector, keep_parent = TRUE)
      expect_true("v_TEST_1" %in% result_with_parent$vector)

      # Test leaves_only parameter
      result_leaves <- child_census_vectors(root_vector, leaves_only = TRUE)
      # Only v_TEST_5 has no children
      expect_true("v_TEST_5" %in% result_leaves$vector)
      expect_false("v_TEST_2" %in% result_leaves$vector) # Has children
    }
  )
})

test_that("parent_census_vectors handles empty results", {
  skip_if_not_installed("dplyr")

  mock_vectors <- data.frame(
    vector = c("v_TEST_1"),
    parent_vector = c(NA),
    label = c("Root"),
    details = c(""),
    aggregation = "Additive",
    type = "Total",
    stringsAsFactors = FALSE
  )
  attr(mock_vectors, "dataset") <- "TEST"

  with_mocked_bindings(
    list_census_vectors = function(dataset, use_cache = TRUE, quiet = TRUE) {
      mock_vectors
    },
    {
      # Root node has no parent
      root_vector <- mock_vectors[1, , drop = FALSE]
      result <- parent_census_vectors(root_vector)

      # Should return empty data frame with correct structure
      expect_equal(nrow(result), 0)
      expect_true("vector" %in% names(result))
      expect_true("parent_vector" %in% names(result))
    }
  )
})

test_that("child_census_vectors handles empty results", {
  skip_if_not_installed("dplyr")

  mock_vectors <- data.frame(
    vector = c("v_TEST_1"),
    parent_vector = c(NA),
    label = c("Leaf"),
    details = c(""),
    aggregation = "Additive",
    type = "Total",
    stringsAsFactors = FALSE
  )
  attr(mock_vectors, "dataset") <- "TEST"

  with_mocked_bindings(
    list_census_vectors = function(dataset, use_cache = TRUE, quiet = TRUE) {
      mock_vectors
    },
    {
      # Leaf node has no children
      leaf_vector <- mock_vectors[1, , drop = FALSE]
      result <- child_census_vectors(leaf_vector)

      # Should return empty data frame with correct structure
      expect_equal(nrow(result), 0)
      expect_true("vector" %in% names(result))
      expect_true("parent_vector" %in% names(result))
    }
  )
})

test_that("parent_census_vectors handles character vector input", {
  skip_if_not_installed("dplyr")

  mock_vectors <- data.frame(
    vector = c("v_TEST_1", "v_TEST_2"),
    parent_vector = c(NA, "v_TEST_1"),
    label = c("Root", "Child"),
    details = c("", ""),
    aggregation = rep("Additive", 2),
    type = rep("Total", 2),
    stringsAsFactors = FALSE
  )
  attr(mock_vectors, "dataset") <- "TEST"

  with_mocked_bindings(
    list_census_vectors = function(dataset, use_cache = TRUE, quiet = TRUE) {
      mock_vectors
    },
    dataset_from_vector_list = function(x) {
      "TEST"
    },
    clean_vector_list = function(x, dataset = NULL) {
      if (is.character(x)) {
        mock_vectors[mock_vectors$vector %in% x, , drop = FALSE]
      } else {
        x
      }
    },
    {
      # Test with character vector
      result <- parent_census_vectors("v_TEST_2")

      # Should include v_TEST_1
      expect_true("v_TEST_1" %in% result$vector)
    }
  )
})

test_that("child_census_vectors handles character vector input", {
  skip_if_not_installed("dplyr")

  mock_vectors <- data.frame(
    vector = c("v_TEST_1", "v_TEST_2"),
    parent_vector = c(NA, "v_TEST_1"),
    label = c("Root", "Child"),
    details = c("", ""),
    aggregation = rep("Additive", 2),
    type = rep("Total", 2),
    stringsAsFactors = FALSE
  )
  attr(mock_vectors, "dataset") <- "TEST"

  with_mocked_bindings(
    list_census_vectors = function(dataset, use_cache = TRUE, quiet = TRUE) {
      mock_vectors
    },
    dataset_from_vector_list = function(x) {
      "TEST"
    },
    clean_vector_list = function(x) {
      if (is.character(x)) {
        result <- mock_vectors[mock_vectors$vector %in% x, , drop = FALSE]
        attr(result, "dataset") <- "TEST"
        result
      } else {
        x
      }
    },
    {
      # Test with character vector
      result <- child_census_vectors("v_TEST_1")

      # Should include v_TEST_2
      expect_true("v_TEST_2" %in% result$vector)
    }
  )
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE, 
  warning = FALSE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)

## ----setup--------------------------------------------------------------------
library(cancensus)
library(dplyr)
library(ggplot2)
library(sf)

## -----------------------------------------------------------------------------
cov_station_buffers <- COV_SKYTRAIN_STATIONS %>%
  st_set_crs(4326)  # needed for Ubuntu or systems with old GDAL but can otherwise be ignored

## -----------------------------------------------------------------------------
station_city_ids <- get_intersecting_geometries("CA16", level = "CSD", geometry = cov_station_buffers,
                                                quiet=TRUE)
station_ct_ids <- get_intersecting_geometries("CA16", level = "CT", geometry = cov_station_buffers,
                                              quiet=TRUE)

## -----------------------------------------------------------------------------
variables <- c(mode_base="v_CA16_5792",transit="v_CA16_5801",walk="v_CA16_5804")
station_city <-  get_census("CA16", regions = station_city_ids, vectors = variables, 
                            geo_format = 'sf', quiet=TRUE) %>% 
  filter(name == "Vancouver (CY)")
station_cts <-   get_census("CA16", regions = station_ct_ids, vectors = variables, 
                            geo_format = 'sf', quiet=TRUE)

## ----fig.alt="CTs within City of Vancouver skytrain station catchments"-------
ggplot(station_city) +
  geom_sf(fill=NA) +
  geom_sf(data=station_cts,aes(fill=((walk+transit)/mode_base))) +
  geom_sf(data=cov_station_buffers,fill=NA,alpha=0.5,color="steelblue") +
  scale_fill_viridis_c(option="magma",labels=scales::percent) +
  coord_sf(datum=NA) +
  labs(title="CTs within City of Vancouver skytrain station catchments",
       fill="Walk or\ntransit\nto work",
       caption="StatCan Census 2016")

## ----fig.alt="DAs within City of Vancouver skytrain station catchments"-------
station_das <-   get_intersecting_geometries("CA16", level = "DA", geometry = cov_station_buffers,
                                             quiet=TRUE) %>% 
  get_census("CA16", regions = ., vectors=variables, geo_format = 'sf', quiet=TRUE)

ggplot(station_city) +
  geom_sf(fill=NA) +
  geom_sf(data=station_das,aes(fill=((walk+transit)/mode_base))) +
  geom_sf(data=cov_station_buffers,fill=NA,alpha=0.5,color="steelblue") +
  scale_fill_viridis_c(option="magma",labels=scales::percent) +
  coord_sf(datum=NA) +
  labs(title="DAs within City of Vancouver skytrain station catchments",
       fill="Walk or\ntransit\nto work",
       caption="StatCan Census 2016")


## ----fig.alt="DAs within City of Vancouver skytrain station catchments (alternative method)"----
station_das2 <- get_census("CA16", regions = station_ct_ids, vectors=variables, 
                           geo_format = 'sf', level="DA", quiet=TRUE) %>%
  sf::st_filter(cov_station_buffers)

ggplot(station_city) +
  geom_sf(fill=NA) +
  geom_sf(data=station_das2,aes(fill=((walk+transit)/mode_base))) +
  geom_sf(data=cov_station_buffers,fill=NA,alpha=0.5,color="steelblue") +
  scale_fill_viridis_c(option="magma",labels=scales::percent) +
  coord_sf(datum=NA) +
  labs(title="DAs within City of Vancouver skytrain station catchments",
       fill="Walk or\ntransit\nto work",
       caption="StatCan Census 2016")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE, 
  warning = FALSE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)

## ----setup--------------------------------------------------------------------
library(cancensus)
library(dplyr)
library(tidyr)
library(ggplot2)

## -----------------------------------------------------------------------------
fed_version = "1.3"

## -----------------------------------------------------------------------------
metadata <- get_statcan_wds_metadata("2021","FED",version=fed_version)

characteristics <- metadata |> 
  filter(`Codelist en`=="Characteristic") |>
  mutate(across(c(ID,`Parent ID`),as.integer))

ethnic_base <- characteristics |> 
  filter(grepl("Total - Ethnic",en))
ukranian <- characteristics |> 
  filter(grepl("Ukrainian",en), `Parent ID`==ethnic_base$ID) 

selected_characteristics <- bind_rows(ethnic_base,ukranian)

selected_characteristics |> select(ID,en)

## -----------------------------------------------------------------------------
dguids <- metadata |>
  filter(`Codelist ID`=="CL_GEO_FED") |>
  pull(ID)

## -----------------------------------------------------------------------------
data <- get_statcan_wds_data(dguids,members=selected_characteristics$ID,
                             gender="Total",version=fed_version)

## -----------------------------------------------------------------------------
plot_data <- data |> 
  select(DGUID=GEO_DESC,Name=GEO_NAME,name=CHARACTERISTIC_NAME,value=OBS_VALUE) |>
  pivot_wider() |>
  mutate(Share=Ukrainian/`Total - Ethnic or cultural origin for the population in private households - 25% sample data`)

## ----fig.height=4.5, fig.width=6, fig.alt="Ukrainian ethnic origin"-----------
plot_data |> slice_max(Share,n=20) |>
  ggplot(aes(y=reorder(Name,Share),x=Share)) +
  geom_bar(stat="identity",fill="steelblue") +
  scale_x_continuous(labels=scales::percent) +
  labs(title="Ukrainian ethnic origin",
       y="Federal electoral district (2013 Represenation Order)",
       x="Share of population in private households",
       caption="StatCan Census 2021")

## -----------------------------------------------------------------------------
fed_geos <- get_statcan_geographies("2021","FED")

## ----fig.height=4.5, fig.width=6, fig.alt="Ukrainian ethnic origin by (2013) Federal Electoral District"----
fed_geos |>
  left_join(plot_data,by="DGUID") |>
  ggplot(aes(fill=Share)) +
  geom_sf() +
  scale_fill_viridis_c(labels=scales::percent) +
  coord_sf(datum=NA) +
  labs(title="Ukrainian ethnic origin by (2013) Federal Electoral District",
       fill="Share of\npopulation",
       caption="StatCan Census 2021")

#' Access to Canadian census data through the CensusMapper API
#'
#' This function allows convenient access to Canadian census data and boundary
#' files through the CensusMapper API. An API key is required to retrieve data.
#'
#' For help selecting regions and vectors, see \code{\link{list_census_regions}}
#' and \code{\link{list_census_vectors}}, or check out the interactive selection
#' tool at \url{https://censusmapper.ca/api} by calling \code{explore_census_vectors()}
#'
#' @param dataset A CensusMapper dataset identifier.
#' @param regions A named list of census regions to retrieve. Names must be valid census aggregation levels.
#' @param level The census aggregation level to retrieve, defaults to \code{"Regions"}. One of \code{"Regions"}, \code{"PR"}, \code{"CMA"}, \code{"CD"}, \code{"CSD"}, \code{"CT"}, \code{"DA"}, \code{"EA"} (for 1996), or \code{"DB"} (for 2001-2016).
#' @param vectors An R vector containing the CensusMapper variable names of the census variables to download. If no vectors are specified only geographic data will get downloaded.
#' @param geo_format By default is set to \code{NA} and appends no geographic information. To include geographic information with census data, specify one of either \code{"sf"} to return an \code{\link[sf]{sf}} object (requires the \code{sf} package) or \code{"sp"} to return a \code{\link[sp]{SpatialPolygonsDataFrame-class}} object (requires the \code{rgdal} package). If user requests geo-spatial data and neither package is available, a context menu will prompt to install the \code{sf} package.
#' @param resolution Resolution of the geographic data. By default simplified geometries will be download. For lower level geometries like DB or DA this will be very close to the high resolution data.
#' Simplification generally increases as the geographic aggregation level increases.
#' If high resolution geometries are required
#' then this option can be set to 'high'. By default this setting is set to \code{'simplified'}.
#' @param labels Set to "detailed" by default, but truncated Census variable names can be selected by setting labels = "short". Use \code{label_vectors(...)} to return variable label information in detail.
#' @param use_cache If set to TRUE (the default) data will be read from the local cache if available.
#' @param quiet When TRUE, suppress messages and warnings.
#' @param api_key An API key for the CensusMapper API. Defaults to \code{options()} and then the \code{CM_API_KEY} environment variable.
#' @param retry Integer If greater than zero, automatically retry failed API requests with exponential backoff for specified maximum number of times. Defaults to 0.
#'
#' @source Census data and boundary geographies are reproduced and distributed on
#' an "as is" basis with the permission of Statistics Canada (Statistics Canada 1996; 2001; 2006; 2011; 2016).
#'
#' @export
#'
#' @examples
#' # Query the API for data on dwellings in Vancouver, at the census subdivision
#' # level:
#' \dontrun{
#' census_data <- get_census(dataset='CA16', regions=list(CMA="59933"),
#'                           vectors=c("v_CA16_408","v_CA16_409","v_CA16_410"),
#'                           level='CSD')
#'
#' # Query the API for data on dwellings in Vancouver, at the census subdivision
#' # level, and return the associated geography files in \code{sf} format:
#' census_data <- get_census(dataset='CA16', regions=list(CMA="59933"),
#'                           vectors=c("v_CA16_408","v_CA16_409","v_CA16_410"),
#'                           level='CSD', geo_format = "sf")
#'
#' # Make the same query, but this time drop descriptive vector names:
#' census_data <- get_census(dataset='CA16', regions=list(CMA="59933"),
#'                           vectors=c("v_CA16_408","v_CA16_409","v_CA16_410"),
#'                           level='CSD', geo_format = "sf", labels="short")
#'
#' # Get details for truncated vectors:
#' label_vectors(census_data)
#'}
get_census <- function (dataset, regions, level=NA, vectors=c(), geo_format = NA,
                        resolution = 'simplified',
                        labels = "detailed",
                        use_cache=TRUE, quiet=FALSE, api_key=Sys.getenv("CM_API_KEY"),
                        retry=0) {

  # API and data recall checks
  first_run_checks()
  api_key <- robust_api_key(api_key)
  have_api_key <- valid_api_key(api_key)
  result <- NULL
  data_version<-NULL
  geo_version<-NULL

  dataset <- translate_dataset(dataset)

  # Check region selection validity
  if (is.na(level)) level="Regions"

  # Check spatial resolution
  if (is.na(geo_format) && !(resolution %in% c("simplified","high"))) stop("The resolution paramerter needs to be either 'simplified' or 'high'.")

  # Turn the region list into a valid JSON dictionary.
  if (is.character(regions)) {
    if (!quiet) warning(paste("Passing `regions` as a character vector is",
                              "depreciated, and will be removed in future",
                              "versions"))
  } else if (is.null(names(regions)) || !all(names(regions) %in% VALID_LEVELS)) {
    stop("Regions must be composed of valid census aggregation levels.")
  } else {
    regions <- jsonlite::toJSON(lapply(regions,as.character)) # cast to character in case regions are supplied as numeric/interger
  }

  # Check if the aggregation level is valid.
  if (!level %in% VALID_LEVELS) {
    stop("the `level` parameter must be one of 'Regions', 'PR', 'CMA', 'CD', 'CSD', 'CT', 'DA', 'EA' or 'DB'")
  }

  # Check that we can read the requested geo format.
  if (is.na(geo_format)) { # This is ok.
  } else if (!geo_format %in% c("sf", "sp")) {
    stop("the `geo_format` parameter must be one of 'sf', 'sp', or NA")
  } else if (!is.na(geo_format) && !requireNamespace("sf", quietly = TRUE)) {
    stop("The `sf` package is required to return geographies.")
  }

  # --------- Spatial format checks --------------------------------------------------------------------#
  # This section checks that proper spatial formats are requested. If users select spatial data and
  # don't have the 'sf' package installed, will prompt them with a menu to install it, otherwise we will
  # return spatial data only. If users select 'sp' format, will advise them that usage is deprecated and nudge
  # to install 'sf' package.
  if (!is.na(geo_format)) {
    if(!geo_format %in% c("sf","sp")) {
      stop("the `geo_format` parameter should be 'sf', 'sp', or NA")
    } else if(geo_format == "sf" && !("sf" %in% utils::installed.packages())) {
      if (utils::menu(c("Install package", "Return tabular data without geo"),
                      title= paste("The `sf` package is required to return geographies. Would you like to install?")) == "1") {
        utils::install.packages('sf')
      } else  {
        message("Retrieving tabular data only. Please install 'sf' package if you wish to use Census data as spatial data.")
        geo_format <- NA
      }
    } else if(geo_format == "sp" && !("sf" %in% utils::installed.packages())) {
      message("The use of 'sp' format in cancensus package is now deprecated.\nPlease install 'sf' package to return spatial format data.")
      if (utils::menu(c("Install package", "Return tabular data without geo"),
                      title= paste("Would you like to install 'sf' to continue?")) == "1") {
        utils::install.packages('sf')
      } else  {
        message("Retrieving tabular data only. Please install 'sf' package if you wish to use Census data as spatial data.")
        geo_format <- NA
      }
    }
  }

  base_url=paste0(cancensus_base_url(),"/api/v1/")
  # load data variables
  if (length(vectors)>0 || is.na(geo_format)) {
    params <- list(regions=as.character(regions),
                   vectors=jsonlite::toJSON(vectors),
                   level=level,
                   dataset=dataset,
                   api_key=api_key)
    param_string <- paste0("regions=", as.character(regions),
                           # Convert vectors to a JSON list.
                           "&vectors=", jsonlite::toJSON(as.character(vectors)),
                           "&level=", level, "&dataset=", dataset)
    if (is.na(geo_format)) param_string=paste0(param_string,"&geo_hierarchy=true")
    if (is.na(geo_format)) params["geo_hierarchy"]="true"
    data_base_name <- paste0("CM_data_",digest::digest(param_string, algo = "md5"))
    data_file <- cache_path(data_base_name, ".rda")
    meta_file <- paste0(data_file, ".meta")
    if (!use_cache || !file.exists(data_file)) {
      if (!have_api_key) {
        stop(paste("No API key set. Use set_cancensus_api_key('<your API key>') to set one, or set_cancensus_api_key('<your API key>', install = TRUE) to save it permanently in your .Renviron."))
      }
      url <- paste0(base_url, "data.csv")
      if (!quiet) message("Querying CensusMapper API for data...")

      # Define the API call
      make_data_call <- function() {
        if (!quiet) {
          httr::POST(url, httr::progress(), body=params)
        } else {
          httr::POST(url, body=params)
        }
      }

      # Execute with or without retry
      response <- if (retry>0) {
        retry_api_call(make_data_call, max_retries = retry+1, quiet = quiet)
      } else {
        make_data_call()
      }

      handle_cm_status_code(response, NULL)

      # Report download size
      content_length <- as.numeric(response$headers$`content-length`)
      if (!quiet && length(content_length)>0 && !is.na(content_length) && content_length > 0) {
        message(sprintf("Downloaded %s of data.", format_bytes(content_length)))
      }

      data_version <- response$headers$`data-version`


      # Read the data file and transform to proper data types
      result <- if (requireNamespace("readr", quietly = TRUE)) {
        # Use readr::read_csv if it's available.
        httr::content(response, type = "text", encoding = "UTF-8") %>%
          readr::read_csv(na = cancensus_na_strings,
                          col_types = list(.default = "c"))
      } else {
        check_recalled_data_and_warn(meta_file,params)
        httr::content(response, type = "text", encoding = "UTF-8") %>%
          textConnection %>%
          utils::read.csv(colClasses = "character", stringsAsFactors = FALSE, check.names = FALSE) %>%
          dplyr::as_tibble(.name_repair = "minimal")
      }

      result <- result %>%
        dplyr::mutate_at(c(dplyr::intersect(names(.),c("Population","Households","Dwellings","Area (sq km)")),
                           names(.)[grepl("^v_",names(.))]), as.num) %>%
        dplyr::mutate(Type = as.factor(.data$Type),
                      `Region Name` = as.factor(.data$`Region Name`))

      if (is.na(geo_format)) result <- result %>% transform_geo(level)
      attr(result, "last_updated") <- Sys.time()
      save(result, file = data_file)
      file_info <- file.info(data_file)
      metadata <- dplyr::tibble(dataset=dataset,regions=as.character(regions),
                                 level=level,
                                 vectors=jsonlite::toJSON(as.character(vectors))  %>% as.character(),
                                 created_at=Sys.time(),
                         version=data_version,size=file_info$size)
      saveRDS(metadata, file = meta_file)
    } else {
      if (!quiet) message("Reading vectors data from local cache.")
      # Load `result` object from cache.
      load(file = data_file)
    }
    touch_metadata(meta_file,params)
  }

  if (!is.na(geo_format)) {
    params <- list(regions=regions,
                   level=level,
                   dataset=dataset,
                   resolution=resolution,
                   api_key=api_key)
    param_string <- paste0("regions=", regions,
                           "&level=", level,
                           "&dataset=", dataset)
    if (resolution !="simplified") param_string <- paste0(param_string,"&resolution=",resolution)
    geo_base_name <- paste0("CM_geo_", digest::digest(param_string, algo = "md5"))
    geo_file <- cache_path(geo_base_name, ".geojson")
    meta_file <- paste0(geo_file, ".meta")
    if (!use_cache || !file.exists(geo_file)) {
      if (!have_api_key) {
        stop(paste("No API key set. Use set_cancensus_api_key('<your API key>') to set one, or set_cancensus_api_key('<your API key>', install = TRUE) to save it permanently in your .Renviron."))
      }
      url <- paste0(base_url, "geo.geojson")
      if (!quiet) message("Querying CensusMapper API for geometry...")

      # Define the API call
      make_geo_call <- function() {
        if (!quiet) {
          httr::POST(url, httr::progress(), body=params)
        } else {
          httr::POST(url, body=params)
        }
      }

      # Execute with or without retry
      response <- if (retry>0) {
        retry_api_call(make_geo_call, max_retries = retry+1, quiet = quiet)
      } else {
        make_geo_call()
      }

      handle_cm_status_code(response, NULL)

      # Report download size
      content_length <- as.numeric(response$headers$`content-length`)
      if (!quiet && length(content_length)>0 && !is.na(content_length) && content_length > 0) {
        message(sprintf("Downloaded %s of geometry.", format_bytes(content_length)))
      }

      geo_version <- response$headers$`data-version`
      write(httr::content(response, type = "text", encoding = "UTF-8"), file = geo_file) # cache result
      file_info <- file.info(geo_file)
      metadata <- dplyr::tibble(dataset=dataset,regions=as.character(regions),
                                level=level,created_at=Sys.time(),
                                resolution=resolution,
                         version=geo_version,size=file_info$size)
      saveRDS(metadata, file = meta_file)
    } else {
      if (!quiet) message("Reading geo data from local cache.")
      check_recalled_data_and_warn(meta_file,params)
    }
    geos <- geojsonsf::geojson_sf(geo_file) %>%
      sf::st_sf(agr="constant") %>% # need to set this, otherwise sf complains
       transform_geo(level) #%>%
      # sf::st_sf(agr="constant") # just in case

    if (!quiet) {
      message(sprintf("Processing %s geographic features.", format(nrow(geos), big.mark = ",")))
    }

    result <- if (is.null(result)) {
      geos
    } else if (!is.na(geo_format)) {
      # the sf object needs to be first in join to retain all spatial information
      to_remove <- setdiff(dplyr::intersect(names(geos),names(result)),"GeoUID")
      dplyr::select(result, -dplyr::one_of(to_remove)) %>%
        dplyr::inner_join(geos, ., by = "GeoUID")
    }
    touch_metadata(meta_file,params)
  }

  # ensure sf format even if library not loaded and set agr columns
  if (!is.na(geo_format) & geo_format=='sf') {
    numerics <- result %>% dplyr::select_if(function(d)is.numeric(d)|is.integer(d)) %>%
      names()
    non_numerics <- setdiff(names(result),c(numerics,"geometry"))
    agr_cols <- c(setNames(rep_len("aggregate",length(numerics)),numerics),
                  setNames(rep_len("identity",length(non_numerics)),non_numerics))
    result <- result %>%
      #sf::st_sf(agr=agr_cols) # something wrong here, does not work for `Region Name` column. maybe bug in sf?
      sf::st_sf(agr="constant")
  }


  if (length(vectors)>0) {
    census_vectors <- names(result)[grep("^v_", names(result))] %>%
      strsplit(., ": ") %>%
      dplyr::as_tibble(x=do.call(rbind, .),.name_repair = "minimal") %>%
      setNames(c("Vector", "Detail"))
    attr(result, "census_vectors") <- census_vectors
    if (labels == "short" | !is.null(names(vectors))) {
      to_rename <- setNames(names(result),gsub(":.*","",names(result)))
      to_rename <- to_rename[names(to_rename)!=as.character(to_rename)]
      if (length(to_rename)>0) result <- result %>% dplyr::rename(!!!to_rename)
      if (!is.null(names(vectors))) {
        to_rename <- vectors[as.character(vectors) %in% names(result)]
        if (length(to_rename)>0) result <- result %>% dplyr::rename(!!! to_rename)
      }
    }
  }

  if (!is.na(geo_format) & geo_format=='sp') { # ensure sf format even if library not loaded
    result <- sf::as_Spatial(result)
  }

  return(result)
}

#' Deprecated, use `get_census` instead
#'
#' @description
#' This function will be removed in future versions.
#'
#' @param dataset A CensusMapper dataset identifier.
#' @param regions A named list of census regions to retrieve. Names must be valid census aggregation levels.
#' @param level The census aggregation level to retrieve, defaults to \code{"Regions"}. One of \code{"Regions"}, \code{"PR"}, \code{"CMA"}, \code{"CD"}, \code{"CSD"}, \code{"CT"} or \code{"DA"}.
#' @param geo_format By default is set to \code{NA} and appends no geographic information. To include geographic information with census data, specify one of either \code{"sf"} to return an \code{\link[sf]{sf}} object (requires the \code{sf} package) or \code{"sp"} to return a \code{\link[sp]{SpatialPolygonsDataFrame-class}} object (requires the \code{rgdal} package).
#' @param ... Further arguments passed to \code{get_census}.
#'
#' @source Census data and boundary geographies are reproduced and distributed on
#' an "as is" basis with the permission of Statistics Canada (Statistics Canada
#' 2006; 2011; 2016).
#'
#'
#' @examples
#' # Query the API for data geographies in Vancouver, at the census subdivision
#' # level:
#' \dontrun{
#' # Query the API for geographies in Vancouver, at the census subdivision
#' # level, and return the associated geography files in \code{sf} format:
#' census_data <- get_census(dataset='CA16', regions=list(CMA="59933"),
#'                           level='CSD', geo_format = "sf")
#'}
#' @export
#' @keywords internal
get_census_geometry <- function (dataset, regions, level=NA, geo_format = "sf", ...) {
  .Deprecated("get_census")
  stop("This function is no longer supported.")
  #return(get_census(dataset=dataset, regions=regions, level=level, geo_format=geo_format, ...))
}

# This is the set of valid census aggregation levels, also used in the named
# elements of the `regions` parameter.
VALID_LEVELS <- c("Regions","C","PR", "CMA", "CD", "CSD", "ADA","CT", "DA", 'EA', "DB")

#' Query the CensusMapper API for available datasets.
#'
#' @param use_cache If set to TRUE (the default), data will be read from a temporary local cache for the
#'   duration of the R session, if
#'   available. If set to FALSE, query the API for the data, and
#'   refresh the temporary cache with the result.
#' @param quiet When TRUE, suppress messages and warnings.
#'
#' @return
#'
#' Returns a data frame with a column \code{dataset} containing the code for the
#' dataset, a column \code{description} describing it, a \code{geo_dataset} column
#' identifying the geography dataset the data is based on, a \code{attribution} column
#' with an attribution string, a \code{reference} column with a reference identifier, and
#' a \code{reference_url} column with a link to reference materials.
#'
#' @export
#'
#' @examples
#'
#' # List available datasets in CensusMapper
#' \dontrun{
#' list_census_datasets()
#' }
list_census_datasets <- function(use_cache = TRUE, quiet = FALSE) {
  cache_file <- file.path(tempdir(),"cancensus_datasets.rda") #cache_path("datasets.rda")
  if (!use_cache || !file.exists(cache_file)) {
    if (!quiet) message("Querying CensusMapper API for available datasets...")
    response <- httr::GET("https://censusmapper.ca/api/v1/list_datasets",
                          httr::accept_json())
    handle_cm_status_code(response, NULL)
    result <- httr::content(response, type = "text", encoding = "UTF-8") %>%
      jsonlite::fromJSON() %>%
      dplyr::as_tibble(.name_repair = "minimal")
    attr(result, "last_updated") <- Sys.time()
    save(result, file = cache_file)
    result
  } else {
    if (!quiet) message("Reading dataset list from temporary cache.")
    load(file = cache_file)
    last_updated <- attr(result, "last_updated")
    if (!quiet && is.null(last_updated) ||
          difftime(Sys.time(), last_updated, units = "days") > 1) {
      warning(paste("Cached dataset list may be out of date. Set `use_cache =",
                    "FALSE` to update it."))
    }
    result
  }
}

#' Get attribution for datasets
#'
#' @param datasets Vector of dataset identifiers
#'
#' @return
#'
#' Returns a string to be used as attribution for the given datasets.
#'
#' @export
#'
#' @examples
#'
#' \dontrun{
#' # Attribution string for the 2006 and 2016 census datasets
#' dataset_attribution(c('CA06','CA16'))
#' }
dataset_attribution <- function(datasets){
  datasets <-   lapply(datasets,translate_dataset) %>% unlist()
  attribution <-list_census_datasets(quiet=TRUE) %>%
    dplyr::filter(.data$dataset %in% datasets) %>%
    dplyr::pull(.data$attribution)

  commons <- gsub("\\d{4}","\\\\\\d{4}",attribution) %>%
    unique()

  commons %>% lapply(function(c){
    matches <- attribution[grepl(paste0("^",c,"$"),attribution)]
    parts <- strsplit(c, split = "\\\\d\\{4\\}") %>%
      unlist()
    years <- matches
    for (p in parts){
      years <- gsub(p,"",years)
    }

    year_string <- paste0(years,collapse=", ")
    gsub("\\d{4}",paste0(years,collapse=", "),matches[[1]])
  }) %>%
    unlist() %>%
    paste0(collapse="; ")
}

#' Return Census variable names and labels as a tidy data frame
#'
#' @param x A data frame, \code{sp} or \code{sf} object returned from
#'   \code{get_census} or similar.
#'
#' @return
#'
#' A data frame with a column \code{variable} containing the truncated
#' variable name, and a column \code{label} describing it.
#'
#'@examples
#'\dontrun{
#' # Query census data with truncated labels:
#' label_data <- get_census(dataset='CA16', regions=list(CMA="59933"),
#'                           vectors=c("v_CA16_408","v_CA16_409","v_CA16_410"),
#'                           level='CSD', geo_format = "sf", labels="short")
#'
#' # Get details for truncated vectors:
#' label_vectors(label_data)
#' }
#' @export
label_vectors <-  function(x) {
  if("census_vectors" %in% names(attributes(x))) {
    attr(x, "census_vectors")
  } else {
    warning("Data does not have variables to labels. No Census variables selected as vectors. See ?get_census() for more information. ")}
}

#' Return Census variable names and labels as a tidy data frame (Deprecated)
#'
#' @param x A data frame, \code{sp} or \code{sf} object returned from
#'   \code{get_census} or similar.
#'
#' @return
#'
#' A data frame with a column \code{variable} containing the truncated
#' variable name, and a column \code{label} describing it.
#'
#'@examples
#' \dontrun{
#' # Query census data with truncated labels:
#' census_data <- get_census(dataset='CA16', regions=list(CMA="59933"),
#'                           vectors=c("v_CA16_408","v_CA16_409","v_CA16_410"),
#'                           level='CSD', geo_format = "sf", labels="short")
#'
#' # Get details for truncated vectors:
#' census_vectors(census_data)
#' }
#' @keywords internal
#' @export
census_vectors <-  function(x) {
  warning("census_vectors() is deprecated. Please use label_vectors() to view details for truncated variable labels.")
  if("census_vectors" %in% names(attributes(x))) {
    attr(x, "census_vectors")
  } else {
    warning("Data does not have variables to labels. No Census variables selected as vectors. See ?get_census() for more information. ")}
}



# Internal function to handle unfavourable HTTP responses
handle_cm_status_code <- function(response,path){
  if (httr::status_code(response)!=200) {
    message=httr::content(response,as="text")
    if (!is.null(path)) {
      file.remove(path)
    }
    if (httr::status_code(response)==401) {
      # Problem with API key
      stop(paste("Download of Census Data failed.",
                 "Please ensure that your API key is valid and has a large enough quota left.",
                 message, sep=' '))
    } else if (httr::status_code(response)==500) {
      stop(paste("Download of Census Data failed.",
                 "The request triggered a server error, the CensusMapper maintainers have been notified and will fix this as soon as possible.",
                 message, sep=' '))
    } else {
      stop(paste("Download of Census Data failed.",
                 message, sep=' '))
    }
  }
}


name_change_for_level <- function(level){
  if (level=='DB') {
    name_change <- c('DA_UID'='rpid',
                     'CSD_UID'='rgid',
                     'CT_UID'='ruid',
                     'CMA_UID'='rguid')
  } else  if (level=='DA'|level=='EA') {
    name_change <- c('CSD_UID'='rpid',
                     'CD_UID'='rgid',
                     'CT_UID'='ruid',
                     'CMA_UID'='rguid')
  } else if (level=='CT') {
    name_change <- c('CMA_UID'='rpid',
                     'PR_UID'='rgid',
                     'CSD_UID'='ruid',
                     'CD_UID'='rguid')
  } else if (level=='CSD') {
    name_change <- c('CD_UID'='rpid',
                     'PR_UID'='rgid',
                     'CMA_UID'='ruid')
  } else if (level=='CD') {
    name_change <- c('PR_UID'='rpid',
                     'C_UID'='rgid')
  } else if (level=='CMA') {
    name_change <- c('PR_UID'='rpid',
                     'C_UID'='rgid')
  } else if (level=='PR') {
    name_change <- c('C_UID'='rpid')
  } else if (level=='C') {
    name_change <- c()
  } else {
    name_change <- c()
    warning(paste0("Unknown level ",level))
  }
  name_change
}

base_name_change <- c("GeoUID"="id",
                      "Shape Area"="a",
                      "Type"="t",
                      "Dwellings"="dw",
                      "Households"="hh",
                      "Population"="pop",
                      "Adjusted Population (previous Census)"="pop2",
                      "NHS Non-Return Rate"="nrr",
                      "Quality Flags"="q",
                      "Population 2011"="pop11",
                      "Population 2016"="pop16",
                      "Households 2011"="hh11",
                      "Households 2016"="hh16",
                      "Dwellings 2011"="dw11",
                      "Dwellings 2016"="dw16")

# Transform and rename geometry data.
transform_geo <- function(g, level) {
  as_character=c("id","rpid","rgid","ruid","rguid","q")
  as_numeric=c("a","nrr")
  as_factor=c("t")
  as_integer=c("pop","dw","hh","pop2","pop11","pop16","hh11","hh16","dw11","dw16")
  #as_character=c(as_character,as_numeric,as_integer)

  to_rename <- base_name_change[as.character(base_name_change) %in% names(g)]

  g <- g %>%
    dplyr::mutate_at(dplyr::intersect(names(g), as_character), as.character) %>%
    dplyr::mutate_at(dplyr::intersect(names(g), as_numeric), as.numeric)  %>%
    dplyr::mutate_at(dplyr::intersect(names(g), as_integer), as.int)  %>%
    dplyr::mutate_at(dplyr::intersect(names(g), as_factor), as.factor) %>%
    dplyr::rename(!!to_rename)

  if (level != "Regions") {
    rc <- name_change_for_level(level)[as.character(name_change_for_level(level)) %in% names(g)]
    if (length(rc)>0) g <- g %>%  dplyr::rename(!!!rc)
  } else if ("Type" %in% names(g)) {
    g <- g$Type %>%
      unique %>%
      lapply(function(t){
        g <- g %>%  dplyr::filter(.data$Type==t)
        rc <- name_change_for_level(t)[as.character(name_change_for_level(t)) %in% names(g)]
        if (length(rc)>0) {
          g <- g %>%
            dplyr::rename(!!!rc) %>%
            dplyr::mutate_at(names(rc),function(d)dplyr::na_if(d,""))
        }
        g
      }) %>%
      dplyr::bind_rows()
  }

  to_remove <- dplyr::intersect(c("rpid","rgid","ruid","rguid"),names(g))
  if (length(to_remove)>0) g <- g %>% dplyr::select(-dplyr::one_of(to_remove))

  return(g)
}



.onAttach <- function(libname, pkgname) {
 # if (!"cancensus.api_key" %in% names(options())) {
    # Try to get the API key from the CM_API_KEY environment variable, if it has not already been specified.
#    options(cancensus.api_key = if (nchar(Sys.getenv("CM_API_KEY")) > 1) { Sys.getenv("CM_API_KEY") } else { NULL })
#  }


  if (!("cancensus.cache_path" %in% names(options())) & nchar(Sys.getenv("CM_CACHE_PATH"))==0) {
    # Cache in tmp dir by default.
    packageStartupMessage(cm_no_cache_path_message)
  }
}

# Suppress warnings for missing bindings for '.' in R CMD check.
if (getRversion() >= "2.15.1") utils::globalVariables(c("."))

#' @importFrom dplyr %>%
#' @importFrom rlang .data
#' @importFrom stats setNames
NULL

#' Query the CensusMapper API for vectors with descriptions matching a search term or phrase (deprecated)
#'
#' @param searchterm The term or phrase to search for e.g. \code{"Ojibway"}.
#' Search terms are case insensitive. If unable to find a given string,
#' this function will suggest similarly named objects.
#' @param dataset The dataset to query for available vectors, e.g.
#'   \code{"CA16"}.
#' @param type One of \code{NA}, \code{'Total'}, \code{'Male'} or \code{'Female'}.
#' If specified, only return variables of specified `type`.
#' @param ... Further arguments passed on to \code{\link{list_census_vectors}}.
#'
#' @export
#'
#' @examples
#'\dontrun{
#' search_census_vectors('Ojibway', 'CA16')
#'
#' # This will return a warning that no match was found, but will suggest similar terms.
#' search_census_vectors('Ojibwe', 'CA16', 'Total')
#' }
search_census_vectors <- function(searchterm, dataset, type=NA, ...) {
  warning("search_census_vectors(). See ?find_census_vectors() for more robust search functions.")
  veclist <- list_census_vectors(dataset, ...)
  result <- veclist[grep(searchterm, veclist$label, ignore.case = TRUE),]

  # filter by type if needed
  if (!is.na(type) && length(rownames(result)) > 0) {
    result <- result[result$type==type,]
  }

  # Check if searchterm returned anything
  if (length(rownames(result)) > 0 ) {
    attr(result, "dataset") <- dataset
    return(result)
  }
  # If nothing matches, throw a warning and suggested alternatives.
  # If no suggested alternatives because the typo is too egregious, throw an error.
  else {
    # Check for similarly named terms. Uses base function agrep which is based on the Levenshtein edit distance for string similarity.
    # Default is set to 0.1 - can expand this to be more tolerant still.
    hintlist <- dplyr::as_tibble(unique(agrep(searchterm, veclist$label, ignore.case = TRUE, value = TRUE)))
    names(hintlist) <- "Similarly named objects"
    #
    if (length(hintlist) > 0) {
      warning("No results found. Please use accurate spelling. See above for list of variables with similar named terms.")
      print(hintlist)
    } else {
      stop("No results found.")
    }
  }
}

#' Query the CensusMapper API for vectors using exact, semantic, or keyword search
#'
#' @description Query the available list of Census vectors based on their label and return
#' details including vector code. Default search behaviour expects an exact match, but
#' keyword or semantic searches can be used instead by setting \code{query_type='keyword'} or
#' \code{query_type = 'semantic'} instead. Keyword search is useful when looking to explore
#' Census vectors based on broad themes like "income" or "language". Keyword search separates
#' the query into unigrams and returns Census vectors with matching words, ranked by incidence
#' of matches. Semantic search is designed for more precise searches while allowing room for error
#' for spelling or phrasing, as well as for finding closely related vector matches. Semantic search
#' separates the query into n-grams and relies on string distance measurement using a generalized
#' Levenshtein distance approach.
#'
#' Some census vectors return population counts segmented by \code{Female} and \code{Male} populations, in
#' addition to a total aggregate. By default, query matches will return matches for the \code{Total}
#' aggregation, but can optionally return only the \code{Female} or \code{Male} aggregations by adding
#' \code{type = 'female'} or \code{type = 'male'} as a parameter.
#'
#' @param query The term or phrase to search for e.g. \code{'Oji-cree'}.
#' Search queries are case insensitive.
#' @param dataset The dataset to query for available vectors, e.g. \code{'CA16'}.
#' To see a list of available datasets: \code{list_census_datasets()}
#' @param type One of \code{'all'}, \code{'total'}, \code{'male'} or \code{'female'}.
#' If specified, only return aggregations of specified `type`. By default, only
#' the \code{'total'} aggregation will be returned.
#' @param query_type One of \code{exact}, \code{'semantic'} or \code{'keyword'}.
#' By default, assumes exact string matching, but the alternatives may be better
#' options in some cases. See description section for more details on query types.
#' @param ... Other arguments passed to internal functions.
#'
#' @export
#'
#' @examples
#'\dontrun{
#' find_census_vectors('Oji-cree', dataset = 'CA16', type = 'total', query_type = 'exact')
#'
#' find_census_vectors('commuting duration', dataset = 'CA11', type = 'female', query_type = 'keyword')
#'
#' find_census_vectors('after tax income', dataset = 'CA16', type = 'total', query_type = 'semantic')
#'
#' # This incorrect spelling will return a warning that no match was found,
#' # but will suggest trying semantic or keyword search.
#' find_census_vectors('Ojibwey', dataset = 'CA16', type = 'total')
#'
#' # This will find near matches as well
#' find_census_vectors('Ojibwey', dataset = 'CA16', type = 'total', query_type = "semantic")
#'
#' find_census_vectors('commute duration', dataset = 'CA16', type = 'female', query_type = 'keyword')
#'
#' find_census_vectors('commute duration', dataset = 'CA11', type = 'all', query_type = 'keyword')
#'
#' find_census_vectors('ukrainian origin', dataset = 'CA16', type = 'total', query_type = 'keyword')
#'
#' }
find_census_vectors <- function(query, dataset, type = "all", query_type = "exact", ...) {
  type = tolower(type)
  query_type = tolower(query_type)

  if(!(any(type %in% c("total","male","female","all")))) {
    stop("Type must be one of 'total','female', or 'male'. See ?find_census_vectors() for more details.",
         call. = FALSE)
  }

  if(!(any(query_type %in% c("exact","semantic","keyword")))) {
    stop("Query type must be one of 'exact','semantic', or 'keyword'. See ?find_census_vectors() for more details.",
    call. = FALSE)
  }
  type = tools::toTitleCase(type)
  census_vector_list <- list_census_vectors(dataset)[, c("vector", "type", "label","details")]
  census_vector_list$details <- gsub("^(.*)Census; |100% data; ","",census_vector_list$details)
  # filter by type if provided
  if (any(type %in% c("Total", "Male", "Female")))
  {
    census_vector_list <- census_vector_list[census_vector_list$type %in% type, ]
  }
  if (query_type == "exact") {
    result <- census_vector_list[grep(query, census_vector_list$details, ignore.case = TRUE), ]
    if(length(result$vector)>=1) result else {
      warning("No exact matches found. Please check spelling and try again or consider using semantic or keyword search.\nSee ?find_census_vectors() for more details.\n\nAlternatively, you can launch the Censusmapper web API in a browser by calling explore_census_vectors(dataset)",
              call. = FALSE)
    }
  } else if (query_type == "semantic") {
    semantic_search(query, census_vector_list = census_vector_list)

  } else if (query_type == "keyword") {
    keyword_search(query, census_vector_list = census_vector_list, ...)
  }
}

#' Internal function for powering semantic searches in find_census_vectors
#'
#' @description Default search behaviour in \code{find_census_vectors()} expects an exact match, but
#' keyword or semantic searches can be used instead by setting \code{query_type="keyword"} or
#' \code{query_type = "semantic"} instead. Keyword search is useful when looking to explore
#' Census vectors based on broad themes like "income" or "language". Keyword search seperates
#' the query into unigrams and returns Census vectors with matching words, ranked by incidence
#' of matches. Semantic search is designed for more precise searches while allowing room for error
#' for spelling or phrasing, as well as for finding closely related vector matches. Semantic search
#' separates the query into n-grams and relies on string distance measurement using a generalized
#' Levenshtein distance approach.
#'
#' @keywords Internal
#'
#' @noRd
#'
#' @noMd
semantic_search <- function(query_terms, census_vector_list) {
  sample_vector_list <- census_vector_list$details
  clean_vector_list <- gsub("\\s+"," ",gsub("[[:punct:]]"," ",sample_vector_list))
  query_words <- unlist(strsplit(tolower(query_terms), "[^a-z]+"))
  word_count <- length(query_words)
  sentence_word_split <- strsplit(tolower(clean_vector_list), "\\s+")

  # Optimized n-gram generation: use vectorized operations where possible
  n_grams <- lapply(sentence_word_split, function(words) {
    n <- length(words)
    if (n == 0) return(character(0))
    if (word_count == 1) return(words)
    if (n < word_count) {
      return(paste(words, collapse = " "))
    }
    # Pre-allocate result vector for efficiency
    result <- character(n)
    for (i in seq_len(n)) {
      end_idx <- min(n, i + word_count - 1)
      result[i] <- paste(words[i:end_idx], collapse = " ")
    }
    return(result)
  })

  ordered_ngram_count <- trimws(names(sort(table(unlist(n_grams)), decreasing = TRUE)), "both")
  revised_query <- c(query_terms, unlist(strsplit(query_terms, "\\s+")))
  lev_dist_df <- setNames(data.frame(sapply(seq_along(revised_query),
                                            function(i){
                                              utils::adist(revised_query[i], ordered_ngram_count, ignore.case = TRUE)
                                            }
  )), gsub("\\s+", "_", revised_query))

  if(min(lev_dist_df) > 2 | is.infinite(min(lev_dist_df))) {
    warning(
      "No close matches found. Please check spelling and try again or consider using keyword search instead.\nSee ?find_census_vectors() for more details.\n\nAlternatively, you can launch the Censusmapper web API in a browser by calling explore_census_vectors(dataset)",
      call. = FALSE
    )} else {
      res <- sample_vector_list[grep(ordered_ngram_count[sapply(seq_along(ncol(lev_dist_df)),
                                                                function(i) {
                                                                  which.min(lev_dist_df[, i])
                                                                })], clean_vector_list, value = FALSE, ignore.case = TRUE)]

      if(length(res) == 1) {census_vector_list[which(census_vector_list$details %in% res),]} else if(length(res) >1) {
        message("Multiple possible matches. Results ordered by closeness.")
        census_vector_list[which(census_vector_list$details %in% res),]
      }
    }
}

#' Internal function for powering keyword searches in find_census_vectors
#'
#' @description Default search behaviour in \code{find_census_vectors()} expects an exact match, but
#' keyword or semantic searches can be used instead by setting \code{query_type="keyword"} or
#' \code{query_type = "semantic"} instead. Keyword search is useful when looking to explore
#' Census vectors based on broad themes like "income" or "language". Keyword search seperates
#' the query into unigrams and returns Census vectors with matching words, ranked by incidence
#' of matches. Semantic search is designed for more precise searches while allowing room for error
#' for spelling or phrasing, as well as for finding closely related vector matches. Semantic search
#' separates the query into n-grams and relies on string distance measurement using a generalized
#' Levenshtein distance approach.
#'
#' @param interactive Set to \code{TRUE} by default, determines how keyword search works.
#' Keyword search will return the best matching results; however, if there are additional
#' matching keywords it will prompt the user with an option to show the additional results.
#' This can be manually overridden by setting \code{interactive=FALSE}, which will result in
#' only the top matched results showing up, which might be useful if this function is used in
#' automated scripts.
#'
#' @keywords Internal
#' @noRd
#' @noMd
keyword_search <- function(query_terms, census_vector_list, interactive = TRUE) {
  sample_vector_list <- census_vector_list$details
  vector_words <- strsplit(gsub("\\s+"," ",gsub("[[:punct:]]"," ",tolower(sample_vector_list))), split = " ")
  clean_vector_list <- lapply(vector_words, function(x) paste(unique(x), collapse = " "))

  query_words <- paste(unlist(strsplit(tolower(query_terms), "[^a-z]+")), collapse = "|")
  index_matches <- grep(query_words, clean_vector_list, ignore.case = TRUE)

  ret_matches <- clean_vector_list[index_matches]

  if (length(ret_matches) == 0) {
    warning(
      "No matches found. Please check spelling and try again or consider using semantic search instead.\nSee ?find_census_vectors() for more details.\n\nAlternatively, you can launch the Censusmapper web API in a browser by calling explore_census_vectors(dataset)",
      call. = FALSE
    )
  } else {
    res <- regmatches(ret_matches, gregexpr(paste0("\\<",gsub("\\|","\\\\>|\\\\<",query_words),"\\>"), ret_matches, ignore.case = TRUE))
    res_df <-
      data.frame(results = sample_vector_list[index_matches], n_match = lengths(res))

    top_res <-
      census_vector_list[which(census_vector_list$details %in% res_df[which(res_df$n_match == max(res_df$n_match)), ]$results), ]
    other_res <-
      census_vector_list[which(census_vector_list$details %in% res_df[which(res_df$n_match != max(res_df$n_match)), ]$results), ]
    if (length(res_df$results) == length(top_res$vector))
      return(top_res)
    else if (!interactive) {return(top_res)}
    else if (length(other_res$vector) > 0 & interactive){
      print(top_res)
      show_more <-
        utils::menu(
          c("Yes", "No"),
          title = paste0(
            "\nThere are ",
            length(other_res$vector),
            " additional keyword matches with less precision. Show more?"
          )
        )
      if (show_more == 1) {return(other_res)} else {
        message(paste0("Showing top ",length(top_res$vector)," results only"))
        return(top_res)}
    }
  }
}

#' Interactively browse Census variables and regions on Censusmapper.ca in a new browser window
#'
#' @description Finding the right Census variables or regions can be complicated.
#' \code{explore_census_vectors(dataset)} and \code{explore_census_regions(dataset)} will open a
#' new browser page or tab to an interactive Census variable and region exploration and selection
#' tool on the \href{https://censusmapper.ca/api}{Censusmapper.ca website}. Interactive
#' tools available for the CA16, CA11, CA06, and CA01 Census datasets and geographies.
#'
#' @param dataset The dataset to query for available vectors, e.g. \code{'CA16'}.
#' Interactive tools available for the CA16, CA11, CA06, and CA01 Census datasets and
#' geographies.
#'
#' @export
#'
#' @examples
#'\dontrun{
#'
#' explore_census_vectors(dataset = "CA16")
#'
#' explore_census_regions(dataset = "CA11")
#'
#' }
explore_census_vectors <- function(dataset = "CA16") {
  dataset <- translate_dataset(dataset)
  message("Opening interactive census variable explorer at censusmapper.ca/api in the browser")
  utils::browseURL(paste0("https://censusmapper.ca/api/",dataset,"#api_variable"))
}

#' Interactively browse Census variables and regions on Censusmapper.ca in a new browser window
#'
#' @description Finding the right Census variables or regions can be complicated.
#' \code{explore_census_vectors(dataset)} and \code{explore_census_regions(dataset)} will open a
#' new browser page or tab to an interactive Census variable and region exploration and selection
#' tool on the \href{https://censusmapper.ca/api}{Censusmapper.ca website}. Interactive
#' tools available for the CA16, CA11, CA06, and CA01 Census datasets and geographies.
#'
#' @param dataset The dataset to query for available vectors, e.g. \code{'CA16'}.
#' Interactive tools available for the CA16, CA11, CA06, and CA01 Census datasets and
#' geographies.
#'
#' @export
#'
#' @examples
#'\dontrun{
#'
#' explore_census_vectors(dataset = "CA16")
#'
#' explore_census_regions(dataset = "CA11")
#'
#' }
explore_census_regions <- function(dataset = "CA16") {
  dataset <- translate_dataset(dataset)
  message("Opening interactive census region explorer at censusmapper.ca/api in the browser")
  utils::browseURL(paste0("https://censusmapper.ca/api/",dataset,"#api_region"))
}

test_that("semantic_search generates correct n-grams for single word query", {
  skip_if_not_installed("dplyr")

  # Create mock census vector data
  mock_vectors <- data.frame(
    vector = c("v_TEST_1", "v_TEST_2", "v_TEST_3"),
    details = c("Total population count", "Male population aged 15", "Female income median"),
    label = c("Population", "Male", "Female"),
    stringsAsFactors = FALSE
  )

  # Test with single word - should match "population"
  result <- cancensus:::semantic_search("population", mock_vectors)

  # Should find vectors with "population" in details
  expect_true(!is.null(result))
  expect_true(nrow(result) > 0)
  expect_true(any(grepl("population", result$details, ignore.case = TRUE)))
})

test_that("semantic_search generates correct n-grams for multi-word query", {
  skip_if_not_installed("dplyr")

  mock_vectors <- data.frame(
    vector = c("v_TEST_1", "v_TEST_2", "v_TEST_3", "v_TEST_4"),
    details = c(
      "Total population count by age",
      "Male population aged 15 to 24",
      "Female population aged 25 to 34",
      "Income median for households"
    ),
    label = c("Pop1", "Pop2", "Pop3", "Income"),
    stringsAsFactors = FALSE
  )

  # Test with two-word query
  result <- cancensus:::semantic_search("male population", mock_vectors)

  # Should match vectors with "male" and "population"
  expect_true(!is.null(result))
  if (nrow(result) > 0) {
    expect_true(any(grepl("male", result$details, ignore.case = TRUE)))
  }
})

test_that("semantic_search handles empty vectors", {
  skip_if_not_installed("dplyr")

  mock_vectors <- data.frame(
    vector = character(0),
    details = character(0),
    label = character(0),
    stringsAsFactors = FALSE
  )

  # Should handle empty input - may error or warn depending on implementation
  # This is an edge case that likely doesn't occur in real usage
  expect_error(
    result <- cancensus:::semantic_search("population", mock_vectors)
  )
})

test_that("semantic_search handles vectors with punctuation", {
  skip_if_not_installed("dplyr")

  mock_vectors <- data.frame(
    vector = c("v_TEST_1", "v_TEST_2"),
    details = c(
      "Population: total, all ages (2021)",
      "Income - median household income"
    ),
    label = c("Pop", "Income"),
    stringsAsFactors = FALSE
  )

  # Punctuation should be handled correctly
  result <- cancensus:::semantic_search("population total", mock_vectors)

  expect_true(!is.null(result))
  # Should find match despite punctuation
  if (nrow(result) > 0) {
    expect_true(any(grepl("population", result$details, ignore.case = TRUE)))
  }
})

test_that("semantic_search handles case insensitivity", {
  skip_if_not_installed("dplyr")

  mock_vectors <- data.frame(
    vector = c("v_TEST_1", "v_TEST_2"),
    details = c(
      "POPULATION TOTAL",
      "population total"
    ),
    label = c("Pop1", "Pop2"),
    stringsAsFactors = FALSE
  )

  # Should find matches regardless of case
  result1 <- cancensus:::semantic_search("POPULATION", mock_vectors)
  result2 <- cancensus:::semantic_search("population", mock_vectors)
  result3 <- cancensus:::semantic_search("Population", mock_vectors)

  # All should return results
  expect_true(!is.null(result1))
  expect_true(!is.null(result2))
  expect_true(!is.null(result3))
})

test_that("semantic_search with no close matches warns user", {
  skip_if_not_installed("dplyr")

  mock_vectors <- data.frame(
    vector = c("v_TEST_1", "v_TEST_2"),
    details = c(
      "Population total count",
      "Income median value"
    ),
    label = c("Pop", "Income"),
    stringsAsFactors = FALSE
  )

  # Query with no close match should warn
  expect_warning(
    result <- cancensus:::semantic_search("zzzzxxxxxqqqqq", mock_vectors),
    "No close matches found"
  )
})

test_that("semantic_search handles short sentences correctly", {
  skip_if_not_installed("dplyr")

  mock_vectors <- data.frame(
    vector = c("v_TEST_1", "v_TEST_2", "v_TEST_3"),
    details = c(
      "Total",  # Very short
      "Population",  # Single word
      "Total population count by age groups"  # Long
    ),
    label = c("T", "P", "TPC"),
    stringsAsFactors = FALSE
  )

  # Should handle varying length details
  result <- cancensus:::semantic_search("total", mock_vectors)

  expect_true(!is.null(result))
  if (nrow(result) > 0) {
    expect_true(any(grepl("total", result$details, ignore.case = TRUE)))
  }
})

test_that("semantic_search n-gram optimization produces identical results", {
  skip_if_not_installed("dplyr")

  # Create a realistic-sized test set
  mock_vectors <- data.frame(
    vector = paste0("v_TEST_", 1:50),
    details = c(
      "Total population by age groups and gender",
      "Male population aged 0 to 14 years",
      "Female population aged 15 to 24 years",
      "Total households by family composition",
      "Median household income after tax",
      "Average household income before tax",
      "Total dwelling units by structure type",
      "Population density per square kilometer",
      "Labour force participation rate by age",
      "Unemployment rate for all ages",
      rep("Other census variable details", 40)
    ),
    label = paste0("Label_", 1:50),
    stringsAsFactors = FALSE
  )

  # Test several queries
  queries <- c("population", "household income", "aged 15", "total")

  for (query in queries) {
    # The function should still work correctly with optimization
    result <- cancensus:::semantic_search(query, mock_vectors)

    # Should return results for reasonable queries
    if (query %in% c("population", "household income", "total")) {
      expect_true(!is.null(result) || inherits(result, "data.frame"))
    }
  }
})

## ----setup, echo=FALSE, message=FALSE, warning=FALSE--------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)
library(cancensus)
library(dplyr)
# set_cancensus_api_key(<your_api_key>', install = TRUE)
# set_cancensus_cache_path(<local cache path>, install = TRUE)

## ----load_package_cran, echo=TRUE, message=FALSE, warning=FALSE, eval = FALSE----
# install.packages("cancensus")
# 
# library(cancensus)

## ----load_package_git, echo=TRUE, message=FALSE, warning=FALSE, eval = FALSE----
# # install.packages("devtools")
# remotes::install_github("mountainmath/cancensus")
# 
# library(cancensus)

## ----install_api_key_and_cache_path, echo=TRUE, message=FALSE, warning=FALSE, eval = FALSE----
# # Only need to install api key can cache path once
# set_cancensus_api_key('<your_api_key>', install = TRUE)
# set_cancensus_cache_path('<local cache path>', install = TRUE)

## ----get_census example, echo=TRUE, warning=FALSE, message=FALSE--------------
# Returns a data frame with data only
census_data <- get_census(dataset='CA21', regions=list(CMA="59933"),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, geo_format = NA, quiet = TRUE)

# Returns data and geography as an sf-class data frame
census_data <- get_census(dataset='CA21', regions=list(CMA="59933"),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, geo_format = 'sf', quiet = TRUE)

# Returns a SpatialPolygonsDataFrame object with data and geography
census_data <- get_census(dataset='CA21', regions=list(CMA="59933"),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, geo_format = 'sp', quiet = TRUE)

## ----list datasets, message=FALSE, warning=FALSE------------------------------
list_census_datasets()

## ----list regions, message=FALSE, warning=FALSE-------------------------------
list_census_regions("CA21")

## ----message=FALSE, warning=FALSE---------------------------------------------
# Retrieves Vancouver and Toronto
list_census_regions('CA21') %>% 
  filter(level == "CMA", name %in% c("Vancouver","Toronto"))

census_data <- get_census(dataset='CA21', regions=list(CMA=c("59933","35535")),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, quiet = TRUE)

## ----list_vectors, message=FALSE, warning=FALSE-------------------------------
list_census_vectors("CA21")

## ----search_vectors1, message=FALSE, warning=FALSE----------------------------
# Find the variable indicating the number of people of Austrian ethnic origin
find_census_vectors("Australia", dataset = "CA16", type = "total", query_type = "exact")

find_census_vectors("Australia origin", dataset = "CA16", type = "total", query_type = "semantic")

find_census_vectors("Australian ethnic", dataset = "CA16", type = "total", query_type = "keyword", interactive = FALSE)

## ----parent_vectors, message=FALSE, warning=FALSE-----------------------------
list_census_vectors("CA16") %>% 
  filter(vector == "v_CA16_4092") %>% 
  parent_census_vectors()

## ----child_vectors1, message=FALSE, warning=FALSE-----------------------------
# Find the variable indicating the Northern European aggregate
find_census_vectors("Northern European", dataset = "CA16", type = "Total")

## ----child_vectors2, message=FALSE, warning=FALSE-----------------------------
# Show all child variable leaves
list_census_vectors("CA16") %>% 
  filter(vector == "v_CA16_4122") %>% child_census_vectors(leaves = TRUE)

#' Read the geosuite data
#'
#' @description
#' Reads the original unprocessed geographic boundary files from Statistics Canada
#'
#' @param census_year census year to get the data for, right now only 2021 is supported
#' @param level geographic level to return the data for, valid choices are
#' "PR","CD","CMACA","CSD","CT","ADA","DA","DB,"ER","FED","DPL","POPCNTR", "FSA", "HR"
#' @param type type of geographic data, valid choices area "cartographic" or "digital"
#' @param cache_path optional path to cache the data. If the cancensus cache path is set the geographic data gets
#' cached in the "geographies" subdirectory of the cancensus cache path.
#' @param timeout optional timeout parameter, adjust as needed if the data download times out when using slow connections
#' @param refresh (logical) refresh the cache if true
#' @param quiet (logical) suppress messages if `TRUE`
#' @return a spatial dataframe with the geographic data
#'
#' @examples
#' # get the digital geographic boundaries for provinces and territories
#' \dontrun{
#' get_statcan_geographies(census_year="2021",level="PR",type="digital")
#' }
#' @export
get_statcan_geographies <- function(census_year,level,type="cartographic",
                                    cache_path = NULL,timeout=1000,
                                    refresh=FALSE,quiet=FALSE) {
  valid_census_years <- c("2021")
  valid_levels <- c("PR","CD","CMACA","CMA","CA","CSD","CT","ADA","DA","DB","ER","FED","DPL","POPCNTR","POPCTR","FSA","HR")
  valid_types <- c("cartographic","digital")
  if (!(census_year %in% valid_census_years)) {
    stop(paste0("Census year must be one of ",paste0(valid_census_years,collapse = ", "),"."))
  }
  if (!(type %in% valid_types)) {
    stop(paste0("Type must be one of ",paste0(valid_types,collapse = ", "),"."))
  }
  if (!(level %in% valid_levels)) {
    stop(paste0("Level must be one of ",paste0(valid_levels,collapse = ", "),"."))
  }
  level_map <-  c("CMACA"="CMA","CA"="CMA","POPCNTR"="PC","POPCTR"="PC")
  if (level %in% names(level_map)) level <-level_map[[level]]
  geo_base_path <- cache_path("geographies")
  if (!dir.exists(geo_base_path)) dir.create(geo_base_path)
  geo_base_path <- file.path(geo_base_path,type)
  if (!dir.exists(geo_base_path)) dir.create(geo_base_path)
  exdir <- file.path(geo_base_path,level)
  if (refresh || !dir.exists(exdir) || length(dir(exdir,"\\.shp$"))==0) {
    old_timeout <- getOption("timeout")
    if (type=="cartographic") typeID <- "b" else typeID <- "a"
    if (nchar(level)==2) filler="_000"
    else if (nchar(level)==3) filler="000"
    else {
      stop(paste0("Problem, don't know how to get geographic data for level ",level,"."))
    }
    url <- paste0("https://www12.statcan.gc.ca/census-recensement/",census_year,"/geo/sip-pis/boundary-limites/files-fichiers/l",tolower(level),filler,typeID,"21a_e.zip")
    if (level=="HR") {
      if (type=="cartographic") {
        url <- "https://www150.statcan.gc.ca/n1/en/pub/82-402-x/2024001/hrbf-flrs/carto/ArcGIS/HR_000b23a_e.zip?st=wQRRL7PQ"
      } else {
        url <- "https://www150.statcan.gc.ca/n1/en/pub/82-402-x/2024001/hrbf-flrs/digi/ArcGIS/HR_000a23a_e.zip"
      }
    }
    tmp <- tempfile()
    options(timeout = timeout)
    utils::download.file(url,tmp,mode="wb",quiet=quiet)
    options(timeout = old_timeout)
    utils::unzip(tmp,exdir = exdir)
    fs <- dir(exdir,full.names = TRUE)
    if (length(fs)==1 && dir.exists(fs)) {
      tmp_dir <- tempfile(pattern = "cancensus_geo_", tmpdir = geo_base_path)
      file.rename(exdir,tmp_dir)
      fs <- dir(tmp_dir,full.names = TRUE)
      file.rename(fs,exdir)
      unlink(tmp_dir)
    }
  } else {
    if (!quiet) message("Reading geographic data from local cache.")
  }
  path <- dir(exdir,"\\.shp$",full.names = TRUE)

  geos <- sf::read_sf(path)
  geos
}

metadata_columns <- c("path" , "dataset", "regions", "level", "vectors", "created_at",
                      "version", "size" ,"last_accessed", "access_count",  "resolution")
metadata_columns_c <- c("path" , "dataset", "regions", "level", "vectors", "version", "resolution")
metadata_columns_d <- c("created_at","last_accessed")
metadata_columns_n <- c("size" , "access_count")


generate_metadata <- function(meta_file,params){
  if (file.exists(meta_file)) {
    metadata <- readRDS(meta_file) %>%
      dplyr::mutate(last_accessed=Sys.time())
    if ("access_count" %in% names(metadata)) {
      metadata <- metadata %>% dplyr::mutate(access_count=.data$access_count+1)
    } else {
      metadata <- metadata %>% dplyr::mutate(access_count=1)
    }
  } else {
    pattern <- basename(meta_file) %>% gsub("\\.meta$","",.)
    path <- dir(cache_path(),pattern = pattern)
    metadata <- dplyr::tibble(last_accessed=Sys.time(),access_count=1)
  }
  if (!is.null(params) & !("dataset" %in% names(metadata)) & "dataset" %in% names(params)) {
    metadata$dataset=params$dataset
  }
  if (!is.null(params) & !("level" %in% names(metadata)) & "level" %in% names(params)) {
    metadata$level=params$level
  }
  if (!is.null(params) & !("regions" %in% names(metadata)) & "regions" %in% names(params)) {
    metadata$regions=as.character(params$regions)
  }
  if (!is.null(params) & !("resolution" %in% names(metadata)) & "resolution" %in% names(params)) {
    metadata$resolution=as.character(params$resolution)
  }
  if (!is.null(params) & !("vectors" %in% names(metadata)) & "vectors" %in% names(params)) {
    metadata$vectors=jsonlite::toJSON(as.character(params$vectors))  %>% as.character()
  }
  if (!("version" %in% names(metadata))) {
    path <- basename(meta_file)
    if (grepl("^CM_geo_",path)) metadata$version="g.1"
    if (grepl("^CM_data_",path)) metadata$version="d.1"
  }

  for (n in setdiff(metadata_columns_c,names(metadata))) {
    metadata[,n]=NA_character_
  }
  # for (n in setdiff(metadata_columns_d,names(metadata))) {
  #   metadata[,n]=NA
  # }
  for (n in setdiff(metadata_columns_n,names(metadata))) {
    metadata[,n]=NA_real_
  }

  metadata
}

touch_metadata <- function(meta_file,params=NULL){
  metadata <- generate_metadata(meta_file,params)
  saveRDS(metadata,file = meta_file)
}


#' List cached files
#'
#' @description
#' Lists all cached data and metadata if available
#'
#' @return tibble with metadata on cached data
#'
#' @examples
#' # list add the cached census data
#' list_cancensus_cache()
#'
#' @export
list_cancensus_cache <- function(){
  cp <- cache_path()
  metas <- dir(cp,"\\.meta$") %>%
    lapply(function(m)readRDS(file.path(cp,m)) %>% dplyr::mutate(path=gsub("\\.meta$","",m))) %>%
    dplyr::bind_rows()

  all <- dplyr::tibble(path=dir(cp,"^CM_")) %>%
    dplyr::filter(!grepl("\\.meta$",.data$path))

  if (nrow(metas)==0){
    all <- all %>% dplyr::mutate(size=NA_real_)
  } else {
    all <- all %>% dplyr::left_join(metas,by="path")
    if (!("size") %in% names(all)) all <- all %>% dplyr::mutate(size=NA_real_)
  }

  missing_sizes <- all %>%
    dplyr::filter(is.na(.data$size)) %>%
    dplyr::pull(.data$path)

  if (length(missing_sizes)>0) {
    manual_sizes <- missing_sizes %>%
      lapply(function(p) dplyr::tibble(path=p,manual_size=file.info(file.path(cp,p))$size)) %>%
      dplyr::bind_rows()

    all <- all %>%
      dplyr::left_join(manual_sizes,by="path") %>%
      dplyr::mutate(size=dplyr::coalesce(.data$size,.data$manual_size)) %>%
      dplyr::select(-.data$manual_size)
  }

  for (n in setdiff(metadata_columns_c,names(all))) {
    all[,n]=NA_character_
  }
  # for (n in setdiff(metadata_columns_d,names(metadata))) {
  #   metadata[,n]=NA
  # }
  for (n in setdiff(metadata_columns_n,names(all))) {
    all[,n]=NA_real_
  }


  all
}


#' Remove cached files
#'
#' @description
#' Remove cached data for paths given
#'
#' @param paths list of paths to remove, as returned by the path column in `list_cancensus_cache`
#' @return freed-up disk space
#'
#'@examples
#'\dontrun{
#' # remove the first cached dataset
#' cache_data <- list_cancensus_cache()
#'
#' remove_from_cancensus_cache(cache_data$path[1])
#' }
#' @export
remove_from_cancensus_cache <- function(paths){
  cp <- cache_path()

  cache_data <- list_cancensus_cache()

  extras <- setdiff(paths,cache_data$path)
  if (length(extras)>0) warning(paste0("Could not find cached data for ",
                                       paste0(extras, collapse = ", ")))


  to_delete <- cache_data %>%
    dplyr::filter(.data$path %in% paths)
  freed_up_size <- to_delete$size %>% sum(na.rm=TRUE)
  if (nrow(to_delete)>0) {
    to_delete$path %>%
      lapply(function(p){
        unlink(file.path(cp,p))
        meta_path <- file.path(cp,paste0(p,".meta"))
        if (file.exists(meta_path)) unlink(meta_path)
        })
  }

  freed_up_size
}

---
title: "cancensus"
author: "" 
date: ""
output: rmarkdown::html_vignette
mainfont: Roboto

vignette: >
  %\VignetteIndexEntry{cancensus}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, echo=FALSE, message=FALSE, warning=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)
library(cancensus)
library(dplyr)
# set_cancensus_api_key(<your_api_key>', install = TRUE)
# set_cancensus_cache_path(<local cache path>, install = TRUE)
```
# Cancensus and CensusMapper

The **cancensus** package was developed to provide users with a way to access Canadian Census in a programmatic way following good [tidy data](http://vita.had.co.nz/papers/tidy-data.pdf) practices. While the structure and data in **cancensus** is unique to Canadian Census data, this package is inspired in part by [tidycensus](https://github.com/walkerke/tidycensus), a package to interface with the US Census Bureau data APIs. 

As Statistics Canada does not provide direct API access to Census data, **cancensus** retrieves Census data indirectly through the [CensusMapper](https://censusmapper.ca) API. CensusMapper is a project by [Jens von Bergmann](https://github.com/mountainMath), one of the authors of **cancensus**, to provide interactive geographic visualizations of Canadian Census data. CensusMapper databases store all publicly available data from Statistics Canada for the 2006, 2011, and 2016 Censuses. CensusMapper data can be accessed via an API and **cancensus** is built to interface directly with it. 

## API Key

**cancensus** requires a valid CensusMapper API key to use. You can obtain a free API key by [signing up](https://censusmapper.ca/auth/register) for a CensusMapper account. CensusMapper API keys are free and public API quotas are generous; however, due to incremental costs of serving large quantities of data, there limits to API usage in place. For most use cases, these API limits should not be an issue. Production uses with large extracts of fine grained geographies may run into API quota limits. For larger quotas, please get in touch with Jens [directly](mailto:jens@censusmapper.ca).  

To check your API key, just go to "Edit Profile" (in the top-right of the CensusMapper menu bar). Once you have your key, you can store it in your system environment so it is automatically used in API calls. To do so just enter `set_cancensus_api_key(<your_api_key>, install = TRUE)`

## Installing cancensus

The stable version of **cancensus** can be easily installed from CRAN.
```{r load_package_cran, echo=TRUE, message=FALSE, warning=FALSE, eval = FALSE}
install.packages("cancensus")

library(cancensus)
```

Alternatively, the latest development version can be installed from Github using `remotes``. 
```{r load_package_git, echo=TRUE, message=FALSE, warning=FALSE, eval = FALSE}
# install.packages("devtools")
remotes::install_github("mountainmath/cancensus")

library(cancensus)
```

If you have not already done so, you can install the API keys and the data cache path. You can get your free API key when you sign up for a [CensusMapper account](https://censusmapper.ca/) and check your profile. Additionally we recommend you set a permanent data cache path so that census data you query is stored persistently across sessions. **Make sure to replace the filler text `<your_api_key>` and `<local cache path>` with your actual API key and local cache path.**

```{r install_api_key_and_cache_path, echo=TRUE, message=FALSE, warning=FALSE, eval = FALSE}
# Only need to install api key can cache path once
set_cancensus_api_key('<your_api_key>', install = TRUE)
set_cancensus_cache_path('<local cache path>', install = TRUE)
```

Data in the persistent cache can be managed via the functions `list_cancensus_cache` and `remove_from_cancensus_cache` if needed.

# Accessing Census Data

**cancensus** provides three different functions for retrieving Census data:
* `get_census` to retrieve Census data and geography as a spatial dataset
* `get_census_data` to retrieve Census data only as a flat data frame
* `get_census_geometry` to retrieve Census geography only as a collection of spatial polygons.

`get_census` takes as inputs a dataset parameter, a list of specified regions, a vector of Census variables, and a Census geography level. You can specify one of three options for spatial formats: `NA` to return data only, `sf` to return an sf-class data frame, or `sp` to return a SpatialPolygonsDataFrame object. 

```{r get_census example, echo=TRUE, warning=FALSE, message=FALSE}
# Returns a data frame with data only
census_data <- get_census(dataset='CA21', regions=list(CMA="59933"),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, geo_format = NA, quiet = TRUE)

# Returns data and geography as an sf-class data frame
census_data <- get_census(dataset='CA21', regions=list(CMA="59933"),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, geo_format = 'sf', quiet = TRUE)

# Returns a SpatialPolygonsDataFrame object with data and geography
census_data <- get_census(dataset='CA21', regions=list(CMA="59933"),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, geo_format = 'sp', quiet = TRUE)
```

**cancensus** utilizes caching to increase speed, minimize API token usage, and to make data available offline. Downloaded data is hashed and stored locally so if a call is made to access the same data, **cancensus** will read the local version instead. To force **cancensus** to refresh the data, specify `use_cache = FALSE` as a parameter for `get_census`. 

Additional parameters for advanced options can be viewed by running `?get_census`.

## Census Datasets

**cancensus** can access Statistics Canada Census data for Census years 1996, 2001, 2006, 2011, 2016, and 2021. You can run `list_census_datasets` to check what datasets are currently available for access through the CensusMapper API. 

Thanks to contributions by the Canada Mortgage and Housing Corporation (CMHC), **cancensus** now includes additional Census-linked datasets as open-data releases. These include annual taxfiler data at the census tract level for tax years 2000 through 2017, which includes data on incomes and demographics, as well as specialized crosstabs for Structural type of dwelling by Document type, which details occupancy status for residences. These crosstabs are available for the 2001, 2006, 2011, and 2016 Census years at all levels starting with census tract.

The function `list_census_datasets()` will show all available datasets alongside their metadata.

```{r list datasets, message=FALSE, warning=FALSE}
list_census_datasets()
```

As other Census datasets become available via the CensusMapper API, they will be listed as output when calling `list_census_datasets()`. 

## Census Regions

Census data is aggregated at multiple [geographic levels](### Census Geographic Levels). Census geographies at the national (C), provincial (PR), census metropolitan area (CMA), census agglomeration (CA), census division (CD), and census subdivision (CSD) are defined as named census regions. 

Canadian Census geography can change in between Census periods. **cancensus** provides a function, `list_census_regions(dataset)`, to display all named census regions and their corresponding id for a given census dataset. 

```{r list regions, message=FALSE, warning=FALSE}
list_census_regions("CA21")
```

The `regions` parameter in `get_census` requires as input a list of region id strings that correspond to that regions geoid. You can combine different regions together into region lists.

```{r, message=FALSE, warning=FALSE}
# Retrieves Vancouver and Toronto
list_census_regions('CA21') %>% 
  filter(level == "CMA", name %in% c("Vancouver","Toronto"))

census_data <- get_census(dataset='CA21', regions=list(CMA=c("59933","35535")),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, quiet = TRUE)
```

## Census Geographic Levels

Census data accessible through **cancensus** comes is available in a number of different aggregation levels including: 

| Code    | Description                    | Count in Census 2016 |
| --------|--------------------------------|:--------------------:|
| C       | Canada (total)                 | 1                    |
| PR      | Provinces/Territories          | 13                   |
| CMA     | Census Metropolitan Area       | 35                   |
| CA      | Census Agglomeration           | 14                   |
| CD      | Census Division                | 287                  |
| CSD     | Census Subdivision             | 713                  |
| CT      | Census Tracts                  | 5621                 |
| DA      | Dissemination Area             | 56589                |
| EA      | Enumeration Area (1996 only)   | -                    |
| DB      | Dissemination Block (2001-2016)| 489676               |
| Regions | Named Census Region            |                      |

Selecting `regions = "59933"` and `level = "CT"` will return data for all 478 census tracts in the Vancouver Census Metropolitan Area. Selecting `level = "DA"` will return data for all 3450 dissemination areas and selecting `level = "DB"` will retrieve data for 15,197 dissemination block. Working with CT, DA, EA, and especially DB level data significantly increases the size of data downloads and API usage. To help minimize additional overhead, **cancensus** supports local data caching to reduce usage and load times. 

Setting `level = "Regions"` will produce data strictly for the selected region without any tiling of data at lower census aggregation levels levels. 

# Working with Census Variables

Census data contains thousands of different geographic regions as well as thousands of unique variables. In addition to enabling programmatic and reproducible access to Census data, **cancensus** has a number of tools to help users find the data they are looking for. 

## Displaying available Census variables

Run `list_census_vectors(dataset)` to view all available Census variables for a given dataset. 

```{r list_vectors, message=FALSE, warning=FALSE}
list_census_vectors("CA21")
```

## Variable characteristics

For each variable (vector) in that Census dataset, this shows:

* Vector: short variable code 
* Type: variables are provided as aggregates of female responses, male responses, or total (male+female) responses
* Label: detailed variable name
* Units: provides information about whether the variable represents a count integer, a ratio, a percentage, or a currency figure
* Parent_vector: shows the immediate hierarchical parent category for that variable, where appropriate
* Aggregation: indicates how the variable should be aggregated with others, whether it is additive or if it is an average of another variable
* Description: a rough description of a variable based on its hierarchical structure. This is constructed by **cancensus** by recursively traversing the labels for every variable's hierarchy, and facilitates searching for specific variables using key terms. 

## Variable search

Each Census dataset features numerous variables making it a bit of a challenge to find the exact variable you are looking for. There is a function, `find_census_vectors()`, for searching through Census variable metadata in a few different ways. There are three types of searches possible using this function: exact search, which simply looks for exact string matches for a given query against the vector dataset; keyword search, which breaks vector metadata into unigram tokens and then tries to find the vectors with the greatest number of unique matches; and, semantic search which works better with search phrases and has tolerance for inexact searches. Switching between search modes is done using the `query_type` argument when calling `find_census_variables()` function. 

```{r search_vectors1, message=FALSE, warning=FALSE}
# Find the variable indicating the number of people of Austrian ethnic origin
find_census_vectors("Australia", dataset = "CA16", type = "total", query_type = "exact")

find_census_vectors("Australia origin", dataset = "CA16", type = "total", query_type = "semantic")

find_census_vectors("Australian ethnic", dataset = "CA16", type = "total", query_type = "keyword", interactive = FALSE)
```

## Managing variable hierarchy 

Census variables are frequently hierarchical. As an example, consider the variable for the number of people of Austrian ethnic background. We can select that vector and quickly look up its entire hierarchy using `parent_census_vectors` on a vector list.  

```{r parent_vectors, message=FALSE, warning=FALSE}
list_census_vectors("CA16") %>% 
  filter(vector == "v_CA16_4092") %>% 
  parent_census_vectors()
```

Sometimes we want to traverse the hierarchy in the opposite direction. This is frequently required when looking to compare different variable stems that share the same aggregate variable. As an example, if we want to look the total count of Northern European ethnic origin respondents disaggregated by individual countries, it is pretty easy to do so. 

```{r child_vectors1, message=FALSE, warning=FALSE}
# Find the variable indicating the Northern European aggregate
find_census_vectors("Northern European", dataset = "CA16", type = "Total")
```

The search result shows that the vector **v_CA16_4092** represents the aggregate for all Northern European origins. The `child_census_vectors` function can return a list of its constituent underlying variables.

```{r child_vectors2, message=FALSE, warning=FALSE}
# Show all child variable leaves
list_census_vectors("CA16") %>% 
  filter(vector == "v_CA16_4122") %>% child_census_vectors(leaves = TRUE)
```

The `leaves = TRUE` parameter specifies whether intermediate aggregates are included or not. If `TRUE` then only the lowest level variables are returns - the "leaves" of the hierarchical tree. 

---
title: "StatCan attribute files"
output: rmarkdown::html_vignette
mainfont: Roboto
vignette: >
  %\VignetteIndexEntry{StatCan attribute files}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE, 
  warning = FALSE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)
```

```{r setup}
library(cancensus)
library(dplyr)
```

## Background
Attribute files describe the detailed relationship of various Statistics Canada geographic levels and provide population, household and dewlling counts. This information can be useful for understanding the hierarchical relationships of different geographic levels. The [CensusMapper API](https://censusmapper.ca/api) that **cancensus** uses for most of the data queries is ill-suited to get comprehensive data on the hierarichal relationships Canada wide, so it can be helpful to have direct access to this data in comprehensive tabular form.

## Match between Census Tracts and Census Subdivisions
If we are interested in understanding which Census Tracts respect municipal boundaries and which ones don't in 2021 we can consult the geographic attributes file. It contains a row for each Census Block, the basic building block of census geographies, and tags other levels of geography the Census Block lies in.

```{r}
attributes <- get_statcan_geographic_attributes("2021")

attributes %>% colnames()
```

To answer our question, we filter Census Block by the ones that lie within a Census Tract, and check for the collection of Census Blocks within each Census Tract how many municipalities they lie in.

```{r}
attributes %>%
  filter(CMATYPE_RMRGENRE %in% c("B","K")) |> # filter areas not in CTs
  group_by(CTCODE_SRCODE,CMATYPE_RMRGENRE) |>
  summarise(`Number of municipalities`=length(unique(CSDUID_SDRIDU)),.groups="drop") |>
  count(`Number of municipalities`,CMATYPE_RMRGENRE) |>
  arrange(CMATYPE_RMRGENRE,`Number of municipalities`)
```

This shows that most census tracts for both Census Metropolitan Areas (CMATYPE_RMRGENRE="B") and tracted Census Agglomerations (CMATYPE_RMRGENRE="K"), with some census tracts spanning several municipalities.

---
title: "Finding intersecting geometries from custom data"
subtitle: "Bring custom data and identify corresponding census geographies"
output: rmarkdown::html_vignette
mainfont: Roboto
vignette: >
  %\VignetteIndexEntry{Finding intersecting geometries from custom data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE, 
  warning = FALSE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)
```

A frequent application of census data is to evaluate values for specific areas that do not necessarily correspond to existing boundaries or administrative units. Census users may have their own defined geographies or other geospatial data of interest and want to be able to quickly and easily identify the collection of census features that correspond to that region.

The `get_intersecting_geometries()` function is projection agnostic and accepts any valid `sf` or `sfc` class object as input. These objects are then reprojected into lat/lon coordinates on the backend to facilitate the intersecting join on the server-side. 

## A simple example

As an example, suppose we are interested in understanding the housing tenure split in census tracts located near Vancouver Skytrain (rapid transit) stations. We can use the `COV_SKYTRAIN_STATIONS` dataset that ships with the package and is derived from the [City of Vancouver Open Data portal](https://opendata.vancouver.ca/explore/dataset/rapid-transit-stations/information/) and contains their locations. For our example we are interested in census tracts within 800m of these stations, which ships with the package.

```{r setup}
library(cancensus)
library(dplyr)
library(ggplot2)
library(sf)
```

We load the example data `COV_SKYTRAIN_STATIONS` from the package. 

```{r}
cov_station_buffers <- COV_SKYTRAIN_STATIONS %>%
  st_set_crs(4326)  # needed for Ubuntu or systems with old GDAL but can otherwise be ignored
```

We then use the `get_intersecting_geometries` call to obtain the list of municipalities (CSDs) and census tracts (CTs) that intersect the 800m station buffer objects. 

```{r}
station_city_ids <- get_intersecting_geometries("CA16", level = "CSD", geometry = cov_station_buffers,
                                                quiet=TRUE)
station_ct_ids <- get_intersecting_geometries("CA16", level = "CT", geometry = cov_station_buffers,
                                              quiet=TRUE)
```

These return a list of census geographic identifiers suitable for use in the 'region' argument in `get_census`. We may be interested in the transit to work mode share in each of these buffers.

```{r}
variables <- c(mode_base="v_CA16_5792",transit="v_CA16_5801",walk="v_CA16_5804")
station_city <-  get_census("CA16", regions = station_city_ids, vectors = variables, 
                            geo_format = 'sf', quiet=TRUE) %>% 
  filter(name == "Vancouver (CY)")
station_cts <-   get_census("CA16", regions = station_ct_ids, vectors = variables, 
                            geo_format = 'sf', quiet=TRUE)
```

To understand how these relate we plot the data.

```{r fig.alt="CTs within City of Vancouver skytrain station catchments"}
ggplot(station_city) +
  geom_sf(fill=NA) +
  geom_sf(data=station_cts,aes(fill=((walk+transit)/mode_base))) +
  geom_sf(data=cov_station_buffers,fill=NA,alpha=0.5,color="steelblue") +
  scale_fill_viridis_c(option="magma",labels=scales::percent) +
  coord_sf(datum=NA) +
  labs(title="CTs within City of Vancouver skytrain station catchments",
       fill="Walk or\ntransit\nto work",
       caption="StatCan Census 2016")
```

To get a closer match we can cut out the dissemination areas intersecting the station catchment areas. 

```{r fig.alt="DAs within City of Vancouver skytrain station catchments"}
station_das <-   get_intersecting_geometries("CA16", level = "DA", geometry = cov_station_buffers,
                                             quiet=TRUE) %>% 
  get_census("CA16", regions = ., vectors=variables, geo_format = 'sf', quiet=TRUE)

ggplot(station_city) +
  geom_sf(fill=NA) +
  geom_sf(data=station_das,aes(fill=((walk+transit)/mode_base))) +
  geom_sf(data=cov_station_buffers,fill=NA,alpha=0.5,color="steelblue") +
  scale_fill_viridis_c(option="magma",labels=scales::percent) +
  coord_sf(datum=NA) +
  labs(title="DAs within City of Vancouver skytrain station catchments",
       fill="Walk or\ntransit\nto work",
       caption="StatCan Census 2016")

```

However, API points for `get_intersecting_geometries` are quite limited at this point, an alternative way to obtain the same data is to first query all DAs withing the CTs identified by the previous `get_intersecting_geometries` call and then filter down to those intersecting the station buffers.

```{r fig.alt="DAs within City of Vancouver skytrain station catchments (alternative method)"}
station_das2 <- get_census("CA16", regions = station_ct_ids, vectors=variables, 
                           geo_format = 'sf', level="DA", quiet=TRUE) %>%
  sf::st_filter(cov_station_buffers)

ggplot(station_city) +
  geom_sf(fill=NA) +
  geom_sf(data=station_das2,aes(fill=((walk+transit)/mode_base))) +
  geom_sf(data=cov_station_buffers,fill=NA,alpha=0.5,color="steelblue") +
  scale_fill_viridis_c(option="magma",labels=scales::percent) +
  coord_sf(datum=NA) +
  labs(title="DAs within City of Vancouver skytrain station catchments",
       fill="Walk or\ntransit\nto work",
       caption="StatCan Census 2016")

```

We may increase the default quotas for the `get_intersecting_geometries` call at some point, but while we throttling API usage and monitor server impacts of the new functionality it may be preferable to use the `get_intersecting_geometries` call for higher level geographies only and add a few lines of code to do the final bit of filtering in R. 

## Addendum

We may take this further by estimate values of census variables strictly within catchments areas. Rather than intersecting, some adjustments for spatial disaggregation and interpolation are needed. The `tongfen_estimate` method from the [`tongfen` package](https://mountainmath.github.io/tongfen/index.html) is useful in this case. This is a related package that is designed to work in tandem with `cancensus` in order to facilitate census geography aggregation and is designed to make census data comparable across several censuses.

---
title: "Data discovery"
subtitle: "Resources for finding available and relevant data"
output: rmarkdown::html_vignette
mainfont: Roboto
vignette: >
  %\VignetteIndexEntry{Data discovery}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE, 
  message = FALSE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)
library(cancensus)
```

# Census datasets

`cancensus` can access Statistics Canada Census data for the 1996, 2001, 2006 Censuses, the 2011 Census and National Household Survey, the 2016 Census, as well as the 2021 Census. You can run `list_census_datasets` to check what datasets are currently available for access through the CensusMapper API. 

Thanks to contributions by the Canada Mortgage and Housing Corporation (CMHC), `cancensus` now includes additional Census-linked datasets as open-data releases. These include annual tax-filer data at the census tract level for tax years 2000 through 2017, which includes data on incomes and demographics, as well as specialized crosstabs for Structural type of dwelling by Document type, which details occupancy status for residences. These crosstabs are available for the 2001, 2006, 2011, and 2016 Census years at all levels starting with census tract.
```{r}
list_census_datasets()
```
The `list_census_datasets()` function also provides additional background like series reference code, catalogue reference, and attribution details. 

# Variable vectors

```{r echo=FALSE, fig.alt="Total number of unique variable vectors by Census dataset"}
library(ggplot2)
library(dplyr)
datasets <- c("CA1996","CA01","CA06","CA11","CA16","CA21")

datasets %>%
  lapply(function(ds){
    dplyr::tibble(dataset=ds, vectors=nrow(list_census_vectors(ds)))
  }) %>%
  bind_rows() %>%
  mutate(dataset=factor(dataset,levels=datasets)) %>%
  ggplot(., aes(x = dataset, y = vectors)) +
  geom_col() +
  theme_minimal() +
  theme(panel.grid = element_blank()) +
  labs(x = "Census dataset", y = "",
       title = "Total number of unique variable vectors by Census dataset") +
  scale_y_continuous(labels = scales::comma)
```

The Census datasets that the `cancensus` package provides access to are rich in detail but they can be complex to navigate. There are thousands of variable vectors, including separate vector indicators for aggregations split by `Total`, `Female`, and `Male` populations. As a result, the total number of vectors per dataset is significant, ranging from 1,715 in the `CA01` dataset to 6,623 in the `CA16` one. 

## View available Census variable vectors

```{r message=FALSE, warning=FALSE}
list_census_vectors('CA21')
```
`list_census_vectors(dataset)` retrieves an index of all available vectors for a given dataset from the CensusMapper API or local cache if recently called. Each Census variable has a vector code assigned to it with naming pattern that goes `v_{dataset}_{index}`. This is the code by which vectors are identified through the CensusMapper API. In addition the vector code, there is additional information showing population type, aggregation type, label and details, as well as variable hierarchy. This function can also be used to show the variables for additional datasets made accessible through the CensusMapper API. 

## Searching for Census variable vectors

Due to the large number of Census variables it can be hard to find the right data. There is a function for searching through Census variable metadata in a few different ways. There are three types of searches possible using this function: exact search, which simply looks for exact string matches for a given query against the vector dataset; keyword search, which breaks vector metadata into unigram tokens and then tries to find the vectors with the greatest number of unique matches; and, semantic search which works better with search phrases and has tolerance for inexact searches. Switching between search modes is done using the `query_type` argument when calling `find_census_vectors()` function. 

Note that variable search is optimized for the Census variables in the main Census datasets. While searches generally work for variables in additional datasets such as cross-tabs and taxfiler data, they have not been extensively tested against these datasets. 

### Exact search

Exact search uses exact string matching. It is best used when you know exactly the term you are looking for, and is not robust to spelling errors. 
```{r, warning=TRUE, message=TRUE}
find_census_vectors("Oji-cree", dataset = "CA16", type = "total", query_type = "exact")
```
This, on the other hand, will return a warning. 
```{r, warning=TRUE, message=TRUE}
find_census_vectors("Ojib-cree", dataset = "CA16", type = "total", query_type = "exact")
```
Unless otherwise specified, `find_census_vectors()` will use exact search as the default option. 

### Keyword search

Keyword search is meant to be used when you know the approximate subject matter of the variable without knowing the exact variable you are looking for. Under the hood, keyword search splits the query and vector details into unique unigram tokens and then looks for the vectors with the greatest number of matches. 
```{r, warning=TRUE, message=TRUE}
find_census_vectors('commute mode', dataset = 'CA16', type = 'female', query_type = 'keyword', interactive = FALSE)
```
Keyword search will show all results that have the highest number of unique keyword matches. What happens if there are other keyword matches that have fewer total matches? By default, the function argument is set as `interactive = TRUE`, which will prompt the user with a console menu option to see the rest of the matches or not. If using `find_census_vectors()` in a script or reproducible documentation, we recommend setting this argument to `interactive = FALSE`. 

### Semantic search

Semantic search works best with phrases. Rather than the decontextualized unigrams of keyword search, semantic search splits queries and vector details into n-grams and finds matches using approximate string distances based on the generalized Levenshtein distance. 
```{r}
find_census_vectors("after tax incomes", dataset = "CA16", type = "total", query_type = "semantic")
```
Semantic search is more robust to spelling and punctuation issues that may come up with exact search. For example, while this throws a warning:
```{r, warning=TRUE, message=TRUE}
find_census_vectors("ojib cree", dataset = "CA16", type = "total", query_type = "exact")
```
This will find the correct Census vector. 
```{r, warning=TRUE, message=TRUE}
find_census_vectors('ojib cree', dataset = 'CA16', type = 'total', query_type = 'semantic')
```
Results are ordered by string proximity if there are multiple possible matches. 

# Census regions

## Standard Geographical Classification 

Statistics Canada uses an official classification of geographic areas known as the [Standard Geographical Classification (SGC)](https://www.statcan.gc.ca/en/subjects/standard/sgc/2021/introduction), which is updated periodically. The latest version is based on the 2021 Census. Geographic classification codes are standardized across Statistics Canada products, including the Census as well as any other Statistics Canada dataset. In practice, this means that the region ID for the Vancouver Census subdivision is 5915022 across all products. In `cancensus` the region ID code is used to identify the appropriate spatial vector data to retrieve alongside Census data. These region IDs have a predictable structure, where provinces are two digits, Census divisions are 4 digits (including 2 for the province), and Census subdivisions have 7 digits (including 2 for the province, and 2 for the Census division). 
```{r echo=FALSE}
tibble(PR = c(35,35,35), CD = c(NA, 18, 18), CSD = c(NA, NA, 013), name = c("Ontario","Durham (Regional municipality","Oshawa (City)"))
```
These levels are hierarchical and complete in that a province is split in Census divisions, which are then split into Census subdivisions. 

Geographies have standardized names for the province, Census division, and Census subdivision levels, as well as Census metropolitan areas and Census agglomerations. Lower geographic levels such as Census tracts or dissemination areas (DA, EA, and DB) are not named or listed but have unique identifying codes derived from their parent Census subdivision.  
```{r echo=FALSE}
list_census_regions('CA21') %>% 
  group_by(level) %>% 
  tally()
```
There is also an additional region, with the id `01` and the level code `C` which represents all of Canada as a whole. 

### A note on Census Metropolitan Areas and Census Agglomerations

Data can also be extracted at the Census Metropolitan Area (CMA) or Census Agglomeration (CA) level, which is derived from a variant of the SGC known as the Statistical Area Classification. Hierarchically, CMAs and CAs represent a collection of constituent Census subdivisions.  

A Census metropolitan area consists of adjacent municipalities with a defined core with a total population of at least 100,000 of which 50,000 or more must live in the core based on Census data. Adjacent municipalities must have a high degree of integration with the core, which Statistics Canada measures based on the commuting flows indicated in Census data. Census Agglomeration areas have to have a core population above 10,000. 

All CMAs and CAs consist of Census subdivisions but not all Census subdivisions are a subset of a CMA or a CA. For more details on CMAs and CAs, consult Statistics Canada's Census Dictionary article for [Census metropolitan areas (CMA) and census agglomerations (CA)](https://www12.statcan.gc.ca/census-recensement/2021/ref/dict/az/Definition-eng.cfm?ID=geo009). All CMAs and some CAs have data at the Census tract level, but most CAs do not. The 2021 Census has 41 CMAs and 9 CAs with Census tracts that have their own defined geography. There are a further 102 CAs without Census tracts that do not have their own distinctly defined geographies. 

### Aside: dissemination areas, blocks, and enumeration areas

[Dissemination areas](https://www12.statcan.gc.ca/census-recensement/2021/ref/dict/az/Definition-eng.cfm?ID=geo021) (DA) are the smallest atomic geographic unit at which all census data is captured. DAs cover the entirety of Canada and follow the boundaries of census subdivisions and census tracts. While inter-census geographic stability is not guaranteed, they generally tend to be as stable as the census tracts and census subdivisions that they make up. In addition to census boundaries, DAs will generally follow natural boundaries created by other spatial features like roads, railways, water features, and designed to be spatially compact and with a target population around 400-700 persons. The 2021 census data has 57,936 distinct DAs. 

[Enumeration areas](https://www12.statcan.gc.ca/English/census01/products/reference/dict/geo024.htm) (EA) were the DA equivalent for censuses prior to 2001. Similar to DAs, EAs were used to as the basic level at which census data was collected. They do not necessarily correspond accurately to DAs in data from 2001 onwards. 

[Dissemination block](https://www12.statcan.gc.ca/census-recensement/2021/ref/dict/az/Definition-eng.cfm?ID=geo014) (DB) level data is available for the 2001-2021 datasets. DBs are essentially city blocks, bounded by intersecting streets and therefore are largely the product of road networks at the time of the census. The geographies and identification codes of DBs are not necessarily stable over time. DBs are split whenever they intersect with boundaries of higher geographic levels in such a way as to ensure that they can be aggregated upwards precisely. DBs only provide data for population, dwelling counts, and number of households (from 2006 onwards) without any additional characteristic data. DBs with population under 15 have their population counts adjusted for privacy. For the 2021 census, there are close to half a million DB distinct regions. 

## Viewing available Census regions

For any valid Census dataset, you can view all available Census regions by calling `list_census_regions(dataset)`. This will retrieve the `region` code, the `name`, and the `level` code indicating the type of geography. Other information includes population, municipal status, as well as parent geographic ids for lower levels. All CMAs are included with their own defined geography, as well as those CAs which have their own Census tracts.
```{r}
list_census_regions('CA21')
```
There are 53 CSD and 12 CD municipal status codes based on official designations used by provinces, territories, and federal authorities. These are often used to distinguish Census divisions and subdivisions with similar or identical names. CAs with Census tracts and defined geography have the code K, while those without have type code D. 
```{r echo=FALSE, paged.print=TRUE}
if(Sys.getenv("COMPILE_VIG")=="TRUE") {
    rmarkdown::paged_table(CODES_TABLE)
}
```

## Searching through named Census regions

We can also search through all named geographies. This will return any geographies that have a name that matches or partially matches the search query. 
```{r}
search_census_regions("Vancouver","CA21")
```

# Exploring Census variable vectors and regions interactively

Sometimes it can be easier to find the right vectors or regions by exploring the layout and hierarchy of Census data. This is especially true when we are not sure of what information is available or are not sure where to start. Finding the right Census geographic code on a map will be easier for some than using named search. This is also handy if we want to assemble a custom aggregation of region codes at different hierarchies. 

To facilitate this, we have included a couple of convenience functions that take you directly to an interactive tool with variable and region details on the [Censusmapper website](https://censusmapper.ca/api). To explore the hierarchical variable structure of a given dataset, say the 2006 Census, running `explore_census_vectors(dataset = "CA06")`. To view Census geography on an interactive map, there is `explore_census_regions(dataset = "CA16")`. As usual, vectors and geographies for different Census datasets can be retrieved by using the appropriate dataset code for the `dataset` argument. 

---
title: "Finding intersecting geometries from custom data"
subtitle: "Bring custom data and identify corresponding census geographies"
output: rmarkdown::html_vignette
mainfont: Roboto
vignette: >
  %\VignetteIndexEntry{Finding intersecting geometries from custom data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE, 
  warning = FALSE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)
```

A frequent application of census data is to evaluate values for specific areas that do not necessarily correspond to existing boundaries or administrative units. Census users may have their own defined geographies or other geospatial data of interest and want to be able to quickly and easily identify the collection of census features that correspond to that region.

The `get_intersecting_geometries()` function is projection agnostic and accepts any valid `sf` or `sfc` class object as input. These objects are then reprojected into lat/lon coordinates on the backend to facilitate the intersecting join on the server-side. 

## A simple example

As an example, suppose we are interested in understanding the housing tenure split in census tracts located near Vancouver Skytrain (rapid transit) stations. We can use the `COV_SKYTRAIN_STATIONS` dataset that ships with the package and is derived from the [City of Vancouver Open Data portal](https://opendata.vancouver.ca/explore/dataset/rapid-transit-stations/information/) and contains their locations. For our example we are interested in census tracts within 800m of these stations, which ships with the package.

```{r setup}
library(cancensus)
library(dplyr)
library(ggplot2)
library(sf)
```

We load the example data `COV_SKYTRAIN_STATIONS` from the package. 

```{r}
cov_station_buffers <- COV_SKYTRAIN_STATIONS %>%
  st_set_crs(4326)  # needed for Ubuntu or systems with old GDAL but can otherwise be ignored
```

We then use the `get_intersecting_geometries` call to obtain the list of municipalities (CSDs) and census tracts (CTs) that intersect the 800m station buffer objects. 

```{r}
station_city_ids <- get_intersecting_geometries("CA16", level = "CSD", geometry = cov_station_buffers,
                                                quiet=TRUE)
station_ct_ids <- get_intersecting_geometries("CA16", level = "CT", geometry = cov_station_buffers,
                                              quiet=TRUE)
```

These return a list of census geographic identifiers suitable for use in the 'region' argument in `get_census`. We may be interested in the transit to work mode share in each of these buffers.

```{r}
variables <- c(mode_base="v_CA16_5792",transit="v_CA16_5801",walk="v_CA16_5804")
station_city <-  get_census("CA16", regions = station_city_ids, vectors = variables, 
                            geo_format = 'sf', quiet=TRUE) %>% 
  filter(name == "Vancouver (CY)")
station_cts <-   get_census("CA16", regions = station_ct_ids, vectors = variables, 
                            geo_format = 'sf', quiet=TRUE)
```

To understand how these relate we plot the data.

```{r}
ggplot(station_city) +
  geom_sf(fill=NA) +
  geom_sf(data=station_cts,aes(fill=((walk+transit)/mode_base))) +
  geom_sf(data=cov_station_buffers,fill=NA,alpha=0.5,color="steelblue") +
  scale_fill_viridis_c(option="magma",labels=scales::percent) +
  coord_sf(datum=NA) +
  labs(title="CTs within City of Vancouver skytrain station catchments",
       fill="Walk or\ntransit\nto work",
       caption="StatCan Census 2016")
```

To get a closer match we can cut out the dissemination areas intersecting the station catchment areas. 

```{r}
station_das <-   get_intersecting_geometries("CA16", level = "DA", geometry = cov_station_buffers,
                                             quiet=TRUE) %>% 
  get_census("CA16", regions = ., vectors=variables, geo_format = 'sf', quiet=TRUE)

ggplot(station_city) +
  geom_sf(fill=NA) +
  geom_sf(data=station_das,aes(fill=((walk+transit)/mode_base))) +
  geom_sf(data=cov_station_buffers,fill=NA,alpha=0.5,color="steelblue") +
  scale_fill_viridis_c(option="magma",labels=scales::percent) +
  coord_sf(datum=NA) +
  labs(title="DAs within City of Vancouver skytrain station catchments",
       fill="Walk or\ntransit\nto work",
       caption="StatCan Census 2016")

```

However, API points for `get_intersecting_geometries` are quite limited at this point, an alternative way to obtain the same data is to first query all DAs withing the CTs identified by the previous `get_intersecting_geometries` call and then filter down to those intersecting the station buffers.

```{r}
station_das2 <- get_census("CA16", regions = station_ct_ids, vectors=variables, 
                           geo_format = 'sf', level="DA", quiet=TRUE) %>%
  sf::st_filter(cov_station_buffers)

ggplot(station_city) +
  geom_sf(fill=NA) +
  geom_sf(data=station_das2,aes(fill=((walk+transit)/mode_base))) +
  geom_sf(data=cov_station_buffers,fill=NA,alpha=0.5,color="steelblue") +
  scale_fill_viridis_c(option="magma",labels=scales::percent) +
  coord_sf(datum=NA) +
  labs(title="DAs within City of Vancouver skytrain station catchments",
       fill="Walk or\ntransit\nto work",
       caption="StatCan Census 2016")

```

We may increase the default quotas for the `get_intersecting_geometries` call at some point, but while we throttling API usage and monitor server impacts of the new functionality it may be preferable to use the `get_intersecting_geometries` call for higher level geographies only and add a few lines of code to do the final bit of filtering in R. 

## Addendum

We may take this further by estimate values of census variables strictly within catchments areas. Rather than intersecting, some adjustments for spatial disaggregation and interpolation are needed. The `tongfen_estimate` method from the [`tongfen` package](https://mountainmath.github.io/tongfen/index.html) is useful in this case. This is a related package that is designed to work in tandem with `cancensus` in order to facilitate census geography aggregation and is designed to make census data comparable across several censuses.

---
title: "Additional datasets: Annual T1FF taxfiler data"
subtitle: "Dataset background and example usage"
output: rmarkdown::html_vignette
mainfont: Roboto
vignette: >
  %\VignetteIndexEntry{Additional datasets: Annual T1FF taxfiler data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## Background

Through collaboration with the [Canadian Mortgage and Housing Corporation](https://www.cmhc-schl.gc.ca/) (CMHC), CensusMapper has added and open-sourced annual T1FF taxfiler data which provides an annual look at some basic demographic variables. Data is available via the `cancensus` package for the years 2001 through 2017. The T1FF dataset contains information on: 

* individual income
* government transfers
* family income
* family composition
* taxfilers by age groups
* taxfilers and dependents by age groups
* marital status and select other demographic variables

The data comes in varying Census geographies, depending on the year. Retrieving any annual dataset via `get_census` will automatically reference to the correct Census geography and attach the correct spatial boundaries.

The taxfiler data is organized with consistent internal referencing. The identifier for the number of families in low income in 2017 is "v_TX2017_786" and that for all families is "v_TX2017_607", and the ones for the other years are given by simply swapping out the year. This makes the variables selection process easy. 

## Example usage: constructing a multi-year series of families in low-income status

As an example we will explore a multi-year time series for families in low income. Data on low income families is available for years 2004 and later, we will start with 2006 just so that the data fits on a nice grid.

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)
```

```{r setup}
# Packages used for example
library(cancensus)
library(dplyr)
library(tidyr)
library(ggplot2)
library(sf)
```

To see all available T1FF datasets and their reference codes we can use `list_census_datasets()`.

```{r}
list_census_datasets() %>% 
  filter(grepl("taxfiler",description))
```
And, as an example, available data vectors for one such T1FF dataset.
```{r}
list_census_vectors('TX2017')
```
This particular dataset has over 800 individual vectors. The vector codes follow a regular pattern across different years, and we can use this to quickly identify all the relevant variables of interest across multiple datasets. We can utilized the [CensusMapper graphical variable selection interface](https://censusmapper.ca/api/CA16#api_variable), which can also be reached by calling `explore_census_vectors()` from the R console. For this example we are interested in low income families and note that the internal CensusMapper vector for all families is of the form *v_TX<year>_607* and that for all families in low income is *v_TX<year>_786*.

While the geography varies across Census periods, the call to `get_census` will automatically attach the correct geography for each annual dataset. We pick four years to look at low income families.

```{r}
years <- c(2006,2011,2014,2018)
# Attribution for the dataset to be used in graphs
attribution <- dataset_attribution(paste0("TX",years))

plot_data <- years %>%
  lapply(function(year) {
    dataset <- paste0("TX",year)
    vectors <- c("Families"=paste0("v_",dataset,"_607"),
                 "CFLIM-AT"=paste0("v_",dataset,"_786"))
    
    get_census(dataset,regions=list(CMA="59933"),vectors = vectors,
                    geo_format = 'sf', level="CT", quiet = TRUE) %>%
      select(c("GeoUID",names(vectors))) %>%
      mutate(Year=year)
  }) %>%
  bind_rows() %>%
  mutate(share=`CFLIM-AT`/Families)
```

Here we also re-organized the data by year. All that's left is to plot the data, one year at a time.

```{r}
ggplot(plot_data,aes(fill=share)) +
  geom_sf(size=0.1,color="white") +
  facet_wrap("Year") +
  scale_fill_viridis_c(labels=scales::percent,option = "inferno",
                       trans="log",breaks = c(0.05,0.1,0.2,0.4)) +
  coord_sf(datum=NA,xlim=c(-123.4, -122.5), ylim=c(49.01, 49.4)) +
  labs(title="Share of census families in low income",fill="Share",
       caption=attribution)
```

We may be tempted to re-arrange the data to create timelines, but we have to be careful as census geographies change over time. Inspecting the dataset tables at the top informs us that the 2006 through 2011 data all come on the common 2006 census geography, so the 2006 and 2011 tax data are directly comparable.

```{r}
change_data <- plot_data %>% 
  filter(Year==2006) %>% 
  select(GeoUID,`2006`=share) %>%
  left_join(plot_data %>%
              st_set_geometry(NULL) %>%
              filter(Year==2011) %>% 
              select(GeoUID,`2011`=share),
            by="GeoUID") %>%
  mutate(change=`2011`-`2006`)
  
ggplot(change_data,aes(fill=change)) +
  geom_sf(size=0.1) +
  scale_fill_gradient2(labels=scales::percent) +
  #scale_fill_viridis_c(labels=scales::percent,option = "inferno") +
  coord_sf(datum=NA,xlim=c(-123.4, -122.5), ylim=c(49.01, 49.4)) +
  labs(title="Change in share of census families in low income 2006-2011",fill="Percentage\npoint change",caption=dataset_attribution(paste0("TX",c(2006,2011))))
```

Analyzing change over longer timelines that span changes in Census geometries involves more work, the [tongfen package](https://mountainmath.github.io/tongfen/index.html) facilitates this and provides a convenient interface for generating timelines spanning geometries from several Census years.

---
title: "cancensus"
author: "" 
date: ""
output: rmarkdown::html_vignette
mainfont: Roboto

vignette: >
  %\VignetteIndexEntry{cancensus}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, echo=FALSE, message=FALSE, warning=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = nzchar(Sys.getenv("COMPILE_VIG"))
)
library(cancensus)
library(dplyr)
# set_cancensus_api_key(<your_api_key>', install = TRUE)
# set_cancensus_cache_path(<local cache path>, install = TRUE)
```
# Cancensus and CensusMapper

The **cancensus** package was developed to provide users with a way to access Canadian Census in a programmatic way following good [tidy data](http://vita.had.co.nz/papers/tidy-data.pdf) practices. While the structure and data in **cancensus** is unique to Canadian Census data, this package is inspired in part by [tidycensus](https://github.com/walkerke/tidycensus), a package to interface with the US Census Bureau data APIs. 

As Statistics Canada does not provide direct API access to Census data, **cancensus** retrieves Census data indirectly through the [CensusMapper](https://censusmapper.ca) API. CensusMapper is a project by [Jens von Bergmann](https://github.com/mountainMath), one of the authors of **cancensus**, to provide interactive geographic visualizations of Canadian Census data. CensusMapper databases store all publicly available data from Statistics Canada for the 2006, 2011, and 2016 Censuses. CensusMapper data can be accessed via an API and **cancensus** is built to interface directly with it. 

## API Key

**cancensus** requires a valid CensusMapper API key to use. You can obtain a free API key by [signing up](https://censusmapper.ca/users/sign_up) for a CensusMapper account. CensusMapper API keys are free and public API quotas are generous; however, due to incremental costs of serving large quantities of data, there limits to API usage in place. For most use cases, these API limits should not be an issue. Production uses with large extracts of fine grained geographies may run into API quota limits. For larger quotas, please get in touch with Jens [directly](mailto:jens@censusmapper.ca).  

To check your API key, just go to "Edit Profile" (in the top-right of the CensusMapper menu bar). Once you have your key, you can store it in your system environment so it is automatically used in API calls. To do so just enter `set_cancensus_api_key(<your_api_key>, install = TRUE)`

## Installing cancensus

The stable version of **cancensus** can be easily installed from CRAN.
```{r load_package_cran, echo=TRUE, message=FALSE, warning=FALSE, eval = FALSE}
install.packages("cancensus")

library(cancensus)
```

Alternatively, the latest development version can be installed from Github using `remotes``. 
```{r load_package_git, echo=TRUE, message=FALSE, warning=FALSE, eval = FALSE}
# install.packages("devtools")
remotes::install_github("mountainmath/cancensus")

library(cancensus)
```

If you have not already done so, you can install the API keys and the data cache path. You can get your free API key when you sign up for a [CensusMapper account](https://censusmapper.ca/) and check your profile. Additionally we recommend you set a permanent data cache path so that census data you query is stored persistently across sessions. **Make sure to replace the filler text `<your_api_key>` and `<local cache path>` with your actual API key and local cache path.**

```{r install_api_key_and_cache_path, echo=TRUE, message=FALSE, warning=FALSE, eval = FALSE}
# Only need to install api key can cache path once
set_cancensus_api_key('<your_api_key>', install = TRUE)
set_cancensus_cache_path('<local cache path>', install = TRUE)
```

Data in the persistent cache can be managed via the functions `list_cancensus_cache` and `remove_from_cancensus_cache` if needed.

# Accessing Census Data

**cancensus** provides three different functions for retrieving Census data:
* `get_census` to retrieve Census data and geography as a spatial dataset
* `get_census_data` to retrieve Census data only as a flat data frame
* `get_census_geometry` to retrieve Census geography only as a collection of spatial polygons.

`get_census` takes as inputs a dataset parameter, a list of specified regions, a vector of Census variables, and a Census geography level. You can specify one of three options for spatial formats: `NA` to return data only, `sf` to return an sf-class data frame, or `sp` to return a SpatialPolygonsDataFrame object. 

```{r get_census example, echo=TRUE, warning=FALSE, message=FALSE}
# Returns a data frame with data only
census_data <- get_census(dataset='CA21', regions=list(CMA="59933"),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, geo_format = NA, quiet = TRUE)

# Returns data and geography as an sf-class data frame
census_data <- get_census(dataset='CA21', regions=list(CMA="59933"),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, geo_format = 'sf', quiet = TRUE)

# Returns a SpatialPolygonsDataFrame object with data and geography
census_data <- get_census(dataset='CA21', regions=list(CMA="59933"),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, geo_format = 'sp', quiet = TRUE)
```

**cancensus** utilizes caching to increase speed, minimize API token usage, and to make data available offline. Downloaded data is hashed and stored locally so if a call is made to access the same data, **cancensus** will read the local version instead. To force **cancensus** to refresh the data, specify `use_cache = FALSE` as a parameter for `get_census`. 

Additional parameters for advanced options can be viewed by running `?get_census`.

## Census Datasets

**cancensus** can access Statistics Canada Census data for Census years 1996, 2001, 2006, 2011, 2016, and 2021. You can run `list_census_datasets` to check what datasets are currently available for access through the CensusMapper API. 

Thanks to contributions by the Canada Mortgage and Housing Corporation (CMHC), **cancensus** now includes additional Census-linked datasets as open-data releases. These include annual taxfiler data at the census tract level for tax years 2000 through 2017, which includes data on incomes and demographics, as well as specialized crosstabs for Structural type of dwelling by Document type, which details occupancy status for residences. These crosstabs are available for the 2001, 2006, 2011, and 2016 Census years at all levels starting with census tract.

The function `list_census_datasets()` will show all available datasets alongside their metadata.

```{r list datasets, message=FALSE, warning=FALSE}
list_census_datasets()
```

As other Census datasets become available via the CensusMapper API, they will be listed as output when calling `list_census_datasets()`. 

## Census Regions

Census data is aggregated at multiple [geographic levels](### Census Geographic Levels). Census geographies at the national (C), provincial (PR), census metropolitan area (CMA), census agglomeration (CA), census division (CD), and census subdivision (CSD) are defined as named census regions. 

Canadian Census geography can change in between Census periods. **cancensus** provides a function, `list_census_regions(dataset)`, to display all named census regions and their corresponding id for a given census dataset. 

```{r list regions, message=FALSE, warning=FALSE}
list_census_regions("CA21")
```

The `regions` parameter in `get_census` requires as input a list of region id strings that correspond to that regions geoid. You can combine different regions together into region lists.

```{r, message=FALSE, warning=FALSE}
# Retrieves Vancouver and Toronto
list_census_regions('CA21') %>% 
  filter(level == "CMA", name %in% c("Vancouver","Toronto"))

census_data <- get_census(dataset='CA21', regions=list(CMA=c("59933","35535")),
                          vectors=c("v_CA21_434","v_CA21_435","v_CA21_440"),
                          level='CSD', use_cache = FALSE, quiet = TRUE)
```

## Census Geographic Levels

Census data accessible through **cancensus** comes is available in a number of different aggregation levels including: 

| Code    | Description                    | Count in Census 2016 |
| --------|--------------------------------|:--------------------:|
| C       | Canada (total)                 | 1                    |
| PR      | Provinces/Territories          | 13                   |
| CMA     | Census Metropolitan Area       | 35                   |
| CA      | Census Agglomeration           | 14                   |
| CD      | Census Division                | 287                  |
| CSD     | Census Subdivision             | 713                  |
| CT      | Census Tracts                  | 5621                 |
| DA      | Dissemination Area             | 56589                |
| EA      | Enumeration Area (1996 only)   | -                    |
| DB      | Dissemination Block (2001-2016)| 489676               |
| Regions | Named Census Region            |                      |

Selecting `regions = "59933"` and `level = "CT"` will return data for all 478 census tracts in the Vancouver Census Metropolitan Area. Selecting `level = "DA"` will return data for all 3450 dissemination areas and selecting `level = "DB"` will retrieve data for 15,197 dissemination block. Working with CT, DA, EA, and especially DB level data significantly increases the size of data downloads and API usage. To help minimize additional overhead, **cancensus** supports local data caching to reduce usage and load times. 

Setting `level = "Regions"` will produce data strictly for the selected region without any tiling of data at lower census aggregation levels levels. 

# Working with Census Variables

Census data contains thousands of different geographic regions as well as thousands of unique variables. In addition to enabling programmatic and reproducible access to Census data, **cancensus** has a number of tools to help users find the data they are looking for. 

## Displaying available Census variables

Run `list_census_vectors(dataset)` to view all available Census variables for a given dataset. 

```{r list_vectors, message=FALSE, warning=FALSE}
list_census_vectors("CA21")
```

## Variable characteristics

For each variable (vector) in that Census dataset, this shows:

* Vector: short variable code 
* Type: variables are provided as aggregates of female responses, male responses, or total (male+female) responses
* Label: detailed variable name
* Units: provides information about whether the variable represents a count integer, a ratio, a percentage, or a currency figure
* Parent_vector: shows the immediate hierarchical parent category for that variable, where appropriate
* Aggregation: indicates how the variable should be aggregated with others, whether it is additive or if it is an average of another variable
* Description: a rough description of a variable based on its hierarchical structure. This is constructed by **cancensus** by recursively traversing the labels for every variable's hierarchy, and facilitates searching for specific variables using key terms. 

## Variable search

Each Census dataset features numerous variables making it a bit of a challenge to find the exact variable you are looking for. There is a function, `find_census_vectors()`, for searching through Census variable metadata in a few different ways. There are three types of searches possible using this function: exact search, which simply looks for exact string matches for a given query against the vector dataset; keyword search, which breaks vector metadata into unigram tokens and then tries to find the vectors with the greatest number of unique matches; and, semantic search which works better with search phrases and has tolerance for inexact searches. Switching between search modes is done using the `query_type` argument when calling `find_census_variables()` function. 

```{r search_vectors1, message=FALSE, warning=FALSE}
# Find the variable indicating the number of people of Austrian ethnic origin
find_census_vectors("Australia", dataset = "CA16", type = "total", query_type = "exact")

find_census_vectors("Australia origin", dataset = "CA16", type = "total", query_type = "semantic")

find_census_vectors("Australian ethnic", dataset = "CA16", type = "total", query_type = "keyword", interactive = FALSE)
```

## Managing variable hierarchy 

Census variables are frequently hierarchical. As an example, consider the variable for the number of people of Austrian ethnic background. We can select that vector and quickly look up its entire hierarchy using `parent_census_vectors` on a vector list.  

```{r parent_vectors, message=FALSE, warning=FALSE}
list_census_vectors("CA16") %>% 
  filter(vector == "v_CA16_4092") %>% 
  parent_census_vectors()
```

Sometimes we want to traverse the hierarchy in the opposite direction. This is frequently required when looking to compare different variable stems that share the same aggregate variable. As an example, if we want to look the total count of Northern European ethnic origin respondents disaggregated by individual countries, it is pretty easy to do so. 

```{r child_vectors1, message=FALSE, warning=FALSE}
# Find the variable indicating the Northern European aggregate
find_census_vectors("Northern European", dataset = "CA16", type = "Total")
```

The search result shows that the vector **v_CA16_4092** represents the aggregate for all Northern European origins. The `child_census_vectors` function can return a list of its constituent underlying variables.

```{r child_vectors2, message=FALSE, warning=FALSE}
# Show all child variable leaves
list_census_vectors("CA16") %>% 
  filter(vector == "v_CA16_4122") %>% child_census_vectors(leaves = TRUE)
```

The `leaves = TRUE` parameter specifies whether intermediate aggregates are included or not. If `TRUE` then only the lowest level variables are returns - the "leaves" of the hierarchical tree. 

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geo_suite.R
\name{get_statcan_geographic_attributes}
\alias{get_statcan_geographic_attributes}
\title{Read the Geographic Attributes File}
\usage{
get_statcan_geographic_attributes(census_year = "2021", refresh = FALSE)
}
\arguments{
\item{census_year}{census year to get the data for, right now only 2006, 2011, 2016, 2021 are supported}

\item{refresh}{(logical) refresh the cache if true}
}
\value{
tibble with the relationship data
}
\description{
Reads the Geographies Attributes File for the given census year. The table contains
the information on how all the geographic levels are related for each area, and population, dwelling and household counts.
Data gets cached after first download if the
cancensus cache path has been set. A reference guide is available
at https://www150.statcan.gc.ca/n1/en/catalogue/92-151-G2021001
}
\examples{
# list add the cached census data
\dontrun{
get_statcan_geographic_attributes("2021")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/census_regions.R
\name{search_census_regions}
\alias{search_census_regions}
\title{Query the CensusMapper API for regions with names matching a searchterm.}
\usage{
search_census_regions(searchterm, dataset, level = NA, ...)
}
\arguments{
\item{searchterm}{The term to search for e.g. \code{"Victoria"}.
Search terms are case insensitive. If unable to find a given search term,
this function will suggest the correct spelling to use when possible.}

\item{dataset}{The dataset to query for available regions, e.g.
\code{"CA16"}.}

\item{level}{One of \code{NA}, \code{'C'}, \code{'PR'}, \code{'CMA'}, \code{'CD'}, or \code{'CSD'}.
If specified, only return variables of specified `level`.}

\item{...}{Further arguments passed on to \code{\link{list_census_regions}}.}
}
\value{
A census region list of the same format as `list_census_regions()` containing the matches.
}
\description{
Runs a query against the CensusMapper API to retrieve region data with
names matching specific queries. Users can optionally specify the target geography level
(e.g. \code{level = 'CMA'}, \code{level = 'CSD'}, etc.). Alternatively, calling
\code{explore_census_vectors()} will launch the interactive region selection tool on
the CensusMapper site in a new web page or tab.
}
\examples{
\dontrun{
# This will return a warning that no match was found, but will suggest similar named regions.
search_census_regions('Victorea', 'CA16')

# This will limit region results to only include CMA level regions
search_census_regions('Victoria', 'CA16', level = "CMA")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cancensus_cache.R
\name{list_cancensus_cache}
\alias{list_cancensus_cache}
\title{List cached files}
\usage{
list_cancensus_cache()
}
\value{
tibble with metadata on cached data
}
\description{
Lists all cached data and metadata if available
}
\examples{
# list add the cached census data
list_cancensus_cache()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/user_settings.R
\name{show_cancensus_cache_path}
\alias{show_cancensus_cache_path}
\title{View saved cache directory path}
\usage{
show_cancensus_cache_path()
}
\description{
View saved cache path'
}
\examples{
show_cancensus_cache_path()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cancensus.R
\name{label_vectors}
\alias{label_vectors}
\title{Return Census variable names and labels as a tidy data frame}
\usage{
label_vectors(x)
}
\arguments{
\item{x}{A data frame, \code{sp} or \code{sf} object returned from
\code{get_census} or similar.}
}
\value{
A data frame with a column \code{variable} containing the truncated
variable name, and a column \code{label} describing it.
}
\description{
Return Census variable names and labels as a tidy data frame
}
\examples{
\dontrun{
# Query census data with truncated labels:
label_data <- get_census(dataset='CA16', regions=list(CMA="59933"),
                          vectors=c("v_CA16_408","v_CA16_409","v_CA16_410"),
                          level='CSD', geo_format = "sf", labels="short")

# Get details for truncated vectors:
label_vectors(label_data)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\docType{data}
\name{COV_SKYTRAIN_STATIONS}
\alias{COV_SKYTRAIN_STATIONS}
\title{A dataset City of Vancouver skytrain station locations}
\description{
A dataset City of Vancouver skytrain station locations
}
\references{
\url{https://opendata.vancouver.ca/explore/dataset/rapid-transit-stations/information/}
}
\author{
City of Vancouver Open Data
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/user_settings.R
\name{show_cancensus_api_key}
\alias{show_cancensus_api_key}
\title{View saved Censusmapper API key}
\usage{
show_cancensus_api_key()
}
\description{
View saved API key'
}
\examples{
show_cancensus_api_key()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geo_suite.R
\name{get_statcan_geo_suite}
\alias{get_statcan_geo_suite}
\title{Read the geosuite data}
\usage{
get_statcan_geo_suite(level, census_year = "2021", refresh = FALSE)
}
\arguments{
\item{level}{geographic level to return the data for, valid choices are
"DB", "DA", "ADA", "CT", "CSD", "CMA", "CD", "PR", "FED", "DPL", "ER", "PN", "POPCTR"}

\item{census_year}{census year to get the data for, right now only 2021 is supported}

\item{refresh}{(logical) refresh the cache if true}
}
\value{
tibble with the geosuite data
}
\description{
Reads the geosuite data for the given level and census year. Data gets cached after first download if the
cancensus cache path has been set. For older
years `get_statcan_geographic_attributes()` can fill in most of the information
}
\examples{
# list add the cached census data
\dontrun{
get_statcan_geo_suite("DA","2021")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vector_discovery.R
\name{find_census_vectors}
\alias{find_census_vectors}
\title{Query the CensusMapper API for vectors using exact, semantic, or keyword search}
\usage{
find_census_vectors(query, dataset, type = "all", query_type = "exact", ...)
}
\arguments{
\item{query}{The term or phrase to search for e.g. \code{'Oji-cree'}.
Search queries are case insensitive.}

\item{dataset}{The dataset to query for available vectors, e.g. \code{'CA16'}.
To see a list of available datasets: \code{list_census_datasets()}}

\item{type}{One of \code{'all'}, \code{'total'}, \code{'male'} or \code{'female'}.
If specified, only return aggregations of specified `type`. By default, only
the \code{'total'} aggregation will be returned.}

\item{query_type}{One of \code{exact}, \code{'semantic'} or \code{'keyword'}.
By default, assumes exact string matching, but the alternatives may be better
options in some cases. See description section for more details on query types.}

\item{...}{Other arguments passed to internal functions.}
}
\description{
Query the available list of Census vectors based on their label and return
details including vector code. Default search behaviour expects an exact match, but
keyword or semantic searches can be used instead by setting \code{query_type='keyword'} or
\code{query_type = 'semantic'} instead. Keyword search is useful when looking to explore
Census vectors based on broad themes like "income" or "language". Keyword search separates
the query into unigrams and returns Census vectors with matching words, ranked by incidence
of matches. Semantic search is designed for more precise searches while allowing room for error
for spelling or phrasing, as well as for finding closely related vector matches. Semantic search
separates the query into n-grams and relies on string distance measurement using a generalized
Levenshtein distance approach.

Some census vectors return population counts segmented by \code{Female} and \code{Male} populations, in
addition to a total aggregate. By default, query matches will return matches for the \code{Total}
aggregation, but can optionally return only the \code{Female} or \code{Male} aggregations by adding
\code{type = 'female'} or \code{type = 'male'} as a parameter.
}
\examples{
\dontrun{
find_census_vectors('Oji-cree', dataset = 'CA16', type = 'total', query_type = 'exact')

find_census_vectors('commuting duration', dataset = 'CA11', type = 'female', query_type = 'keyword')

find_census_vectors('after tax income', dataset = 'CA16', type = 'total', query_type = 'semantic')

# This incorrect spelling will return a warning that no match was found,
# but will suggest trying semantic or keyword search.
find_census_vectors('Ojibwey', dataset = 'CA16', type = 'total')

# This will find near matches as well
find_census_vectors('Ojibwey', dataset = 'CA16', type = 'total', query_type = "semantic")

find_census_vectors('commute duration', dataset = 'CA16', type = 'female', query_type = 'keyword')

find_census_vectors('commute duration', dataset = 'CA11', type = 'all', query_type = 'keyword')

find_census_vectors('ukrainian origin', dataset = 'CA16', type = 'total', query_type = 'keyword')

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geo_suite.R
\name{get_statcan_geography_relationships}
\alias{get_statcan_geography_relationships}
\title{Read the Dissemination Geographies Relationship File}
\usage{
get_statcan_geography_relationships(census_year = "2021", refresh = FALSE)
}
\arguments{
\item{census_year}{census year to get the data for, right now only 2021 is supported, for older
years `get_statcan_geographic_attributes()` can fill in most of the information}

\item{refresh}{(logical) refresh the cache if true}
}
\value{
tibble with the relationship data
}
\description{
Reads the Dissemination Geographies Relationship File for the given census year. The table contains
the information on how all the geographic levels are related for each area. Data gets cached after first download if the
cancensus cache path has been set. A reference guide is available
at https://www150.statcan.gc.ca/n1/en/catalogue/982600032021001
}
\examples{
# list add the cached census data
\dontrun{
get_statcan_geography_relationships("2021")
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/census_regions.R
\name{add_unique_names_to_region_list}
\alias{add_unique_names_to_region_list}
\title{Convenience function for creating unique names from region list}
\usage{
add_unique_names_to_region_list(region_list)
}
\arguments{
\item{region_list}{a subset of a regions list as gotten from `list_census_regions()`}
}
\value{
The same list of regions with an extra column `Name` with de-duplicated names.
}
\description{
Names of municipalities are not always unique, especially at the CSD level. This function
takes as input a subset of a regions list as generated from `list_census_regions()` and de-duplicates names as
needed by adding the municipal status in parenthesis. If this does not de-duplicate the name then the
geographic identifier will be further added in parenthesis behind that.
}
\examples{
\dontrun{
# This will return a warning that no match was found, but will suggest similar named regions.
library(dplyr)
list_census_regions("CA21") \%>\%
  filter(level=="CSD", CMA_UID=="59933") \%>\%
  add_unique_names_to_region_list()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/census_regions.R
\name{as_census_region_list}
\alias{as_census_region_list}
\title{Convert a (suitably filtered) data frame from
\code{\link{list_census_regions}} to a list suitable for passing to
\code{\link{get_census}}.}
\usage{
as_census_region_list(tbl)
}
\arguments{
\item{tbl}{A data frame, suitably filtered, as returned by
\code{\link{list_census_regions}}.}
}
\description{
Convert a (suitably filtered) data frame from
\code{\link{list_census_regions}} to a list suitable for passing to
\code{\link{get_census}}.
}
\examples{
\dontrun{
library(dplyr, warn.conflicts = FALSE)

# Query the CensusMapper API for the total occupied dwellings
# of 20 random Census Subdivisions, in Census 2016.
regions <- list_census_regions("CA16") \%>\%
  filter(level == "CSD") \%>\%
  sample_n(20) \%>\%
  as_census_region_list()

occupied <- get_census("CA16", regions = regions,
                            vectors = c("v_CA16_408"),
                            level = "Regions")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/census_regions.R
\name{list_census_regions}
\alias{list_census_regions}
\title{Query the CensusMapper API for available regions in a given dataset.}
\usage{
list_census_regions(dataset, use_cache = TRUE, quiet = FALSE)
}
\arguments{
\item{dataset}{The dataset to query for available regions, e.g.
\code{"CA16"}.}

\item{use_cache}{If set to TRUE (the default), data will be read from a local cache
that is maintained for the duration of the R session, if
available. If set to FALSE, query the API for the data, and
refresh the local cache with the result.}

\item{quiet}{When TRUE, suppress messages and warnings.}
}
\value{
Returns a data frame with the following columns:

\describe{
  \item{\code{region}}{The region identifier.}

  \item{\code{name}}{The name of that region.}

  \item{\code{level}}{The census aggregation level of that region.}

  \item{\code{pop}}{The population of that region.}

  \item{\code{municipal_status}}{Additional identifiers to distinguish the
    municipal status of census subdivisions.}

  \item{\code{CMA_UID}}{The identifier for the Census Metropolitan Area the region is in (if any).}

  \item{\code{CD_UID}}{The identifier for the Census District the region is in (if any).}

  \item{\code{PR_UID}}{The identifier for the Province the region is in (if unique).}
}
}
\description{
Query the CensusMapper API for available regions in a given dataset.
}
\examples{
\dontrun{
list_census_regions('CA16')
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/census_vectors.R
\name{child_census_vectors}
\alias{child_census_vectors}
\title{List all child variables from vector hierarchies given either a list of Census
variables returned by
\code{list_census_vectors}, \code{search_census_vectors}, \code{find_census_vectors}, or a direct string reference to the vector code.}
\usage{
child_census_vectors(
  vector_list,
  leaves_only = FALSE,
  max_level = NA,
  keep_parent = FALSE
)
}
\arguments{
\item{vector_list}{the list of vectors to be used, either a character vector or a filtered tibble
as returned from \code{list_census_vectors}.}

\item{leaves_only}{boolean flag to indicate if only final leaf vectors should be returned,
i.e. terminal vectors that themselves do not have children.}

\item{max_level}{optional, maximum depth to look for child vectors. Default is \code{NA} and will return all
child census vectors.}

\item{keep_parent}{optional, also return parent vector in list of results. Default is set to \code{FALSE}.}
}
\description{
List all child variables from vector hierarchies given either a list of Census
variables returned by
\code{list_census_vectors}, \code{search_census_vectors}, \code{find_census_vectors}, or a direct string reference to the vector code.
}
\examples{
# Query parent vectors directly using vector identifier
child_census_vectors("v_CA16_2510")

\dontrun{

# Example using multiple vectors coerced into a list
child_census_vectors(c("v_CA16_2510","v_CA16_2511","v_CA16_2512"))

# or, equivalently
selected_vectors <- c("v_CA16_2510","v_CA16_2511","v_CA16_2512")
child_census_vectors(selected_vectors)

# Example using dplyr and piped arguments
library(dplyr, warn.conflicts = FALSE)

list_census_vectors("CA16") \%>\%
  filter(vector == "v_CA16_2510") \%>\%
  child_census_vectors(TRUE)

# this will return the equivalent of c("v_CA16_2510", child_census_vectors("v_CA16_2510"))
list_census_vectors("CA16") \%>\%
  filter(vector == "v_CA16_2510") \%>\%
  child_census_vectors(TRUE, keep_parent = TRUE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wds.R
\name{get_statcan_wds_metadata}
\alias{get_statcan_wds_metadata}
\title{Query the StatCan WDS for metadata}
\usage{
get_statcan_wds_metadata(census_year, level, version = NULL, refresh = FALSE)
}
\arguments{
\item{census_year}{census year to get the data for, right now only 2021 is supported}

\item{level}{geographic level to return the data for, valid choices are
"PR","CD","CMACA","CSD","CT","ADA","DA","ER","FED","DPL","POPCNTR", "FSA", "HR"}

\item{version}{optionally specify a version of the data to download. For example, for FED level data, version 1.3 will
access the 2013 represenation order, whereas version 2.0 will access the 2023 representation order. By default the latest
available version is accessed.}

\item{refresh}{default is `FALSE` will refresh the temporary cache if `TRUE`}
}
\value{
tibble with the metadata
}
\description{
Get official metadata information from Statistics Canada for a given geographic level. Only available for the 2021 census.
Data is cached for the duration of the R session.
}
\examples{
# get metadata for federal electoral districts
\dontrun{
get_statcan_wds_metadata(census_year="2021",level="FED")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wds.R
\name{get_statcan_wds_data}
\alias{get_statcan_wds_data}
\title{Query the StatCan WDS for data}
\usage{
get_statcan_wds_data(
  DGUIDs,
  members = NULL,
  version = NULL,
  gender = "All",
  language = "en",
  refresh = FALSE
)
}
\arguments{
\item{DGUIDs}{census year to get the data for, right now only 2021 is supported. Valid DGUIDs for a given geographic
level can be queried via `get_statcan_wds_metadata()`.}

\item{members}{list of Member IOs to download data for. By default all characteristics are downloaded. Valid
Member IDs and their descriptions can be queried via the `get_statcan_wds_metadata()` call.}

\item{version}{optionally specify a version of the data to download. For example, for FED level data, version 1.3 will
access the 2013 represenation order, whereas version 2.0 will access the 2023 representation order. By default the latest
available version is accessed.}

\item{gender}{optionally query data for only one gender. By default this queries data for all genders, possible
values are "Total", "Male", "Female" to only query total data, or for males only or for females only.}

\item{language}{specify language for geography and characteristic names that get added, valid choices are "en" and "fr"}

\item{refresh}{default is `FALSE` will refresh the temporary cache if `TRUE`}
}
\value{
tibble with the enriched census data
}
\description{
Get official census data from Statistics Canada for a given set of DGUIDs. Only available for the 2021 census. The
downloaded data gets enriched by geographic and characteristic names based on metadata obtained via `get_statcan_wds_metadata()`.
Data is cached for the duration of the R session.
}
\examples{
# get data for federal electoral district 2013A000459021
\dontrun{
get_statcan_wds_data(DGUIDs="2013A000459021",level="FED")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/census_vectors.R
\name{parent_census_vectors}
\alias{parent_census_vectors}
\title{List all parent variables from vector hierarchies given either a list of Census
variables returned by
\code{list_census_vectors}, \code{search_census_vectors}, \code{find_census_vectors}, or a direct string reference to the vector code.}
\usage{
parent_census_vectors(vector_list)
}
\arguments{
\item{vector_list}{The list of vectors to be used, either a character vector or a filtered tibble
as returned from \code{list_census_vectors}.}
}
\description{
List all parent variables from vector hierarchies given either a list of Census
variables returned by
\code{list_census_vectors}, \code{search_census_vectors}, \code{find_census_vectors}, or a direct string reference to the vector code.
}
\examples{
# Query parent vectors directly using vector identifier
parent_census_vectors("v_CA16_2519")
\dontrun{
# Example using multiple vectors coerced into a list
parent_census_vectors(c("v_CA16_2519","v_CA16_2520","v_CA16_2521"))

# or, equivalently
selected_vectors <- c("v_CA16_2519","v_CA16_2520","v_CA16_2521")
parent_census_vectors(selected_vectors)

# Example using dplyr and piped arguments
library(dplyr, warn.conflicts = FALSE)

list_census_vectors("CA16") \%>\%
  filter(vector == "v_CA16_2519") \%>\%
  parent_census_vectors()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{retry_api_call}
\alias{retry_api_call}
\title{Retry an API call with exponential backoff}
\usage{
retry_api_call(call_fn, max_retries = 3, quiet = FALSE)
}
\arguments{
\item{call_fn}{A function that performs the API call and returns an httr response}

\item{max_retries}{Maximum number of retry attempts (default: 3)}

\item{quiet}{If TRUE, suppress retry messages}
}
\value{
The httr response object
}
\description{
Retry an API call with exponential backoff
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{format_bytes}
\alias{format_bytes}
\title{Format bytes to human-readable size}
\usage{
format_bytes(bytes)
}
\description{
Format bytes to human-readable size
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recalled_data.R
\name{get_recalled_database}
\alias{get_recalled_database}
\title{Get metadata for recalled data}
\usage{
get_recalled_database(refresh = FALSE, warn_only_once = FALSE)
}
\arguments{
\item{refresh}{Will force refresh of recalled database}

\item{warn_only_once}{Will only warn on first run, `FALSE` by default}
}
\value{
tibble with rows describing recalled data
}
\description{
Grabs recall data from server if needed and returns list of recalled data
}
\examples{
\dontrun{
get_recalled_database()
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vector_viz.R
\name{visualize_vector_hierarchy}
\alias{visualize_vector_hierarchy}
\title{Visualize Census vector hierarchies as ASCII tree}
\usage{
visualize_vector_hierarchy(
  vector,
  dataset = NULL,
  max_depth = NA,
  show_type = FALSE,
  quiet = FALSE
)
}
\arguments{
\item{vector}{A Census vector code (e.g., "v_CA16_2510") or a filtered tibble
as returned from \code{list_census_vectors}.}

\item{dataset}{The dataset to query for vector information, e.g. \code{"CA16"}.
Only required if \code{vector} is a character string.}

\item{max_depth}{Maximum depth of the tree to display. Default is \code{NA}
which shows the entire hierarchy.}

\item{show_type}{Logical. If \code{TRUE}, shows the type (Total/Male/Female)
next to each vector. Default is \code{FALSE}.}

\item{quiet}{When \code{TRUE}, suppress messages. Default is \code{FALSE}.}
}
\value{
Invisibly returns a tibble of the vectors displayed in the tree.
  The tree is printed to the console as a side effect.
}
\description{
Displays an ASCII tree representation of the hierarchical structure
of Census vectors. This helps users understand the relationship between parent
and child vectors when selecting variables for their analysis.
}
\examples{
\dontrun{
# Visualize the age hierarchy for 2016 Census
visualize_vector_hierarchy("v_CA16_2510", dataset = "CA16")

# Show only first two levels with type information
visualize_vector_hierarchy("v_CA16_2510", dataset = "CA16",
                           max_depth = 2, show_type = TRUE)

# Using a vector tibble from list_census_vectors
library(dplyr)
list_census_vectors("CA16") \%>\%
  filter(vector == "v_CA16_2510") \%>\%
  visualize_vector_hierarchy()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geographies.R
\name{get_statcan_geographies}
\alias{get_statcan_geographies}
\title{Read the geosuite data}
\usage{
get_statcan_geographies(
  census_year,
  level,
  type = "cartographic",
  cache_path = NULL,
  timeout = 1000,
  refresh = FALSE,
  quiet = FALSE
)
}
\arguments{
\item{census_year}{census year to get the data for, right now only 2021 is supported}

\item{level}{geographic level to return the data for, valid choices are
"PR","CD","CMACA","CSD","CT","ADA","DA","DB,"ER","FED","DPL","POPCNTR", "FSA", "HR"}

\item{type}{type of geographic data, valid choices area "cartographic" or "digital"}

\item{cache_path}{optional path to cache the data. If the cancensus cache path is set the geographic data gets
cached in the "geographies" subdirectory of the cancensus cache path.}

\item{timeout}{optional timeout parameter, adjust as needed if the data download times out when using slow connections}

\item{refresh}{(logical) refresh the cache if true}

\item{quiet}{(logical) suppress messages if `TRUE`}
}
\value{
a spatial dataframe with the geographic data
}
\description{
Reads the original unprocessed geographic boundary files from Statistics Canada
}
\examples{
# get the digital geographic boundaries for provinces and territories
\dontrun{
get_statcan_geographies(census_year="2021",level="PR",type="digital")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intersect_geometry.R
\name{get_intersecting_geometries}
\alias{get_intersecting_geometries}
\title{Get identifiers for census regions intersecting a geometry}
\source{
Census data and boundary geographies are reproduced and distributed on
an "as is" basis with the permission of Statistics Canada (Statistics Canada
1996; 2001; 2006; 2011; 2016).
}
\usage{
get_intersecting_geometries(
  dataset,
  level,
  geometry,
  simplified = FALSE,
  use_cache = TRUE,
  quiet = FALSE,
  api_key = Sys.getenv("CM_API_KEY")
)
}
\arguments{
\item{dataset}{A CensusMapper dataset identifier.}

\item{level}{The census aggregation level to retrieve. One of \code{"Regions"}, \code{"PR"}, \code{"CMA"}, \code{"CD"}, \code{"CSD"}, \code{"CT"}, \code{"DA"}, \code{"EA"} (for 1996 census), or \code{"DB"} (for 2001-2016 censuses)..}

\item{geometry}{A valid \code{sf} or \code{sfc} class object. As long as the geometry is valid, any projection is accepted. Objects will be reprojected as server-side intersections use lat/lon projection.}

\item{simplified}{If \code{TRUE} will return a region list compatible with \link{get_census}, otherwise will return a character vector of matching region ids.}

\item{use_cache}{If set to \code{TRUE} (the default) data will be read from the local cache if available.}

\item{quiet}{When TRUE, suppress messages and warnings.}

\item{api_key}{An API key for the CensusMapper API. Defaults to \code{options()} and then the \code{CM_API_KEY} environment variable.}
}
\description{
This function returns a list of regions that intersect a given geometry input. This list of regions can be used
directly to query census when one is interested in census data for a particular geographic region that does
not coincide with defined census geometries. The returned value can be used as the \code{regions}
parameter in \link{get_census} to get corresponding census geographies and variables that cover the give geometry.
Input spatial objects can be any \code{sf} or \code{sfc} class objects such as \code{POINT}, \code{MULTIPOINT} or \code{POLYGON}.

This function comes with CensusMapper API limits
}
\examples{

\dontrun{
# Example using a POINT-class object from a pair of lat/lon coordinates
point_geo <- sf::st_sfc(sf::st_point(c(-123.25149, 49.27026)), crs=4326)
regions <- get_intersecting_geometries(dataset = 'CA16', level = 'CT', geometry = point_geo)
census_data <- get_census(dataset='CA16', regions=regions,
                          vectors=c("v_CA16_408","v_CA16_409","v_CA16_410"),
                          level='CT')

# Example using a POLYGON-class object from a bounding box with four coordinates
poly_geo <- sf::st_as_sfc(sf::st_bbox(c(ymin = 49.25, ymax = 49.30,
                          xmin = -123.25, xmax = -123.30)), crs = 4326)
regions <- get_intersecting_geometries(dataset = 'CA16', level = 'CT', geometry = poly_geo)
census_data <- get_census(dataset='CA16', regions=regions,
                         vectors=c("v_CA16_408","v_CA16_409","v_CA16_410"), level='CT')

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cancensus.R
\name{get_census_geometry}
\alias{get_census_geometry}
\title{Deprecated, use `get_census` instead}
\source{
Census data and boundary geographies are reproduced and distributed on
an "as is" basis with the permission of Statistics Canada (Statistics Canada
2006; 2011; 2016).
}
\usage{
get_census_geometry(dataset, regions, level = NA, geo_format = "sf", ...)
}
\arguments{
\item{dataset}{A CensusMapper dataset identifier.}

\item{regions}{A named list of census regions to retrieve. Names must be valid census aggregation levels.}

\item{level}{The census aggregation level to retrieve, defaults to \code{"Regions"}. One of \code{"Regions"}, \code{"PR"}, \code{"CMA"}, \code{"CD"}, \code{"CSD"}, \code{"CT"} or \code{"DA"}.}

\item{geo_format}{By default is set to \code{NA} and appends no geographic information. To include geographic information with census data, specify one of either \code{"sf"} to return an \code{\link[sf]{sf}} object (requires the \code{sf} package) or \code{"sp"} to return a \code{\link[sp]{SpatialPolygonsDataFrame-class}} object (requires the \code{rgdal} package).}

\item{...}{Further arguments passed to \code{get_census}.}
}
\description{
This function will be removed in future versions.
}
\examples{
# Query the API for data geographies in Vancouver, at the census subdivision
# level:
\dontrun{
# Query the API for geographies in Vancouver, at the census subdivision
# level, and return the associated geography files in \code{sf} format:
census_data <- get_census(dataset='CA16', regions=list(CMA="59933"),
                          level='CSD', geo_format = "sf")
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/user_settings.R
\name{set_cancensus_api_key}
\alias{set_cancensus_api_key}
\title{Set Censusmapper API key}
\usage{
set_cancensus_api_key(key, overwrite = FALSE, install = FALSE)
}
\arguments{
\item{key}{a Censusmapper API key. For more information on keys see the \href{https://mountainmath.github.io/cancensus/index.html#api-key}{API key section}}

\item{overwrite}{Option to overwrite any existing Censusmapper keys already stored locally.}

\item{install}{Option to install permanently for use across sessions.}
}
\description{
Cancensus requires a free Censusmapper API key to retrieve data. This function helps set the key for either the duration of the session (default) or permanently for use across sessions.
}
\examples{
\dontrun{
set_cancensus_api_key("YOUR_CM_API_KEY")

# This will set the key permanently until overwritten again
set_cancensus_api_key("YOUR_CM_API_KEY", install = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vector_discovery.R
\name{search_census_vectors}
\alias{search_census_vectors}
\title{Query the CensusMapper API for vectors with descriptions matching a search term or phrase (deprecated)}
\usage{
search_census_vectors(searchterm, dataset, type = NA, ...)
}
\arguments{
\item{searchterm}{The term or phrase to search for e.g. \code{"Ojibway"}.
Search terms are case insensitive. If unable to find a given string,
this function will suggest similarly named objects.}

\item{dataset}{The dataset to query for available vectors, e.g.
\code{"CA16"}.}

\item{type}{One of \code{NA}, \code{'Total'}, \code{'Male'} or \code{'Female'}.
If specified, only return variables of specified `type`.}

\item{...}{Further arguments passed on to \code{\link{list_census_vectors}}.}
}
\description{
Query the CensusMapper API for vectors with descriptions matching a search term or phrase (deprecated)
}
\examples{
\dontrun{
search_census_vectors('Ojibway', 'CA16')

# This will return a warning that no match was found, but will suggest similar terms.
search_census_vectors('Ojibwe', 'CA16', 'Total')
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vector_discovery.R
\name{explore_census_regions}
\alias{explore_census_regions}
\title{Interactively browse Census variables and regions on Censusmapper.ca in a new browser window}
\usage{
explore_census_regions(dataset = "CA16")
}
\arguments{
\item{dataset}{The dataset to query for available vectors, e.g. \code{'CA16'}.
Interactive tools available for the CA16, CA11, CA06, and CA01 Census datasets and
geographies.}
}
\description{
Finding the right Census variables or regions can be complicated.
\code{explore_census_vectors(dataset)} and \code{explore_census_regions(dataset)} will open a
new browser page or tab to an interactive Census variable and region exploration and selection
tool on the \href{https://censusmapper.ca/api}{Censusmapper.ca website}. Interactive
tools available for the CA16, CA11, CA06, and CA01 Census datasets and geographies.
}
\examples{
\dontrun{

explore_census_vectors(dataset = "CA16")

explore_census_regions(dataset = "CA11")

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recalled_data.R
\name{list_recalled_cached_data}
\alias{list_recalled_cached_data}
\title{List recalled data stored in local cache}
\usage{
list_recalled_cached_data(
  cached_data = list_cancensus_cache(),
  warn_only_once = FALSE
)
}
\arguments{
\item{cached_data}{List of locally cached data to check for recall, default is `list_cancensus_cache()` in which case
it will get checked against all locally cached data}

\item{warn_only_once}{Will only warn on first run during each session, `FALSE` by default}
}
\value{
tibble with rows describing locally cached recalled data
}
\description{
Checks the local cached database for recalled data and lists all recalled cached entries
}
\examples{
\dontrun{
list_recalled_cached_data()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recalled_data.R
\name{remove_recalled_cached_data}
\alias{remove_recalled_cached_data}
\title{Remove recalled data from local cache}
\usage{
remove_recalled_cached_data()
}
\value{
Storage size of removed locally cached data that got freed up in number of bytes.
}
\description{
Checks the local cached database for recalled data and removes cached data that has been recalled
}
\examples{
\dontrun{
remove_recalled_cached_data()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cancensus.R
\name{list_census_datasets}
\alias{list_census_datasets}
\title{Query the CensusMapper API for available datasets.}
\usage{
list_census_datasets(use_cache = TRUE, quiet = FALSE)
}
\arguments{
\item{use_cache}{If set to TRUE (the default), data will be read from a temporary local cache for the
duration of the R session, if
available. If set to FALSE, query the API for the data, and
refresh the temporary cache with the result.}

\item{quiet}{When TRUE, suppress messages and warnings.}
}
\value{
Returns a data frame with a column \code{dataset} containing the code for the
dataset, a column \code{description} describing it, a \code{geo_dataset} column
identifying the geography dataset the data is based on, a \code{attribution} column
with an attribution string, a \code{reference} column with a reference identifier, and
a \code{reference_url} column with a link to reference materials.
}
\description{
Query the CensusMapper API for available datasets.
}
\examples{

# List available datasets in CensusMapper
\dontrun{
list_census_datasets()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\docType{data}
\name{CODES_TABLE}
\alias{CODES_TABLE}
\title{A dataset with code table summaries for census data}
\description{
A dataset with code table summaries for census data
}
\references{
\url{https://www12.statcan.gc.ca/census-recensement/2021/geo/ref/domain-domaine/index2021-eng.cfm?lang=e&id=CSDtype}, \url{https://www12.statcan.gc.ca/census-recensement/2021/geo/ref/domain-domaine/index2021-eng.cfm?lang=e&id=CDtype}
}
\author{
derived from StatCan definitions
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/user_settings.R
\name{set_cancensus_cache_path}
\alias{set_cancensus_cache_path}
\title{Set persistent cancensus cache location}
\usage{
set_cancensus_cache_path(cache_path, overwrite = FALSE, install = FALSE)
}
\arguments{
\item{cache_path}{a local directory to use for saving cached data}

\item{overwrite}{Option to overwrite any existing cache path already stored locally.}

\item{install}{Option to install permanently for use across sessions.}
}
\description{
Cancensus provides session caching for retrieved data to increase speeds and reduce API usage. This function will create a persistent cache across sessions.
}
\examples{
\dontrun{
set_cancensus_cache_path("~/cancensus_cache")

# This will set the cache path permanently until overwritten again
set_cancensus_cache_path("~/cancensus_cache", install = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cancensus.R
\name{get_census}
\alias{get_census}
\title{Access to Canadian census data through the CensusMapper API}
\source{
Census data and boundary geographies are reproduced and distributed on
an "as is" basis with the permission of Statistics Canada (Statistics Canada 1996; 2001; 2006; 2011; 2016).
}
\usage{
get_census(
  dataset,
  regions,
  level = NA,
  vectors = c(),
  geo_format = NA,
  resolution = "simplified",
  labels = "detailed",
  use_cache = TRUE,
  quiet = FALSE,
  api_key = Sys.getenv("CM_API_KEY"),
  retry = 0
)
}
\arguments{
\item{dataset}{A CensusMapper dataset identifier.}

\item{regions}{A named list of census regions to retrieve. Names must be valid census aggregation levels.}

\item{level}{The census aggregation level to retrieve, defaults to \code{"Regions"}. One of \code{"Regions"}, \code{"PR"}, \code{"CMA"}, \code{"CD"}, \code{"CSD"}, \code{"CT"}, \code{"DA"}, \code{"EA"} (for 1996), or \code{"DB"} (for 2001-2016).}

\item{vectors}{An R vector containing the CensusMapper variable names of the census variables to download. If no vectors are specified only geographic data will get downloaded.}

\item{geo_format}{By default is set to \code{NA} and appends no geographic information. To include geographic information with census data, specify one of either \code{"sf"} to return an \code{\link[sf]{sf}} object (requires the \code{sf} package) or \code{"sp"} to return a \code{\link[sp]{SpatialPolygonsDataFrame-class}} object (requires the \code{rgdal} package). If user requests geo-spatial data and neither package is available, a context menu will prompt to install the \code{sf} package.}

\item{resolution}{Resolution of the geographic data. By default simplified geometries will be download. For lower level geometries like DB or DA this will be very close to the high resolution data.
Simplification generally increases as the geographic aggregation level increases.
If high resolution geometries are required
then this option can be set to 'high'. By default this setting is set to \code{'simplified'}.}

\item{labels}{Set to "detailed" by default, but truncated Census variable names can be selected by setting labels = "short". Use \code{label_vectors(...)} to return variable label information in detail.}

\item{use_cache}{If set to TRUE (the default) data will be read from the local cache if available.}

\item{quiet}{When TRUE, suppress messages and warnings.}

\item{api_key}{An API key for the CensusMapper API. Defaults to \code{options()} and then the \code{CM_API_KEY} environment variable.}

\item{retry}{Integer If greater than zero, automatically retry failed API requests with exponential backoff for specified maximum number of times. Defaults to 0.}
}
\description{
This function allows convenient access to Canadian census data and boundary
files through the CensusMapper API. An API key is required to retrieve data.
}
\details{
For help selecting regions and vectors, see \code{\link{list_census_regions}}
and \code{\link{list_census_vectors}}, or check out the interactive selection
tool at \url{https://censusmapper.ca/api} by calling \code{explore_census_vectors()}
}
\examples{
# Query the API for data on dwellings in Vancouver, at the census subdivision
# level:
\dontrun{
census_data <- get_census(dataset='CA16', regions=list(CMA="59933"),
                          vectors=c("v_CA16_408","v_CA16_409","v_CA16_410"),
                          level='CSD')

# Query the API for data on dwellings in Vancouver, at the census subdivision
# level, and return the associated geography files in \code{sf} format:
census_data <- get_census(dataset='CA16', regions=list(CMA="59933"),
                          vectors=c("v_CA16_408","v_CA16_409","v_CA16_410"),
                          level='CSD', geo_format = "sf")

# Make the same query, but this time drop descriptive vector names:
census_data <- get_census(dataset='CA16', regions=list(CMA="59933"),
                          vectors=c("v_CA16_408","v_CA16_409","v_CA16_410"),
                          level='CSD', geo_format = "sf", labels="short")

# Get details for truncated vectors:
label_vectors(census_data)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cancensus_cache.R
\name{remove_from_cancensus_cache}
\alias{remove_from_cancensus_cache}
\title{Remove cached files}
\usage{
remove_from_cancensus_cache(paths)
}
\arguments{
\item{paths}{list of paths to remove, as returned by the path column in `list_cancensus_cache`}
}
\value{
freed-up disk space
}
\description{
Remove cached data for paths given
}
\examples{
\dontrun{
# remove the first cached dataset
cache_data <- list_cancensus_cache()

remove_from_cancensus_cache(cache_data$path[1])
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/census_vectors.R
\name{list_census_vectors}
\alias{list_census_vectors}
\title{Query the CensusMapper API for available vectors for a given dataset.}
\usage{
list_census_vectors(dataset, use_cache = TRUE, quiet = TRUE)
}
\arguments{
\item{dataset}{The dataset to query for available vectors, e.g.
\code{"CA16"}.}

\item{use_cache}{If set to TRUE (the default), data will be read from a local cache
that is maintained for the duration of the R session, if
available. If set to FALSE, query the API for the data, and
refresh the local cache with the result.}

\item{quiet}{When FALSE, shows messages and warnings. Set to TRUE by default.}
}
\value{
Returns a data frame detailing the available Census vectors (i.e. variables) for a given Census
dataset. This data frame has columns \code{vector} containing the short code for the
variable, \code{type} describing whether it's a female, male, or total aggregate, \code{label}
indicating the name of the variable, \code{units} indicating whether the value represents a
numeric integer, percentage, dollar figure, or ratio, \code{parent_vector} to show hierarchical
relationship, \code{aggregation} indicating whether the value is additive or a transformation,
and a column \code{details} with a detailed description of the variable generated by traversing
all labels within its hierarchical structure.
}
\description{
Query the CensusMapper API for available vectors for a given dataset.
}
\examples{

\dontrun{
# List all vectors for a given Census dataset in CensusMapper
list_census_vectors('CA16')
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cancensus.R
\name{dataset_attribution}
\alias{dataset_attribution}
\title{Get attribution for datasets}
\usage{
dataset_attribution(datasets)
}
\arguments{
\item{datasets}{Vector of dataset identifiers}
}
\value{
Returns a string to be used as attribution for the given datasets.
}
\description{
Get attribution for datasets
}
\examples{

\dontrun{
# Attribution string for the 2006 and 2016 census datasets
dataset_attribution(c('CA06','CA16'))
}
}
